 /**
 * \file	GLAboutWidget.cpp
 * \author	Krzysztof Kaczor kinio4444@gmail.com
 * \date 	12.01.2008 
 * \version	1.0
 * \brief	This file contains class definition that is an object drawn in OpenGL.
 * \note This file is a part of HQEd. It has been develped within the HeKatE Project.
 * HQEd is a free software that you can redistribute and/or modify
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License or 
 * (at your option) any later version.
 * You should have received a copy of the GNU General Public License with HQEd.
 * \note HQEd is distributed in hope that will be useful.
 * It has NO ANY WARRANTY, neither implied warranty of
 * MERCHANTABILITY nor FITNESS FOR A PARTICULAR PURPOSE. 
 * \note Copyright (C) 2006-9 by the HeKatE Project.
 * \see GNU General Public License for more details http://www.gnu.org/licenses/
 * \see HeKatE project http://hekate.ia.agh.edu.pl
 */

#include <QtGui>
#include <QtOpenGL>

#include <math.h>

#include "GLAboutWidget.h"
// -----------------------------------------------------------------------------

#define WALL_SIZE 10
// -----------------------------------------------------------------------------

// #brief Constructor GLAboutWidget class.
//
// #param parent Pointer to parent object.
GLAboutWidget::GLAboutWidget(QWidget *parent)
    : QGLWidget(parent)
{
	QTimer* timer = new QTimer(this);
     connect(timer, SIGNAL(timeout()), this, SLOT(renderScene()));
     timer->start(20);
}
// -----------------------------------------------------------------------------

// #brief Destructor GLAboutWidget class.
GLAboutWidget::~GLAboutWidget(void)
{
}
// -----------------------------------------------------------------------------

// #brief Inits OpenGl including creating context of drawing and preparing scene parameters.
//
// #return No return value.
void GLAboutWidget::initializeGL(void)
{
	// Ustalanie wartosci zmiennych
	bkgAplpha = 0.0;
	d_bkgAlpha = 1.0;
	
	gl_x_min = -10.0;
	gl_x_max = +10.0;
	gl_y_min = -10.0;
	gl_y_max = +10.0;
	gl_z_min = 0.0;
	gl_z_max = +10.0;
	
	d_xrot1 = 3.0;
	d_yrot1 = 1.0;
	d_zrot1 = 2.0;
	d_xpos1 = 0.1;
	d_ypos1 = 0.2;
	
	d_xrot2 = 1.5;
	d_yrot2 = 1.9;
	d_zrot2 = 1.0;
	d_xpos2 = 0.3;
	d_ypos2 = 0.2;
	
	xpos1 = ypos1 = xrot1 = yrot1 = zrot1 = 0.0;
	xpos2 = ypos2 = xrot2 = yrot2 = zrot2 = 0.0;
	
	tmpVar = 0.0;
	

	wall = bindTexture(QPixmap(":/all_images/images/hekate.bmp"), GL_TEXTURE_2D);
	bkgwall = bindTexture(QPixmap(":/all_images/images/hekate.bmp"), GL_TEXTURE_2D);
	flag = bindTexture(QPixmap(":/all_images/images/aghLogoFull.bmp"), GL_TEXTURE_2D);
	eagle = bindTexture(QPixmap(":/all_images/images/aghLogo.bmp"), GL_TEXTURE_2D);
	eaglebw = bindTexture(QPixmap(":/all_images/images/aghLogoBW.bmp"), GL_TEXTURE_2D);
	xttt = bindTexture(QPixmap(":/all_images/images/xtt_apperance.png"), GL_TEXTURE_2D);
	ardt = bindTexture(QPixmap(":/all_images/images/ard_apperance.png"), GL_TEXTURE_2D);
	
	// Budowanie list
	buildLists();
	
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
	makeCurrent();
	glClearColor(1.0, 1.0, 1.0, 0.0);
}
// -----------------------------------------------------------------------------

// #brief Active when window is resized.
//
// #param width New width.
// #param height New height.
// #return No return value.
void GLAboutWidget::resizeGL(int width, int height)
{
	int side = qMin(width, height);
     glViewport((width - side) / 2, (height - side) / 2, side, side);

     glMatrixMode(GL_PROJECTION);
     glLoadIdentity();
	glFrustum(gl_x_min, gl_x_max, gl_y_min, gl_y_max, gl_z_min, gl_z_max);
	gluPerspective(45.0f, 1.0, 0.1f, 700.0f);
	
     glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	
	// Wlaczenie opcji OpenGL
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_TEXTURE_2D);
}
// -----------------------------------------------------------------------------

// #brief Draws scene in OpenGl.
//
// #return No return value.
void GLAboutWidget::paintGL(void)
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
	glLoadIdentity();
	glTranslatef(0.0, 0.0, -24.3);
	glPushMatrix();
	
	qglColor(QColor(255, 255, 255));
	glShadeModel(GL_SMOOTH);
	glEnable(GL_BLEND);
	glDisable(GL_DEPTH_TEST);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	
	// Rysowanie tla
	float ws = WALL_SIZE;
	QColor wallColor(255, 255, 255, qMin((int)bkgAplpha, 255));
	glBindTexture(GL_TEXTURE_2D, bkgwall);
	glBegin(GL_QUADS);
		qglColor(wallColor);	glTexCoord2f(0.0f, 0.0f);	glVertex3f(-ws, -ws, 0.0);
		qglColor(wallColor);	glTexCoord2f(0.0f, 1.0f);	glVertex3f(-ws, ws, 0.0);
		qglColor(wallColor);	glTexCoord2f(1.0f, 1.0f);	glVertex3f(ws, ws, 0.0);
		qglColor(wallColor);	glTexCoord2f(1.0f, 0.0f);	glVertex3f(ws, -ws, 0.0);
	glEnd();
	
	glBindTexture(GL_TEXTURE_2D, flag);
	wallColor.setAlpha(255 - qMin((int)bkgAplpha, 255));
	glBegin(GL_QUADS);
		qglColor(wallColor);	glTexCoord2f(0.0f, 0.0f);	glVertex3f(-ws, -ws, 0.0);
		qglColor(wallColor);	glTexCoord2f(0.0f, 1.0f);	glVertex3f(-ws, ws, 0.0);
		qglColor(wallColor);	glTexCoord2f(1.0f, 1.0f);	glVertex3f(ws, ws, 0.0);
		qglColor(wallColor);	glTexCoord2f(1.0f, 0.0f);	glVertex3f(ws, -ws, 0.0);
	glEnd();
	
	// Rysowanie orzelka AGH
	wallColor.setAlpha(255);
	ws = 1.5;
	float x = 10.0-ws;
	float y = 10.0-ws;
	
	glBlendFunc(GL_DST_COLOR, GL_ZERO); 
	glBindTexture(GL_TEXTURE_2D, eaglebw);	// Maska
	glBegin(GL_QUADS);
		qglColor(wallColor);	glTexCoord2f(0.0f, 0.0f);	glVertex3f(x-ws, y-ws, 0.0);
		qglColor(wallColor);	glTexCoord2f(0.0f, 1.0f);	glVertex3f(x-ws, y+ws, 0.0);
		qglColor(wallColor);	glTexCoord2f(1.0f, 1.0f);	glVertex3f(x+ws, y+ws, 0.0);
		qglColor(wallColor);	glTexCoord2f(1.0f, 0.0f);	glVertex3f(x+ws, y-ws, 0.0);
	glEnd();
	
	glBlendFunc(GL_ONE, GL_ONE);
	glBindTexture(GL_TEXTURE_2D, eagle);	// Wlasciwy obraz
	glBegin(GL_QUADS);
		qglColor(wallColor);	glTexCoord2f(0.0f, 0.0f);	glVertex3f(x-ws, y-ws, 0.0);
		qglColor(wallColor);	glTexCoord2f(0.0f, 1.0f);	glVertex3f(x-ws, y+ws, 0.0);
		qglColor(wallColor);	glTexCoord2f(1.0f, 1.0f);	glVertex3f(x+ws, y+ws, 0.0);
		qglColor(wallColor);	glTexCoord2f(1.0f, 0.0f);	glVertex3f(x+ws, y-ws, 0.0);
	glEnd();

	// Rysowanie szescianow
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	
	glScalef(0.4, 0.4, 0.4);
	glTranslatef(xpos1, ypos1, 0.0);
	glRotatef(xrot1, 1.0, 0.0, 0.0);
	glRotatef(yrot1, 0.0, 1.0, 0.0);
	glRotatef(zrot1, 0.0, 0.0, 1.0);
	glCallList(cube1);
	
	glPopMatrix();
	glScalef(0.4, 0.4, 0.4);
	glTranslatef(xpos2, ypos2, 0.0);
	glRotatef(xrot2, 1.0, 0.0, 0.0);
	glRotatef(yrot2, 0.0, 1.0, 0.0);
	glRotatef(zrot2, 0.0, 0.0, 1.0);
	glCallList(cube2);
	
	glDisable(GL_BLEND);
	glEnable(GL_DEPTH_TEST);
}
// -----------------------------------------------------------------------------

// #brief Updates scene in OpenGl.
//
// #return No return value.
void GLAboutWidget::renderScene(void)
{
	if(((xrot1 + d_xrot1) > 360.) || ((xrot1 + d_xrot1) < 0.))
		d_xrot1 = -d_xrot1;
	xrot1 += d_xrot1;
	if(((yrot1 + d_yrot1) > 360.) || ((yrot1 + d_yrot1) < 0.))
		d_yrot1 = -d_yrot1;
	yrot1 += d_yrot1;
	if(((zrot1 + d_zrot1) > 360.) || ((zrot1 + d_zrot1) < 0.))
		d_zrot1 = -d_zrot1;
	zrot1 += d_zrot1;
	
	if(qAbs(xpos1 + d_xpos1) > 20.)
		d_xpos1 = -d_xpos1;
	xpos1 += d_xpos1;
	if(qAbs(ypos1 + d_ypos1) > 20.)
		d_ypos1 = -d_ypos1;
	ypos1 += d_ypos1;
	
	if(((xrot2 + d_xrot2) > 360.) || ((xrot2 + d_xrot2) < 0.))
		d_xrot2 = -d_xrot2;
	xrot2 += d_xrot2;
	if(((yrot2 + d_yrot2) > 360.) || ((yrot2 + d_yrot2) < 0.))
		d_yrot2 = -d_yrot2;
	yrot2 += d_yrot2;
	if(((zrot2 + d_zrot2) > 360.) || ((zrot2 + d_zrot2) < 0.))
		d_zrot2 = -d_zrot2;
	zrot2 += d_zrot2;
	
	if(qAbs(xpos2 + d_xpos2) > 20.)
		d_xpos2 = -d_xpos2;
	xpos2 += d_xpos2;
	if(qAbs(ypos2 + d_ypos2) > 20.)
		d_ypos2 = -d_ypos2;
	ypos2 += d_ypos2;
	
	if(((bkgAplpha + d_bkgAlpha) > 320.) || ((bkgAplpha + d_bkgAlpha) < 0.))
		d_bkgAlpha = -d_bkgAlpha;
	bkgAplpha += d_bkgAlpha;
	
	updateGL();
}
// -----------------------------------------------------------------------------

// #brief Event pressing key down.
//
// #param event Pointer to key event object.
// #return No return value.
void GLAboutWidget::keyPressEvent(QKeyEvent* event)
{
	float drot1 = 1.;
	tmpVar = bkgAplpha;

	if(event->key() == Qt::Key_Z)
		tmpVar -= drot1;
	if(event->key() == Qt::Key_X)
		tmpVar += drot1;
	if(event->key() == Qt::Key_M)
		QMessageBox ::question(NULL, "HQEd", "tmpVar = " + QString::number(tmpVar));
}
// -----------------------------------------------------------------------------

// #brief Creates lists of OpenGl commands.
//
// #return No return value.
void GLAboutWidget::buildLists(void)
{
	// Szeroksoc sciany
	float ws = WALL_SIZE / 2.;
	QColor wallColor(255, 255, 255, 125);
	
	// Tworzenie cube
	cube1 = glGenLists(1);
	glNewList(cube1, GL_COMPILE);
	
		glBindTexture(GL_TEXTURE_2D, wall);
		
		glBegin(GL_QUADS);
			// Przednia sciana
			qglColor(wallColor);	glTexCoord2f(0.0f, 0.0f);	glVertex3f(-ws, -ws, ws);
			qglColor(wallColor);	glTexCoord2f(0.0f, 1.0f);	glVertex3f(-ws, ws, ws);
			qglColor(wallColor);	glTexCoord2f(1.0f, 1.0f);	glVertex3f(ws, ws, ws);
			qglColor(wallColor);	glTexCoord2f(1.0f, 0.0f);	glVertex3f(ws, -ws, ws);
			
			// Tylnia sciana
			qglColor(wallColor);	glTexCoord2f(0.0f, 0.0f);	glVertex3f(ws, -ws, -ws);
			qglColor(wallColor);	glTexCoord2f(0.0f, 1.0f);	glVertex3f(ws, ws, -ws);
			qglColor(wallColor);	glTexCoord2f(1.0f, 1.0f);	glVertex3f(-ws, ws, -ws);
			qglColor(wallColor);	glTexCoord2f(1.0f, 0.0f);	glVertex3f(-ws, -ws, -ws);
			
			// Gorna sciana
			qglColor(wallColor);	glTexCoord2f(0.0f, 0.0f);	glVertex3f(-ws, ws, ws);
			qglColor(wallColor);	glTexCoord2f(0.0f, 1.0f);	glVertex3f(-ws, ws, -ws);
			qglColor(wallColor);	glTexCoord2f(1.0f, 1.0f);	glVertex3f(ws, ws, -ws);
			qglColor(wallColor);	glTexCoord2f(1.0f, 0.0f);	glVertex3f(ws, ws, ws);
			
			// Dolna sciana
			qglColor(wallColor);	glTexCoord2f(0.0f, 0.0f);	glVertex3f(-ws, -ws, -ws);
			qglColor(wallColor);	glTexCoord2f(0.0f, 1.0f);	glVertex3f(-ws, -ws, ws);
			qglColor(wallColor);	glTexCoord2f(1.0f, 1.0f);	glVertex3f(ws, -ws, ws);
			qglColor(wallColor);	glTexCoord2f(1.0f, 0.0f);	glVertex3f(ws, -ws, -ws);
			
			// Lewa sciana
			qglColor(wallColor);	glTexCoord2f(0.0f, 0.0f);	glVertex3f(-ws, -ws, -ws);
			qglColor(wallColor);	glTexCoord2f(0.0f, 1.0f);	glVertex3f(-ws, -ws, ws);
			qglColor(wallColor);	glTexCoord2f(1.0f, 1.0f);	glVertex3f(-ws, ws, ws);
			qglColor(wallColor);	glTexCoord2f(1.0f, 0.0f);	glVertex3f(-ws, ws, -ws);
			
			// Prawa sciana
			qglColor(wallColor);	glTexCoord2f(0.0f, 0.0f);	glVertex3f(ws, -ws, -ws);
			qglColor(wallColor);	glTexCoord2f(0.0f, 1.0f);	glVertex3f(ws, -ws, ws);
			qglColor(wallColor);	glTexCoord2f(1.0f, 1.0f);	glVertex3f(ws, ws, ws);
			qglColor(wallColor);	glTexCoord2f(1.0f, 0.0f);	glVertex3f(ws, ws, -ws);
		glEnd();
		
	
     glEndList();     
	
	
	cube2 = glGenLists(1);
	wallColor.setAlpha(170);
	glNewList(cube2, GL_COMPILE);
		
		glBindTexture(GL_TEXTURE_2D, xttt);
		glBegin(GL_QUADS);
		
			// Przednia sciana
			qglColor(wallColor);	glTexCoord2f(0.0f, 0.0f);	glVertex3f(-ws, -ws, ws);
			qglColor(wallColor);	glTexCoord2f(0.0f, 1.0f);	glVertex3f(-ws, ws, ws);
			qglColor(wallColor);	glTexCoord2f(1.0f, 1.0f);	glVertex3f(ws, ws, ws);
			qglColor(wallColor);	glTexCoord2f(1.0f, 0.0f);	glVertex3f(ws, -ws, ws);
			
			// Tylnia sciana
			qglColor(wallColor);	glTexCoord2f(0.0f, 0.0f);	glVertex3f(ws, -ws, -ws);
			qglColor(wallColor);	glTexCoord2f(0.0f, 1.0f);	glVertex3f(ws, ws, -ws);
			qglColor(wallColor);	glTexCoord2f(1.0f, 1.0f);	glVertex3f(-ws, ws, -ws);
			qglColor(wallColor);	glTexCoord2f(1.0f, 0.0f);	glVertex3f(-ws, -ws, -ws);
			
			// Gorna sciana
			qglColor(wallColor);	glTexCoord2f(0.0f, 0.0f);	glVertex3f(-ws, ws, ws);
			qglColor(wallColor);	glTexCoord2f(0.0f, 1.0f);	glVertex3f(-ws, ws, -ws);
			qglColor(wallColor);	glTexCoord2f(1.0f, 1.0f);	glVertex3f(ws, ws, -ws);
			qglColor(wallColor);	glTexCoord2f(1.0f, 0.0f);	glVertex3f(ws, ws, ws);
		glEnd();
			
		glBindTexture(GL_TEXTURE_2D, ardt);
		glBegin(GL_QUADS);
			
			// Dolna sciana
			qglColor(wallColor);	glTexCoord2f(0.0f, 0.0f);	glVertex3f(-ws, -ws, -ws);
			qglColor(wallColor);	glTexCoord2f(0.0f, 1.0f);	glVertex3f(-ws, -ws, ws);
			qglColor(wallColor);	glTexCoord2f(1.0f, 1.0f);	glVertex3f(ws, -ws, ws);
			qglColor(wallColor);	glTexCoord2f(1.0f, 0.0f);	glVertex3f(ws, -ws, -ws);
			
			// Lewa sciana
			qglColor(wallColor);	glTexCoord2f(0.0f, 0.0f);	glVertex3f(-ws, -ws, -ws);
			qglColor(wallColor);	glTexCoord2f(0.0f, 1.0f);	glVertex3f(-ws, -ws, ws);
			qglColor(wallColor);	glTexCoord2f(1.0f, 1.0f);	glVertex3f(-ws, ws, ws);
			qglColor(wallColor);	glTexCoord2f(1.0f, 0.0f);	glVertex3f(-ws, ws, -ws);
			
			// Prawa sciana
			qglColor(wallColor);	glTexCoord2f(0.0f, 0.0f);	glVertex3f(ws, -ws, -ws);
			qglColor(wallColor);	glTexCoord2f(0.0f, 1.0f);	glVertex3f(ws, -ws, ws);
			qglColor(wallColor);	glTexCoord2f(1.0f, 1.0f);	glVertex3f(ws, ws, ws);
			qglColor(wallColor);	glTexCoord2f(1.0f, 0.0f);	glVertex3f(ws, ws, -ws);
		glEnd();
	
     glEndList();   
}
// -----------------------------------------------------------------------------
