 /**
 * \file	GLStartWidget.h
 * \author	Krzysztof Kaczor kinio4444@gmail.com
 * \date 	14.01.2008 
 * \version	1.0
 * \brief	This file contains class definition that is an object drawn in OpenGL.
 * \note This file is a part of HQEd. It has been develped within the HeKatE Project.
 * HQEd is a free software that you can redistribute and/or modify
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License or 
 * (at your option) any later version.
 * You should have received a copy of the GNU General Public License with HQEd.
 * \note HQEd is distributed in hope that will be useful.
 * It has NO ANY WARRANTY, neither implied warranty of
 * MERCHANTABILITY nor FITNESS FOR A PARTICULAR PURPOSE. 
 * \note Copyright (C) 2006-9 by the HeKatE Project.
 * \see GNU General Public License for more details http://www.gnu.org/licenses/
 * \see HeKatE project http://hekate.ia.agh.edu.pl
 */
#ifndef GLSTARTWIDGETH
#define GLSTARTWIDGETH
// -----------------------------------------------------------------------------

#include <QGLWidget>
#include <QtOpenGL>
// -----------------------------------------------------------------------------
/**
* \class 	GLStartWidget
* \author	Krzysztof Kaczor kinio4444@gmail.com
* \date 	14.01.2008
* \brief 	This class defines an object drawn in OpenGL.
*/
class GLStartWidget : public QGLWidget
{
    Q_OBJECT
    
    float bkgAplpha;	///< Transparency of background.
    float d_bkgAlpha;	///< Transparency of d_background.
    
    float xrot1, 	///< Rotation x1.
		yrot1, 		///< Rotation y1.
		zrot1;		///< Rotation z1.

    float xpos1, 	///< Position x1.
		ypos1;		///< Position y1.

    float d_xrot1, 	///< Rotation d_x1.
		d_yrot1,	///< Rotation d_y1.
		d_zrot1, 	///< Rotation d_z1.
		d_xpos1, 	///< Position d_x1.
		d_ypos1;	///< Position d_y1.
    
    float xrot2, 	///< Rotation x2.
		yrot2, 		///< Rotation y2.
		zrot2;		///< Rotation z2.

    float xpos2, 	///< Position x2.
		ypos2;		///< Position y2.

    float d_xrot2, 	///< Rotation d_x2.
		d_yrot2, 	///< Rotation d_y2.
		d_zrot2, 	///< Rotation d_z2.
		d_xpos2, 	///< Position d_x2.
		d_ypos2;	///< Position d_y2.
        
    float gl_x_min, ///< Min coordinate x cutting block.
		gl_x_max;	///< Max coordinate x cutting block.
    float gl_y_min, ///< Min coordinate y cutting block.
		gl_y_max;	///< Max coordinate y cutting block.
    float gl_z_min, ///< Min coordinate z cutting block.
		gl_z_max;	///< Max coordinate z cutting block.
    
    GLuint eagle;	///< Eagle texture.
    GLuint eaglebw;	///< Eaglebw texture.
    GLuint wall;	///< Wall texture.
    GLuint bkgwall;	///< Wall background texture.
    GLuint flag;	///< Flag texture.
    GLuint xttt;	///< Xttt texture.
    GLuint ardt;	///< Ardt texture.
    
    GLuint cube1;	///< Cube1 list.
    GLuint cube2;	///< Cube2 list.
    GLuint cube3;	///< Cube3 list.
    
    QRect fCopyArea;	///< Copy desktop area.
    
    QPixmap backgroundPixmap;	///< Desktop image.
    
    bool cubesJoined;	///< True when cubes are joined.
    
    bool runAppSend;	///< Determines weather background should be changed.
    
    float tmpVar;		///< Temporary variable.

public:

	/// \brief Constructor.
	///
	/// \param parent Pointer to parent object.
	GLStartWidget(QWidget *parent = 0);
	
	/// \brief Destructor.
	~GLStartWidget(void);
	
	/// \brief Function is triggered when key is pressed.
	///
	/// \param event Pointer to event object.
	/// \return No return value.
	void keyPressEvent(QKeyEvent* event);
	
	/// \brief Set copy desktop area.
	///
	/// \param rect Desktop area.
	/// \return No return value.
	void setCopyArea(QRect rect);

protected:

	/// \brief Init OpenGl including creating context of drawing and preparing scene parameters.
	///
	/// \return No return value.
	void initializeGL(void);
	
	/// \brief Active when window is resized.
	///
	/// \param width New width.
	/// \param height New height.
	/// \return No return value.
	void resizeGL(int width, int height);
	
	/// \brief Draw scene in OpenGl.
	///
	/// \return No return value.
	void paintGL(void);
	
	/// \brief Create lists of OpenGl commands.
	///
	/// \return No return value.
	void buildLists(void);
	
public slots:
	
	/// \brief Update scene in OpenGl.
	///
	/// \return No return value.
	void renderScene(void);
	
	/// \brief Close OpenGl functions.
	///
	/// \return No return value.
	void finish(void);
	
signals:

	/// \brief Signal is emitted when program should be started.
	///
	/// \return No return value.
	void runApp(void);
};
// -----------------------------------------------------------------------------

#endif
