 /**
 * \file     MainWin_ui.cpp
 * \author     Krzysztof Kaczor kinio4444@gmail.com
 * \date      24.04.2007
 * \version     1.0
 * \brief     This file contains class definition arranges main window of application. CONTROLLER.
 * \note This file is a part of HQEd. It has been develped within the HeKatE Project.
 * HQEd is a free software that you can redistribute and/or modify
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License or
 * (at your option) any later version.
 * You should have received a copy of the GNU General Public License with HQEd.
 * \note HQEd is distributed in hope that will be useful.
 * It has NO ANY WARRANTY, neither implied warranty of
 * MERCHANTABILITY nor FITNESS FOR A PARTICULAR PURPOSE.
 * \note Copyright (C) 2006-9 by the HeKatE Project.
 * \see GNU General Public License for more details http://www.gnu.org/licenses/
 * \see HeKatE project http://hekate.ia.agh.edu.pl
 */

#include <math.h>

#include <QList>
#include <QtGui>
#include <QDialog>
#include <QObject>
#include <QPainter>
#include <QPrinter>
#include <QMessageBox>

#include <QPixmap>

#include <Qt>
#include <QtGui>
#include <QDir>

#include <QDial>
#include <QSpinBox>

#include <QDomDocument>

#if QT_VERSION >= 0x040300
     #include <QXmlStreamWriter>
#endif

#include "ARD/ardGraphicsView.h"
#include "ARD/ardGraphicsScene.h"

#include "../M/ARD/ARD_Level.h"
#include "../M/ARD/ARD_Property.h"
#include "../M/ARD/ARD_LevelList.h"
#include "../M/ARD/ARD_Dependency.h"
#include "../M/ARD/ARD_PropertyList.h"
#include "../M/ARD/ARD_AttributeList.h"
#include "../M/ARD/ARD_DependencyList.h"

#include "../V/vasProlog.h"
#include "../V/vasPicture.h"
#include "../V/vasXML.h"

#include "../V/ARD/gTPHtree.h"
#include "../V/ARD/gARDlevel.h"
#include "../V/ARD/gARDlevelList.h"
#include "../V/XTT/GTable.h"

#include "XTT/MyGraphicsView.h"
#include "XTT/MyGraphicsScene.h"
#include "XTT/ErrorsListWidget.h"
#include "XTT/ExecuteMessagesListWidget.h"


#include "../M/XTT/XTT_Row.h"
#include "../M/XTT/XTT_Cell.h"
#include "../M/XTT/XTT_Table.h"
#include "../M/XTT/XTT_TableList.h"
#include "../M/XTT/XTT_AttributeGroups.h"
#include "../M/XTT/XTT_ConnectionsList.h"

#include "../M/hTypeList.h"
#include "../M/hMultipleValue.h"
#include "../M/hMultipleValueResult.h"

#include "XTT/ItemFinder_ui.h"
#include "XTT/CellEditor_ui.h"
#include "XTT/TableEditor_ui.h"
#include "XTT/DeleteTable_ui.h"
#include "XTT/TypeManager_ui.h"
#include "XTT/ColumnEditor_ui.h"
#include "XTT/GroupManager_ui.h"
#include "XTT/AttributeDelete_ui.h"
#include "XTT/AttributeEditor_ui.h"
#include "XTT/AttributeManager_ui.h"
#include "XTT/DeleteConnection_ui.h"
#include "XTT/UnusedAttributes_ui.h"
#include "XTT/ConnectionEditor_ui.h"
#include "XTT/ConnectionManager_ui.h"

#include "../namespaces/ns_ard.h"
#include "../namespaces/ns_xtt.h"
#include "../namespaces/ns_hqed.h"

#include "About_ui.h"
#include "Settings_ui.h"
#include "sysTrayObject.h"

#include "MainWin_ui.h"
// -----------------------------------------------------------------------------

#define ARD_DOCKED_WIDGETS 1
// -----------------------------------------------------------------------------

// Main application window.
MainWin_ui* MainWin;
// -----------------------------------------------------------------------------

// #brief Constructor MainWin_ui class.
//
// #param *parent Pointer to parent.
MainWin_ui::MainWin_ui(QMainWindow*)
{
     setupUi(this);

     setAcceptDrops(true);
     setWindowIcon(QIcon(":/all_images/images/mainico.png"));
     hqed::centerWindow(this);

     // ARD
     ard_modelView = new ardGraphicsView(centralwidget);
     ard_modelView->setObjectName(QString::fromUtf8("ard_graphicsView"));
     gridLayout->addWidget(ard_modelView, 0, 0, 1, 1);

     ard_modelScene = new ardGraphicsScene(ard_modelView);
     ard_modelScene->setItemIndexMethod(QGraphicsScene::NoIndex);
     ard_modelScene->setSceneRect(-200, -200, 400, 400);
     fCurrentARDscene = ard_modelScene;

     // XTT
     xtt_graphicsView = new MyGraphicsView(centralwidget);
     xtt_graphicsView->setObjectName(QString::fromUtf8("graphicsView"));
     gridLayout->addWidget(xtt_graphicsView, 0, 0, 1, 1);

     xtt_scene = new MyGraphicsScene(xtt_graphicsView);
     xtt_scene->setItemIndexMethod(QGraphicsScene::NoIndex);
     xtt_scene->setSceneRect(-200, -200, 400, 400);

     printer = 0;

     action_New->setStatusTip("Create new project");
     action_Open->setStatusTip("Open project...");
     action_Save->setStatusTip("Save project");
     actionSave_as->setStatusTip("Save project under new name...");
     action_Close->setStatusTip("Close program");
     actionEdit_tabele->setStatusTip("Run table editor...");
     actionEdit_attribute->setStatusTip("Run attribute editor...");
     actionNew_tabele->setStatusTip("Create new table dialog...");
     actionEdit_Attribute->setStatusTip("Delete table dialog...");
     actionAbout->setStatusTip("Show information about this program...");
     action_New_attribute->setStatusTip("Create new attribute...");
     actionDelete_attribute->setStatusTip("Delete attribute dialog...");
     action_Attribute_manager->setStatusTip("Run attributes magager...");
     actionGroups_manager->setStatusTip("Run groups manager...");
     actionQLine->setStatusTip("qline");
     actionZoom_in->setStatusTip("Increase zoom factor");
     actionZoom_out->setStatusTip("Decrease zoom factor");
     actionRotate_90->setStatusTip("rot90");
     actionCell_editor->setStatusTip("Run cell editor...");
     action_Add_new_connection->setStatusTip("Add new table dialog...");
     actionConnection_manager->setStatusTip("Run connections manager...");
     action_Edit_connection->setStatusTip("Connections editor...");
     action_Delete_connection->setStatusTip("Delete connections dialog...");
     actionZoom_in_2->setStatusTip("Increase zoom factor");
     actionZoom_out_2->setStatusTip("Decrease zoom factor");
     actionMove_all_left->setStatusTip("Moves all items left");
     actionMove_all_right->setStatusTip("Moves all items right");
     actionMove_all_down->setStatusTip("Moves all items down");
     actionMove_all_up->setStatusTip("Moves all items up");
     actionAdapt_scene_size->setStatusTip("Adapt scene size");
     actionAntialiasing->setStatusTip("On/Off antialiasing");
     actionTextAntialiasing->setStatusTip("On/Off text antialiasing");
     actionSmooth_pixmap_transform->setStatusTip("On/Off smooth pixmap transform");
     actionUnused_attributes->setStatusTip("Show unused attributes...");
     actionNew_with_the_same_attributes->setStatusTip("Create new project, but keep defined attributes");
     actionRun->setStatusTip("Verify and execute model");
     actionGenerate_PROLOG_code->setStatusTip("Generates the PROLOG code according to the XTT model");
     actionB_W_view->setStatusTip("Change color mode");
     actionXTTML_file_ver_1_0->setStatusTip("Save model to xttml ver. 1.0 file...");
     actionXTTML_file_ver_2_0->setStatusTip("Save model to xttml ver. 2.0 file...");
     actionXTTML_file_ver_3_0->setStatusTip("Save model to xttml ver. 3.0 file...");
     actionXTTML_file_ver_4_0->setStatusTip("Save model to xttml ver. 4.0 file...");
     actionATTML_file_ver_2_0->setStatusTip("Save attributes to attml ver. 2.0 file...");
     actionAttribute_import->setStatusTip("Import attributes from attml file...");
     actionReset_rotate->setStatusTip("Set the rotate angle to zero");
     actionExoprt_scene_to_SVG_file->setStatusTip("Save scene as SVG picture (Scalable Vector Graphics)");
     actionExoprt_scene_to_PNG_file->setStatusTip("Save scene as PNG picture (Portable Network Graphics)");
     actionExoprt_scene_to_BMP_file->setStatusTip("Save scene as BMP picture (Windows Bitmap)");
     actionExoprt_scene_to_JPG_file->setStatusTip("Save scene as JPG picture (Joint Photographic Experts Group)");
     actionExoprt_scene_to_PPM_file->setStatusTip("Save scene as PPM picture (Portable Pixmap)");
     actionExoprt_scene_to_XBM_file->setStatusTip("Save scene as XBM picture (X11 Bitmap)");
     actionExoprt_scene_to_XPM_file->setStatusTip("Save scene as XPM picture (X11 Pixmap)");
     actionPrint->setStatusTip("Print the scene");
     actionPrint_setup->setStatusTip("Printer setup...");
     actionSettings->setStatusTip("Application settingss");
     actionType_manager->setStatusTip("The type manager dialog window");

     // signals/slots mechanism in action
     connect(action_New_attribute, SIGNAL(triggered()), this, SLOT(AddNewAttribute()));
     connect(actionEdit_attribute, SIGNAL(triggered()), this, SLOT(EditAttribute()));
     connect(actionDelete_attribute, SIGNAL(triggered()), this, SLOT(DeleteAttribute()));
     connect(actionGroups_manager, SIGNAL(triggered()), this, SLOT(GroupManagerShow()));
     connect(action_Attribute_manager, SIGNAL(triggered()), this, SLOT(AttributeManagerShow()));
     connect(actionNew_tabele, SIGNAL(triggered()), this, SLOT(TableEditorShow()));
     connect(actionEdit_tabele, SIGNAL(triggered()), this, SLOT(TableEditorEdit()));
     connect(actionEdit_Attribute, SIGNAL(triggered()), this, SLOT(DeleteTableShow()));
     connect(actionCell_editor, SIGNAL(triggered()), this, SLOT(ShowCellEditor()));
     connect(action_Add_new_connection, SIGNAL(triggered()), this, SLOT(ShowConnEditor()));
     connect(action_Edit_connection, SIGNAL(triggered()), this, SLOT(ShowConnEditorInEditMode()));
     connect(action_Delete_connection, SIGNAL(triggered()), this, SLOT(ShowConnectionDeleteWindow()));
     connect(actionConnection_manager, SIGNAL(triggered()), this, SLOT(ShowConnectionManager()));
     connect(actionAntialiasing, SIGNAL(triggered()), this, SLOT(UpdateRenderHints()));
     connect(actionTextAntialiasing, SIGNAL(triggered()), this, SLOT(UpdateRenderHints()));
     connect(actionSmooth_pixmap_transform, SIGNAL(triggered()), this, SLOT(UpdateRenderHints()));
     connect(this, SIGNAL(destroyed()), this, SLOT(CloseForm()));
     connect(action_Close, SIGNAL(triggered()), this, SLOT(CloseForm()));
     connect(actionUnused_attributes, SIGNAL(triggered()), this, SLOT(ShowUnusedAttributes()));
     connect(actionAbout, SIGNAL(triggered()), this, SLOT(ShowAboutWin()));
     connect(action_New, SIGNAL(triggered()), this, SLOT(NewClick()));
     connect(actionNew_with_the_same_attributes, SIGNAL(triggered()), this, SLOT(NewWhithOldAttributiesClick()));
     connect(action_Save, SIGNAL(triggered()), this, SLOT(save()));
     connect(actionSave_as, SIGNAL(triggered()), this, SLOT(saveAs_HML_2_0()));
     connect(actionCreate_XTT, SIGNAL(triggered()), this, SLOT(generateXTT()));
     connect(actionGenerate_PROLOG_code, SIGNAL(triggered()), this, SLOT(generatePROLOG()));
     connect(actionRun, SIGNAL(triggered()), this, SLOT(ExecuteModel()));
     connect(actionB_W_view, SIGNAL(triggered()), this, SLOT(SwitchColorMode()));
     //connect(actionAttribute_import, SIGNAL(triggered()), this, SLOT(openATTMLFile()));
     connect(att_export_to_hml_file, SIGNAL(triggered()), this, SLOT(exportAttributesToHML20file()));
     connect(actionReset_rotate, SIGNAL(triggered()), this, SLOT(resetRotate()));
     connect(actionExoprt_scene_to_SVG_file, SIGNAL(triggered()), this, SLOT(exportSceneToSVGfile()));
     connect(actionExoprt_scene_to_PNG_file, SIGNAL(triggered()), this, SLOT(exportSceneToPNGfile()));
     connect(actionExoprt_scene_to_BMP_file, SIGNAL(triggered()), this, SLOT(exportSceneToBMPfile()));
     connect(actionExoprt_scene_to_JPG_file, SIGNAL(triggered()), this, SLOT(exportSceneToJPGfile()));
     connect(actionExoprt_scene_to_PPM_file, SIGNAL(triggered()), this, SLOT(exportSceneToPPMfile()));
     connect(actionExoprt_scene_to_XBM_file, SIGNAL(triggered()), this, SLOT(exportSceneToXBMfile()));
     connect(actionExoprt_scene_to_XPM_file, SIGNAL(triggered()), this, SLOT(exportSceneToXPMfile()));
     connect(actionExoprt_TPH_diagram_to_SVG_file, SIGNAL(triggered()), this, SLOT(exportTPHtoSVGfile()));
     connect(actionExoprt_TPH_diagram_to_PNG_file, SIGNAL(triggered()), this, SLOT(exportTPHtoPNGfile()));
     connect(actionExoprt_TPH_diagram_to_BMP_file, SIGNAL(triggered()), this, SLOT(exportTPHtoBMPfile()));
     connect(actionExoprt_TPH_diagram_to_JPG_file, SIGNAL(triggered()), this, SLOT(exportTPHtoJPGfile()));
     connect(actionExoprt_TPH_diagram_to_PPM_file, SIGNAL(triggered()), this, SLOT(exportTPHtoPPMfile()));
     connect(actionExoprt_TPH_diagram_to_XBM_file, SIGNAL(triggered()), this, SLOT(exportTPHtoXBMfile()));
     connect(actionExoprt_TPH_diagram_to_XPM_file, SIGNAL(triggered()), this, SLOT(exportTPHtoXPMfile()));
     connect(actionAbout_Qt, SIGNAL(triggered()), qApp, SLOT(aboutQt()));
     connect(actionPrint, SIGNAL(triggered()), this, SLOT(printModel()));
     connect(actionPrint_setup, SIGNAL(triggered()), this, SLOT(printSetup()));
     connect(actionSettings, SIGNAL(triggered()), this, SLOT(showConfigurationDialog()));
     connect(actionARD_editor, SIGNAL(triggered()), this, SLOT(useARDmode()));
     connect(actionXTT_editor, SIGNAL(triggered()), this, SLOT(useXTTmode()));
     connect(actionLay_out_tables, SIGNAL(triggered()), this, SLOT(tablesArrangment()));
     connect(actionType_manager, SIGNAL(triggered()), this, SLOT(typeManagerShow()));

     // Create dock windows and toolbars.
     xtt_createMenu();
     ard_createDockWindows();
     xtt_createDockWindows();
     xtt_createToolBars();
     createStatusBar();

     // Add shortcut to menu.
     actionMove_all_right->setShortcut(Qt::CTRL + Qt::Key_Right);
     actionMove_all_left->setShortcut(Qt::CTRL + Qt::Key_Left);
     actionMove_all_up->setShortcut(Qt::CTRL + Qt::Key_Up);
     actionMove_all_down->setShortcut(Qt::CTRL + Qt::Key_Down);
     actionReset_rotate->setShortcut(tr("Ctrl+R"));
     actionARD_editor->setShortcut(tr("F11"));
     actionXTT_editor->setShortcut(tr("F12"));

     // Set active render buttons.
     actionAntialiasing->setChecked(true);
     actionTextAntialiasing->setChecked(true);
     actionSmooth_pixmap_transform->setChecked(true);

     xtt_graphicsView->setScene(xtt_scene);
     xtt_graphicsView->setCacheMode(QGraphicsView::CacheBackground);
     xtt_graphicsView->setRenderHints(QPainter::Antialiasing | QPainter::TextAntialiasing | QPainter::SmoothPixmapTransform);
     xtt_graphicsView->setTransformationAnchor(QGraphicsView::AnchorUnderMouse);
     xtt_graphicsView->setResizeAnchor(QGraphicsView::AnchorViewCenter);
     xtt_scene->AdaptSceneSize();

     ard_modelView->setScene(ard_modelScene);
     ard_modelView->setCacheMode(QGraphicsView::CacheBackground);
     ard_modelView->setRenderHints(QPainter::Antialiasing | QPainter::TextAntialiasing | QPainter::SmoothPixmapTransform);
     ard_modelView->setTransformationAnchor(QGraphicsView::AnchorUnderMouse);
     ard_modelView->setResizeAnchor(QGraphicsView::AnchorViewCenter);
     ard_modelScene->AdaptSceneSize();

     ard_tphView->setScene(ard_thpScene);
     ard_tphView->setCacheMode(QGraphicsView::CacheBackground);
     ard_tphView->setRenderHints(QPainter::Antialiasing | QPainter::TextAntialiasing | QPainter::SmoothPixmapTransform);
     ard_tphView->setTransformationAnchor(QGraphicsView::AnchorUnderMouse);
     ard_tphView->setResizeAnchor(QGraphicsView::AnchorViewCenter);
     ard_thpScene->AdaptSceneSize();

     menu_Execute->setTearOffEnabled(true);
     menu_Edit->setTearOffEnabled(true);
     menu_Connections->setTearOffEnabled(true);
     menu_View->setTearOffEnabled(true);
     menu_Help->setTearOffEnabled(true);
     menu_File->setTearOffEnabled(true);
     menuAttribute_export->setTearOffEnabled(true);
     menuExport_scene_view->setTearOffEnabled(true);
     menuTools->setTearOffEnabled(true);
     menu_View_2->setTearOffEnabled(true);
     menuRender_hint->setTearOffEnabled(true);
     menuMode->setTearOffEnabled(true);
     menuExport_TPH_diagram->setTearOffEnabled(true);
     menuTypes->setTearOffEnabled(true);

     // Determine whether tips for various program procedures should be visible.
     fShowARDtip = Settings->showTrayTips();
     fShowXTTtip = Settings->showTrayTips();

     isXTTFile = false;
     fXttFileName = "New project";
     setXTT_Changed(false);

     isARDFile = false;
     fArdFileName = "New project";
     //setARD_Changed(false);

     // Set icon in resources.
     trayIcon = new sysTrayObject(this);
     if((Settings->showSystemTrayIcon()) && (QSystemTrayIcon::isSystemTrayAvailable()))
     trayIcon->show();

     setCurrentMode(XTT_MODE);
}
// -----------------------------------------------------------------------------

// #brief Destructore MainWin_ui class.
MainWin_ui::~MainWin_ui(void)
{
     delete xtt_graphicsView;
     delete xtt_scene;

     delete ard_modelView;
     delete ard_tphView;
     delete ard_modelScene;
     delete ard_thpScene;

     delete printer;

     delete toolBarMenu;
     delete fileToolBar;
     delete viewToolBar;
     delete tableToolBar;
     delete attributesToolBar;
     delete connectionsToolBar;
     delete executeToolBar;
     delete helpToolBar;
     delete dial;
     delete rotateAngleEdit;
     delete fCurrentARDscene;
     delete rotateAngleEdit;
     delete dial;
     delete dockedWindows;
     delete printer;

     delete ard_tphDockWindow;
     delete xtt_errorsDockWindow;
     delete xtt_runModelMessagesDockWindow;

     delete xtt_runModelMessagesDockWindowContents;
     delete xtt_runModelMessagesDockWindowGridLayout;
     delete xtt_runModelMessagesDockWindowMessagesList;
     delete xtt_runModelMessagesDockWindowSpacerItem;
     delete xtt_runModelMessagesDockWindowInfoCheckBox;
     delete xtt_runModelMessagesDockWindowWarningCheckBox;
     delete xtt_runModelMessagesDockWindowErrorCheckBox;
     delete xtt_runModelMessagesDockWindowLabel;

     delete trayIcon;
     //delete trayIconMenu;
}
// -----------------------------------------------------------------------------

// #brief Function executes when window is showing.
//
// #param event Event parameters.
// #return No return value.
void MainWin_ui::showEvent(QShowEvent*)
{
     if((qApp->arguments().size() > 1) && (QFile::exists(qApp->arguments().last())))
          openFile(qApp->arguments().last());
}
// -----------------------------------------------------------------------------

// #brief Handle event of dragging file on window area and check it data tape.
//
// #param event Pointer to parameters of dragging data on form.
// #return No return value.
void MainWin_ui::dragEnterEvent(QDragEnterEvent *event)
{
     if(event->mimeData()->hasFormat("text/uri-list"))
          event->acceptProposedAction();
}
// -----------------------------------------------------------------------------

// #brief Handle event of dropping file on window area and check it data tape.
//
// #param event Pointer to parameters of dropping data on form.
// #return No return value.
void MainWin_ui::dropEvent(QDropEvent *event)
{
     QList<QUrl> urlList = event->mimeData()->urls();
     if(urlList.size() > 0)
     {
            QString url = urlList.at(0).toLocalFile();
            openFile(url);
     }

     event->acceptProposedAction();
}
// -----------------------------------------------------------------------------

// #brief Create menu for XTT mode.
//
// #return No return value.
void MainWin_ui::xtt_createMenu(void)
{
     toolBarMenu = new QMenu("Tollbar");
     menu_View_2->insertMenu(actionZoom_in_2, toolBarMenu);
     menu_View_2->insertSeparator(actionZoom_in_2);

     dockedWindows = new QMenu("Docked windows");
     menu_View_2->insertMenu(actionZoom_in_2, dockedWindows);
     menu_View_2->insertSeparator(actionZoom_in_2);
}
// -----------------------------------------------------------------------------

// #brief Create toolbars for XTT mode.
//
// #return No return value.
void MainWin_ui::xtt_createToolBars(void)
{
     // Menu File.
     fileToolBar = addToolBar(tr("File"));
     fileToolBar->addAction(action_Close);
     fileToolBar->addSeparator();
     fileToolBar->addAction(action_New);
     fileToolBar->addAction(action_Open);
     fileToolBar->addAction(action_Save);
     fileToolBar->addSeparator();
     fileToolBar->addAction(actionExoprt_scene_to_SVG_file);
     fileToolBar->addSeparator();
     fileToolBar->addAction(actionPrint);
     fileToolBar->addAction(actionPrint_setup);
     toolBarMenu->addAction(fileToolBar->toggleViewAction());

     // Object on toolbars.
     dial = new QDial(NULL);
     dial->setGeometry(0, 0, 32, 32);
     dial->setMaximumSize(32, 32);
     dial->setMinimum(0);
     dial->setMaximum(359);
     dial->setStatusTip("Rotate scene");
     rotateAngleEdit = new QSpinBox(NULL);
     rotateAngleEdit->setMaximumSize(40, 20);
     rotateAngleEdit->setMinimum(0);
     rotateAngleEdit->setMaximum(359);
     connect(dial, SIGNAL(valueChanged(int)), rotateAngleEdit, SLOT(setValue(int)));
     connect(rotateAngleEdit, SIGNAL(valueChanged(int)), this, SLOT(angleEditorChange(int)));

     // Menu view.
     viewToolBar = addToolBar(tr("View"));
     viewToolBar->addAction(actionZoom_in_2);
     viewToolBar->addAction(actionZoom_out_2);
     viewToolBar->addSeparator();
     viewToolBar->addAction(actionMove_all_left);
     viewToolBar->addAction(actionMove_all_right);
     viewToolBar->addAction(actionMove_all_up);
     viewToolBar->addAction(actionMove_all_down);
     viewToolBar->addSeparator();
     viewToolBar->addAction(actionAdapt_scene_size);
     viewToolBar->addSeparator();
     viewToolBar->addWidget(dial);
     viewToolBar->addWidget(rotateAngleEdit);
     toolBarMenu->addAction(viewToolBar->toggleViewAction());

     // Menu of types.
     typeToolBar = addToolBar(tr("Type"));
     typeToolBar->addAction(actionType_manager);
     toolBarMenu->addAction(typeToolBar->toggleViewAction());

     // Menu of attributes.
     attributesToolBar = addToolBar(tr("Atributes"));
     attributesToolBar->addAction(action_New_attribute);
     attributesToolBar->addAction(actionEdit_attribute);
     attributesToolBar->addAction(actionDelete_attribute);
     attributesToolBar->addSeparator();
     attributesToolBar->addAction(actionUnused_attributes);
     attributesToolBar->addSeparator();
     attributesToolBar->addAction(action_Attribute_manager);
     attributesToolBar->addSeparator();
     attributesToolBar->addAction(actionGroups_manager);
     toolBarMenu->addAction(attributesToolBar->toggleViewAction());

     // Menu of tabels.
     tableToolBar = addToolBar(tr("Table"));
     tableToolBar->addAction(actionNew_tabele);
     tableToolBar->addAction(actionEdit_tabele);
     tableToolBar->addAction(actionEdit_Attribute);
     tableToolBar->addSeparator();
     tableToolBar->addAction(actionCell_editor);
     toolBarMenu->addAction(tableToolBar->toggleViewAction());

     // Menu of connections.
     connectionsToolBar = addToolBar(tr("Connections"));
     connectionsToolBar->addAction(action_Add_new_connection);
     connectionsToolBar->addAction(action_Edit_connection);
     connectionsToolBar->addAction(action_Delete_connection);
     connectionsToolBar->addSeparator();
     connectionsToolBar->addAction(actionConnection_manager);
     toolBarMenu->addAction(connectionsToolBar->toggleViewAction());

     // Menu of runtime.
     executeToolBar = addToolBar(tr("Execute"));
     executeToolBar->addAction(actionRun);
     executeToolBar->addSeparator();
     executeToolBar->addAction(actionGenerate_PROLOG_code);
     toolBarMenu->addAction(executeToolBar->toggleViewAction());

     // Menu of tools.
     toolsToolBar = addToolBar(tr("Settings"));
     toolsToolBar->addAction(actionSettings);
     toolBarMenu->addAction(toolsToolBar->toggleViewAction());

     // Menu of help.
     helpToolBar = addToolBar(tr("Help"));
     helpToolBar->addAction(actionAbout);
     toolBarMenu->addAction(helpToolBar->toggleViewAction());

     addToolBar(Qt::LeftToolBarArea, viewToolBar);
}
// -----------------------------------------------------------------------------

// #brief Create dock windows for ARD mode.
//
// #return No return value.
void MainWin_ui::ard_createDockWindows(void)
{
     fDockWidgetsVisibleStatus[0] = true;

     // Create windows for error raporting.
     ard_tphDockWindow = new QDockWidget(tr("TPH diagram"), this);
     ard_tphView = new ardGraphicsView(ard_tphDockWindow);
     ard_thpScene = new ardGraphicsScene(ard_tphView);
     ard_tphDockWindow->setWidget(ard_tphView);
     addDockWidget(Qt::LeftDockWidgetArea, ard_tphDockWindow);
     dockedWindows->addAction(ard_tphDockWindow->toggleViewAction());
}
// -----------------------------------------------------------------------------

// #brief Create dock windows for XTT mode.
//
// #return No return value.
void MainWin_ui::xtt_createDockWindows(void)
{
     fDockWidgetsVisibleStatus[1] = Settings->xttSearchDockWidget();
     fDockWidgetsVisibleStatus[2] = Settings->xttErrorDockWidget();
     fDockWidgetsVisibleStatus[3] = Settings->xttExecuteDockWidget();

     // Create windows for error raporting.
     xtt_errorsDockWindow = new QDockWidget(tr("Errors"), this);
     xtt_errorsDockWindow->setAllowedAreas(Qt::TopDockWidgetArea | Qt::BottomDockWidgetArea);
     xtt_errorsList = new ErrorsListWidget(xtt_errorsDockWindow);
     xtt_errorsDockWindow->setWidget(xtt_errorsList);
     addDockWidget(Qt::BottomDockWidgetArea, xtt_errorsDockWindow);
     dockedWindows->addAction(xtt_errorsDockWindow->toggleViewAction());

     // Create windows for runtime raporting.
     xtt_runModelMessagesDockWindow = new QDockWidget(tr("Execute messages"), this);
     xtt_runModelMessagesDockWindow->setObjectName(QString::fromUtf8("Execute messages"));
     xtt_runModelMessagesDockWindow->setGeometry(QRect(40, 0, 261, 221));
     xtt_runModelMessagesDockWindowContents = new QWidget(xtt_runModelMessagesDockWindow);
     xtt_runModelMessagesDockWindowContents->setObjectName(QString::fromUtf8("xtt_runModelMessagesDockWindowContents"));
     xtt_runModelMessagesDockWindowMessagesList = new ExecMsgListWidget(xtt_runModelMessagesDockWindowContents);
     xtt_runModelMessagesDockWindowMessagesList->setObjectName(QString::fromUtf8("xtt_runModelMessagesDockWindowMessagesList"));
     xtt_runModelMessagesDockWindowInfoCheckBox = new QCheckBox(xtt_runModelMessagesDockWindowContents);
     xtt_runModelMessagesDockWindowInfoCheckBox->setObjectName(QString::fromUtf8("xtt_runModelMessagesDockWindowInfoCheckBox"));
     xtt_runModelMessagesDockWindowInfoCheckBox->setIcon(QIcon(QString::fromUtf8(":/all_images/images/information_small.png")));
     xtt_runModelMessagesDockWindowWarningCheckBox = new QCheckBox(xtt_runModelMessagesDockWindowContents);
     xtt_runModelMessagesDockWindowWarningCheckBox->setObjectName(QString::fromUtf8("xtt_runModelMessagesDockWindowWarningCheckBox"));
     xtt_runModelMessagesDockWindowWarningCheckBox->setIcon(QIcon(QString::fromUtf8(":/all_images/images/warning_small.png")));
     xtt_runModelMessagesDockWindowLabel = new QLabel(this);
     xtt_runModelMessagesDockWindowLabel->setText("Message filter:");
     xtt_runModelMessagesDockWindowLabel->setMargin(5);
     xtt_runModelMessagesDockWindowErrorCheckBox = new QCheckBox(xtt_runModelMessagesDockWindowContents);
     xtt_runModelMessagesDockWindowErrorCheckBox->setObjectName(QString::fromUtf8("xtt_runModelMessagesDockWindowErrorCheckBox"));
     xtt_runModelMessagesDockWindowErrorCheckBox->setIcon(QIcon(QString::fromUtf8(":/all_images/images/critical_small.png")));
     xtt_runModelMessagesDockWindowInfoCheckBox->setText(QString());
     xtt_runModelMessagesDockWindowWarningCheckBox->setText(QString());
     xtt_runModelMessagesDockWindowErrorCheckBox->setText(QString());
     xtt_runModelMessagesDockWindowInfoCheckBox->setChecked(true);
     xtt_runModelMessagesDockWindowWarningCheckBox->setChecked(true);
     xtt_runModelMessagesDockWindowErrorCheckBox->setChecked(true);
     xtt_runModelMessagesDockWindowErrorCheckBox->setIconSize(QSize(18, 18));
     xtt_runModelMessagesDockWindowWarningCheckBox->setIconSize(QSize(18, 18));
     xtt_runModelMessagesDockWindowInfoCheckBox->setIconSize(QSize(18, 18));
     xtt_runModelMessagesDockWindowSpacerItem = new QSpacerItem(131, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);
     xtt_runModelMessagesDockWindowGridLayout = new QGridLayout(xtt_runModelMessagesDockWindowContents);
     #ifndef Q_OS_MAC
          xtt_runModelMessagesDockWindowGridLayout->setSpacing(6);
     #endif
     #ifndef Q_OS_MAC
          xtt_runModelMessagesDockWindowGridLayout->setMargin(9);
     #endif
     #ifdef __USE_SVG
          xtt_runModelMessagesDockWindowGridLayout->setContentsMargins(0, 0, 0, 0);
     #endif
     xtt_runModelMessagesDockWindowGridLayout->setObjectName(QString::fromUtf8("xtt_runModelMessagesDockWindowGridLayout"));
     xtt_runModelMessagesDockWindowGridLayout->addItem(xtt_runModelMessagesDockWindowSpacerItem,        0, 4, 1, 1);
     xtt_runModelMessagesDockWindowGridLayout->addWidget(xtt_runModelMessagesDockWindowMessagesList,    1, 0, 1, 5);
     xtt_runModelMessagesDockWindowGridLayout->addWidget(xtt_runModelMessagesDockWindowLabel,           0, 0, 1, 1);
     xtt_runModelMessagesDockWindowGridLayout->addWidget(xtt_runModelMessagesDockWindowErrorCheckBox,   0, 1, 1, 1);
     xtt_runModelMessagesDockWindowGridLayout->addWidget(xtt_runModelMessagesDockWindowWarningCheckBox, 0, 2, 1, 1);
     xtt_runModelMessagesDockWindowGridLayout->addWidget(xtt_runModelMessagesDockWindowInfoCheckBox,    0, 3, 1, 1);
     xtt_runModelMessagesDockWindow->setWidget(xtt_runModelMessagesDockWindowContents);
     connect(xtt_runModelMessagesDockWindowErrorCheckBox, SIGNAL(toggled(bool)), xtt_runModelMessagesDockWindowMessagesList, SLOT(showErrors(bool)));
     connect(xtt_runModelMessagesDockWindowWarningCheckBox, SIGNAL(toggled(bool)), xtt_runModelMessagesDockWindowMessagesList, SLOT(showWarnings(bool)));
     connect(xtt_runModelMessagesDockWindowInfoCheckBox, SIGNAL(toggled(bool)), xtt_runModelMessagesDockWindowMessagesList, SLOT(showInformations(bool)));
     addDockWidget(Qt::BottomDockWidgetArea, xtt_runModelMessagesDockWindow);
     dockedWindows->addAction(xtt_runModelMessagesDockWindow->toggleViewAction());


     // Creating itemFinder dock window
     xtt_itemFinderDockWindow = new QDockWidget(tr("Item finder"), this);
     xtt_itemFinderDockWindow->setAllowedAreas(Qt::AllDockWidgetAreas);
     xtt_itemFinderDockWindow->setWidget(ItemFinderWin);
     addDockWidget(Qt::LeftDockWidgetArea, xtt_itemFinderDockWindow);
     QAction* ifa = xtt_itemFinderDockWindow->toggleViewAction();
     ifa->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_F));
     dockedWindows->addAction(ifa);
}
// -----------------------------------------------------------------------------

// #brief Create statusbar.
//
// #return No return value.
void MainWin_ui::createStatusBar(void)
{
     statusBar()->showMessage(tr("Ready"));
}
// -----------------------------------------------------------------------------

// #brief Get current mode of application.
//
// #return 0 - editor ARD, 1 - editor XTT.
int MainWin_ui::currentMode(void)
{
     return appMode;
}
// -----------------------------------------------------------------------------

// #brief Set current mode of application.
//
// #param _newMode New mode of application: 0 - editor ARD, 1 - editor XTT.
// #return No return value.
void MainWin_ui::setCurrentMode(int _newMode)
{
     if((_newMode < 0) || (_newMode > 1))
          return;

     // Saving visibility status
     if(appMode == ARD_MODE)
     {
          fDockWidgetsVisibleStatus[0] = ard_tphDockWindow->isVisible();
     }
     if(appMode == XTT_MODE)
     {
          fDockWidgetsVisibleStatus[1] = xtt_itemFinderDockWindow->isVisible();
          fDockWidgetsVisibleStatus[2] = xtt_errorsDockWindow->isVisible();
          fDockWidgetsVisibleStatus[3] = xtt_runModelMessagesDockWindow->isVisible();
     }

     ard_modelView->setVisible(_newMode == ARD_MODE);
     ard_tphDockWindow->setVisible((_newMode == ARD_MODE) && (fDockWidgetsVisibleStatus[0]));
     actionARD_editor->setChecked(_newMode == ARD_MODE);
     actionExoprt_TPH_diagram_to_SVG_file->setVisible(_newMode == ARD_MODE);
     actionExoprt_TPH_diagram_to_BMP_file->setVisible(_newMode == ARD_MODE);
     actionExoprt_TPH_diagram_to_PNG_file->setVisible(_newMode == ARD_MODE);
     actionExoprt_TPH_diagram_to_JPG_file->setVisible(_newMode == ARD_MODE);
     actionExoprt_TPH_diagram_to_PPM_file->setVisible(_newMode == ARD_MODE);
     actionExoprt_TPH_diagram_to_XBM_file->setVisible(_newMode == ARD_MODE);
     actionExoprt_TPH_diagram_to_XPM_file->setVisible(_newMode == ARD_MODE);
     actionCreate_XTT->setVisible(_newMode == ARD_MODE);
     actionARDML_file_ver_1_0->setVisible(_newMode == ARD_MODE);

     action_New->setVisible(_newMode == XTT_MODE);
     att_export_to_hml_file->setVisible(_newMode == XTT_MODE);
     xtt_graphicsView->setVisible(_newMode == XTT_MODE);
     xtt_errorsDockWindow->setVisible((_newMode == XTT_MODE) && (fDockWidgetsVisibleStatus[2]));
     xtt_runModelMessagesDockWindow->setVisible((_newMode == XTT_MODE) && (fDockWidgetsVisibleStatus[3]));
     xtt_itemFinderDockWindow->setVisible((_newMode == XTT_MODE) && (fDockWidgetsVisibleStatus[1]));
     actionXTT_editor->setChecked(_newMode == XTT_MODE);
     tableToolBar->setVisible(_newMode == XTT_MODE);
     attributesToolBar->setVisible(_newMode == XTT_MODE);
     connectionsToolBar->setVisible(_newMode == XTT_MODE);
     executeToolBar->setVisible(_newMode == XTT_MODE);
     actionRun->setVisible(_newMode == XTT_MODE);
     actionGenerate_PROLOG_code->setVisible(_newMode == XTT_MODE);
     actionNew_with_the_same_attributes->setVisible(_newMode == XTT_MODE);
     action_Save->setVisible(_newMode == XTT_MODE);
     actionSave_as->setVisible(_newMode == XTT_MODE);

     for(int i=0;i<dockedWindows->actions().size();++i)
          dockedWindows->actions().at(i)->setVisible(((i < ARD_DOCKED_WIDGETS) && (_newMode == ARD_MODE)) || ((i >= ARD_DOCKED_WIDGETS) && (_newMode == XTT_MODE)));

     // Disconnect slots.
     disconnect(action_Open, 0, 0, 0);
     disconnect(dial, SIGNAL(valueChanged(int)), 0, 0);
     disconnect(rotateAngleEdit, SIGNAL(valueChanged(int)), 0, 0);
     disconnect(actionZoom_out_2, SIGNAL(triggered()), 0, 0);
     disconnect(actionZoom_in_2, SIGNAL(triggered()), 0, 0);
     disconnect(actionMove_all_right, SIGNAL(triggered()), 0, 0);
     disconnect(actionMove_all_left, SIGNAL(triggered()), 0, 0);
     disconnect(actionMove_all_up, SIGNAL(triggered()), 0, 0);
     disconnect(actionMove_all_down, SIGNAL(triggered()), 0, 0);
     disconnect(actionAdapt_scene_size, SIGNAL(triggered()), 0, 0);

     if(_newMode == ARD_MODE)
     {
          dial->setValue(fCurrentARDscene->currentAngle());
          rotateAngleEdit->setValue(fCurrentARDscene->currentAngle());

          setWindowTitle("ARD Editor - [" + fArdFileName + "]");
          connect(action_Open, SIGNAL(triggered()), this, SLOT(openARDFileDialog()));

          connect(dial, SIGNAL(valueChanged(int)), rotateAngleEdit, SLOT(setValue(int)));
          connect(rotateAngleEdit, SIGNAL(valueChanged(int)), this, SLOT(angleEditorChange(int)));
          connect(dial, SIGNAL(valueChanged(int)), fCurrentARDscene, SLOT(Rotate(int)));
          connect(actionZoom_out_2, SIGNAL(triggered()), fCurrentARDscene, SLOT(zout()));
          connect(actionZoom_in_2, SIGNAL(triggered()), fCurrentARDscene, SLOT(zin()));
          connect(actionMove_all_right, SIGNAL(triggered()), fCurrentARDscene, SLOT(MoveItemsRight()));
          connect(actionMove_all_left, SIGNAL(triggered()), fCurrentARDscene, SLOT(MoveItemsLeft()));
          connect(actionMove_all_up, SIGNAL(triggered()), fCurrentARDscene, SLOT(MoveItemsUp()));
          connect(actionMove_all_down, SIGNAL(triggered()), fCurrentARDscene, SLOT(MoveItemsDown()));
          connect(actionAdapt_scene_size, SIGNAL(triggered()), fCurrentARDscene, SLOT(AdaptSceneSize()));

          menu_View->setTitle("");
          menu_Edit->setTitle("");
          menu_Connections->setTitle("");

          // Show tips for ARD mode.
          if(((QSystemTrayIcon::isSystemTrayAvailable())) && (QSystemTrayIcon::supportsMessages()) && (Settings->showSystemTrayIcon()) && (fShowARDtip))
          {
               fShowARDtip = false;
               trayIcon->drawTip(ARD_MODE);
               trayIcon->showDrawedTip();
          }
     }

     if(_newMode == XTT_MODE)
     {
          dial->setValue(xtt_scene->currentAngle());
          rotateAngleEdit->setValue(xtt_scene->currentAngle());

          setWindowTitle("XTT Editor - [" + fXttFileName + "]");
          connect(action_Open, SIGNAL(triggered()), this, SLOT(openXTTFileDialog()));

          connect(dial, SIGNAL(valueChanged(int)), rotateAngleEdit, SLOT(setValue(int)));
          connect(rotateAngleEdit, SIGNAL(valueChanged(int)), this, SLOT(angleEditorChange(int)));
          connect(dial, SIGNAL(valueChanged(int)), xtt_scene, SLOT(Rotate(int)));
          connect(actionZoom_out_2, SIGNAL(triggered()), xtt_scene, SLOT(zout()));
          connect(actionZoom_in_2, SIGNAL(triggered()), xtt_scene, SLOT(zin()));
          connect(actionMove_all_right, SIGNAL(triggered()), xtt_scene, SLOT(MoveItemsRight()));
          connect(actionMove_all_left, SIGNAL(triggered()), xtt_scene, SLOT(MoveItemsLeft()));
          connect(actionMove_all_up, SIGNAL(triggered()), xtt_scene, SLOT(MoveItemsUp()));
          connect(actionMove_all_down, SIGNAL(triggered()), xtt_scene, SLOT(MoveItemsDown()));
          connect(actionAdapt_scene_size, SIGNAL(triggered()), xtt_scene, SLOT(AdaptSceneSize()));

          menu_View->setTitle("&Attributes");
          menu_Edit->setTitle("&Table");
          menu_Connections->setTitle("&Connections");

          // Show tips for ARD mode.
          if(((QSystemTrayIcon::isSystemTrayAvailable())) && (QSystemTrayIcon::supportsMessages()) && (Settings->showSystemTrayIcon()) && (fShowXTTtip))
          {
               fShowXTTtip = false;
               trayIcon->drawTip(XTT_MODE);
               trayIcon->showDrawedTip();
          }
     }

     appMode = _newMode;

     // Uaktualnienie trybu kolorow
     actionB_W_view->setChecked(Settings->colorMode(_newMode));
     SwitchColorMode();

     // Uaktualnienie flag renderingu
     actionAntialiasing->setChecked(Settings->useAntialiasing(_newMode));
     actionTextAntialiasing->setChecked(Settings->useTextAntialiasing(_newMode));
     actionSmooth_pixmap_transform->setChecked(Settings->useSmoothPixmapTransform(_newMode));
     UpdateRenderHints();
}
// -----------------------------------------------------------------------------

// #brief Switch application in ARD mode.
//
// #return No return value.
void MainWin_ui::useARDmode(void)
{
     setCurrentMode(ARD_MODE);
}
// -----------------------------------------------------------------------------

// #brief Switch application in XTT mode.
//
// #return No return value.
void MainWin_ui::useXTTmode(void)
{
     setCurrentMode(XTT_MODE);
}
// -----------------------------------------------------------------------------

// #brief Set current ART scene.
//
// #param ardncs Pointer to current scene ARD.
// #return No return value..
void MainWin_ui::setCurrentARDscene(ardGraphicsScene* ardncs)
{
     if(fCurrentARDscene == ardncs)
          return;

     fCurrentARDscene = ardncs;

     // Disconnect slot.
     disconnect(dial, SIGNAL(valueChanged(int)), 0, 0);
     disconnect(actionZoom_out_2, SIGNAL(triggered()), 0, 0);
     disconnect(actionZoom_in_2, SIGNAL(triggered()), 0, 0);
     disconnect(actionMove_all_right, SIGNAL(triggered()), 0, 0);
     disconnect(actionMove_all_left, SIGNAL(triggered()), 0, 0);
     disconnect(actionMove_all_up, SIGNAL(triggered()), 0, 0);
     disconnect(actionMove_all_down, SIGNAL(triggered()), 0, 0);
     disconnect(actionAdapt_scene_size, SIGNAL(triggered()), 0, 0);
     disconnect(rotateAngleEdit, SIGNAL(valueChanged(int)), 0, 0);

     // Create new slots.
     connect(dial, SIGNAL(valueChanged(int)), rotateAngleEdit, SLOT(setValue(int)));
     connect(rotateAngleEdit, SIGNAL(valueChanged(int)), this, SLOT(angleEditorChange(int)));
     connect(dial, SIGNAL(valueChanged(int)), fCurrentARDscene, SLOT(Rotate(int)));
     connect(actionZoom_out_2, SIGNAL(triggered()), fCurrentARDscene, SLOT(zout()));
     connect(actionZoom_in_2, SIGNAL(triggered()), fCurrentARDscene, SLOT(zin()));
     connect(actionMove_all_right, SIGNAL(triggered()), fCurrentARDscene, SLOT(MoveItemsRight()));
     connect(actionMove_all_left, SIGNAL(triggered()), fCurrentARDscene, SLOT(MoveItemsLeft()));
     connect(actionMove_all_up, SIGNAL(triggered()), fCurrentARDscene, SLOT(MoveItemsUp()));
     connect(actionMove_all_down, SIGNAL(triggered()), fCurrentARDscene, SLOT(MoveItemsDown()));
     connect(actionAdapt_scene_size, SIGNAL(triggered()), fCurrentARDscene, SLOT(AdaptSceneSize()));

     dial->setValue(fCurrentARDscene->currentAngle());
     rotateAngleEdit->setValue(fCurrentARDscene->currentAngle());
}
// -----------------------------------------------------------------------------

// #brief Print model.
//
// #return No return value.
void MainWin_ui::printModel(void)
{
     if(!printer)
          printer = new QPrinter;

     if(printer)
     {
          QPainter pp(printer);

          if(appMode == ARD_MODE)
          {
               QMessageBox msgBox(this);
               msgBox.setText("Which scene would you like print?");
               QPushButton *tphButton = msgBox.addButton(tr("TPH"), QMessageBox::YesRole);
               QPushButton *ardButton = msgBox.addButton(tr("ARD"), QMessageBox::NoRole);
               msgBox.exec();

               if(msgBox.clickedButton() == tphButton)
                    ard_thpScene->render(&pp);
               else if (msgBox.clickedButton() == ardButton)
                    ard_modelScene->render(&pp);
          }

          if(appMode == XTT_MODE)
               xtt_scene->render(&pp);
     }
}
// -----------------------------------------------------------------------------

// #brief Show print setup window.
//
// #return No return value.
void MainWin_ui::printSetup(void)
{
     if(!printer)
          printer = new QPrinter;

     QPrintDialog *dlg = new QPrintDialog(printer, this);
     if(dlg->exec() != QDialog::Accepted)
          return;

     printModel();
}
// -----------------------------------------------------------------------------

// #brief Close application.
//
// #return No return value.
void MainWin_ui::CloseForm(void)
{
     close();
}
// -----------------------------------------------------------------------------

// #brief Function triggered when window is closed.
//
// #param event Pointer to event object.
// #return No return value.
void MainWin_ui::closeEvent(QCloseEvent* event)
{
     bool confirmOnlyExit = true;          // Okresla czy nalezy potwierdzuc samo wyjscie z programu

     // Sprawdzanie modyfiakcji pliku
     // if((currentMode() == ARD_MODE) && (ARD_Changed()))
     // {
          // switch(QMessageBox::question(NULL, "HQEd", "Current ARD project has been changed. Do you want to save changes?", QMessageBox::Yes | QMessageBox::No | QMessageBox::Cancel))
          // {
               // case QMessageBox::Yes:
                    // saveToXTTML1_0(); !!!!!!!!!!!!
                    // confirmOnlyExit = false;
                    // break;
               // case QMessageBox::No:
                    // confirmOnlyExit = false;
                    // break;
               // case QMessageBox::Cancel:
                    // event->ignore();
                    // return;
                    // break;
               // default:
                    // event->accept();
          // }
     // }

     // Check file modyfication.
     if((currentMode() == XTT_MODE) && (XTT_Changed()))
     {
          switch(QMessageBox::question(NULL, "HQEd", "Current XTT project has been changed. Do you want to save changes?", QMessageBox::Yes | QMessageBox::No | QMessageBox::Cancel))
          {
               case QMessageBox::Yes:
                    save();
                    confirmOnlyExit = false;
                    break;
               case QMessageBox::No:
                    confirmOnlyExit = false;
                    break;
               case QMessageBox::Cancel:
                    event->ignore();
                    return;
                    break;
               default:
                    event->accept();
          }
     }

     // Confirm exit from application.
     QString msgCaption = "HQEd";

     if((confirmOnlyExit) && (QMessageBox::question(this, msgCaption, "Do you really want to exit?", QMessageBox::Yes | QMessageBox::No) == QMessageBox::No))
     {
          event->ignore();
          return;
     }

     // Hide icon.
     trayIcon->hide();

     // End application.
     QCoreApplication::exit();
}
// -----------------------------------------------------------------------------

// #brief Switch color of model (colour, black and white).
//
// #return No return value.
void MainWin_ui::SwitchColorMode(void)
{
    // Refresh window.
     Settings->setColorMode(actionB_W_view->isChecked(), currentMode());

     if(appMode == ARD_MODE)
     {
          ardPropertyList->tphTree()->refresh();
          ardLevelList->gLevelList()->alignLevels(0, true);
     }
     if(appMode == XTT_MODE)
     {
          for(unsigned int i=0;i<ConnectionsList->Count();i++)
               ConnectionsList->Connection(i)->Repaint();
          TableList->RefreshAll();
     }
}
// -----------------------------------------------------------------------------

// #brief Set antialiasing status.
//
// #param isON Determines whether antiasliasing in on.
// #return No return value.
void MainWin_ui::setAntialiasingStatus(bool isON)
{
     actionAntialiasing->setChecked(isON);
     UpdateRenderHints();
}
// -----------------------------------------------------------------------------

// #brief Set antialiasing status for text.
//
// #param isON Determines whether antiasliasing in on.
// #return No return value.
void MainWin_ui::setTextAntialiasingStatus(bool isON)
{
     actionTextAntialiasing->setChecked(isON);
     UpdateRenderHints();
}
// -----------------------------------------------------------------------------

// #brief Set smooth pixel map tranformation status.
//
// #param isON Determines whether smooth pixel map tranformation status is active.
// #return No return value.
void MainWin_ui::setSmoothPixmapTransformStatus(bool isON)
{
     actionSmooth_pixmap_transform->setChecked(isON);
     UpdateRenderHints();
}
// -----------------------------------------------------------------------------

// #brief Set color mode.
//
// #param isBW Determines whether black and white model should be visible.
// #return No return value.
void MainWin_ui::setColorMode(bool isBW)
{
     actionB_W_view->setChecked(isBW);
     SwitchColorMode();
}
// -----------------------------------------------------------------------------

// #brief Delete connection.
//
// #param cID Id connection.
// #return No return value.
void MainWin_ui::deleteConnection(QString cID)
{
     int cIndex = ConnectionsList->IndexOfID(cID);

     if(cIndex == -1)
          return;

     if(QMessageBox::question(NULL, "HQEd", "Do you really want to delete selected connection?", QMessageBox::Yes | QMessageBox::No) == QMessageBox::No)
          return;

     // Jezeli wybrane bylo zaznaczone
     if(xtt_scene->SelectedConnectionID() == cID)
          xtt_scene->setSelectedConnectionID("");

     // Delete connection.
     ConnectionsList->Delete(cIndex);
}
// -----------------------------------------------------------------------------

// #brief Delete table.
//
// #param tID Table id.
// #return No return value.
void MainWin_ui::deleteTable(QString tID)
{
     int tIndex = TableList->IndexOfID(tID);

     if(tIndex == -1)
          return;

     // Potwierdzenie usuniecia
     if(QMessageBox::question(NULL, "HQEd", "Do you really want to remove selected table?", QMessageBox::Yes | QMessageBox::No) == QMessageBox::No)
          return;

     // Jezeli wybrana byla zaznaczona
     if(xtt_scene->SelectedTableID() == tID)
          xtt_scene->setSelectedTableID("");

     // Usuwanie
     TableList->Delete(tIndex);
}
// -----------------------------------------------------------------------------

// #brief Delete selected table.
//
// #return No return value.
void MainWin_ui::deleteSelectedTable(void)
{
     xtt_scene->setSelectedTableID("");
     deleteTable(xtt_scene->SelectedTableID());
}
// -----------------------------------------------------------------------------

// #brief Delete selected connection.
//
// #return No return value.
void MainWin_ui::deleteSelectedConnection(void)
{
     xtt_scene->setSelectedConnectionID("");
     deleteConnection(xtt_scene->SelectedConnectionID());
}
// -----------------------------------------------------------------------------

// #brief Function is triggered when key is pressed.
//
// #param event Pointer to event object.
// #return No return value.
void MainWin_ui::keyPressEvent(QKeyEvent *event)
{
     if(appMode == ARD_MODE)
     {
          switch(event->key())
          {
               case Qt::Key_F5:
                    fCurrentARDscene->AdaptSceneSize();
                    break;

               case Qt::Key_F9:
                    break;

               case Qt::Key_Insert:
                    break;

               case Qt::Key_Delete:
                    break;

               case Qt::Key_Plus:
                    fCurrentARDscene->zin();
                    break;

               case Qt::Key_Minus:
                    fCurrentARDscene->zout();
                    break;

               default:
                    QWidget::keyPressEvent(event);
          }
     }

     if(appMode == XTT_MODE)
     {
          switch(event->key())
          {
               case Qt::Key_F5:
                    xtt_scene->AdaptSceneSize();
                    break;

               case Qt::Key_F9:
                    ExecuteModel();
                    break;

               case Qt::Key_Insert:
                    AddNewAttribute();
                    break;

               case Qt::Key_Delete:
                    if((xtt_scene->SelectedTableID() != "") && (TableList->Count() > 0))
                         deleteTable(xtt_scene->SelectedTableID());
                    else if((xtt_scene->SelectedConnectionID() != "") && (ConnectionsList->Count() > 0))
                         deleteConnection(xtt_scene->SelectedConnectionID());
                    xtt::fullVerification();
                    break;

               case Qt::Key_Plus:
                    xtt_scene->zin();
                    break;

               case Qt::Key_Minus:
                    xtt_scene->zout();
                    break;

               default:
                    QWidget::keyPressEvent(event);
          }
     }
}
// -----------------------------------------------------------------------------

// #brief Change antialiasing status.
//
// #return No return value.
void MainWin_ui::ChangeAntialiasingStatus(void)
{
     QPainter::RenderHints flags = 0;
     if(actionAntialiasing->isChecked())
          flags = flags | QPainter::Antialiasing;
     if(actionTextAntialiasing->isChecked())
          flags = flags | QPainter::TextAntialiasing;
     if(actionSmooth_pixmap_transform->isChecked())
          flags = flags | QPainter::SmoothPixmapTransform;

     xtt_graphicsView->setRenderHints(flags);
}
// -----------------------------------------------------------------------------

// #brief Update render hints.
//
// #return No return value.
void MainWin_ui::UpdateRenderHints(void)
{
     QPainter::RenderHints flags = 0;
     if(actionAntialiasing->isChecked())
          flags = flags | QPainter::Antialiasing;
     if(actionTextAntialiasing->isChecked())
          flags = flags | QPainter::TextAntialiasing;
     if(actionSmooth_pixmap_transform->isChecked())
          flags = flags | QPainter::SmoothPixmapTransform;

     Settings->setAntialiasingMode(actionAntialiasing->isChecked(), currentMode());
     Settings->setTextAntialiasingMode(actionTextAntialiasing->isChecked(), currentMode());
     Settings->setSmoothPixmapTransformMode(actionSmooth_pixmap_transform->isChecked(), currentMode());

     if(appMode == ARD_MODE)
     {
          QList<QGraphicsView*> views_list = ard_modelScene->views();
          for(int i=0;i<views_list.count();i++)
               views_list.at(i)->setRenderHints(flags);

          views_list = ard_thpScene->views();
          for(int i=0;i<views_list.count();i++)
               views_list.at(i)->setRenderHints(flags);
     }

     if(appMode == XTT_MODE)
     {
          QList<QGraphicsView*> views_list = xtt_scene->views();
          for(int i=0;i<views_list.count();i++)
               views_list.at(i)->setRenderHints(flags);
     }
}
// -----------------------------------------------------------------------------

// #brief Show configuration dialog.
//
// #return No return value.
void MainWin_ui::showConfigurationDialog(void)
{
     Settings->show();
}
// -----------------------------------------------------------------------------

// #brief Show window where you can add new attribute.
//
// #return No return value.
void MainWin_ui::AddNewAttribute(void)
{
     delete AttributeEditor;
     AttributeEditor = new AttrEditor(true, 0, -1);
     AttributeEditor->show();
}
// -----------------------------------------------------------------------------

// #brief Show window where you can edit attribute.
//
// #return No return value.
void MainWin_ui::EditAttribute(void)
{
     if(AttributeGroups->IsEmpty())
     {
          const QString str2 = "There is no attributes defined yet.";
          QMessageBox::information(NULL, "HQEd", str2, QMessageBox::Ok);
          return;
     }

     delete AttributeEditor;
     AttributeEditor = new AttrEditor(false, 0, -1);
     AttributeEditor->show();
}
// -----------------------------------------------------------------------------

// #brief Show window where you can delete attribute.
//
// #return No return value.
void MainWin_ui::DeleteAttribute(void)
{
     if(AttributeGroups->IsEmpty())
     {
          const QString str2 = "There is no attributes defined yet.";
          QMessageBox::information(NULL, "HQEd", str2, QMessageBox::Ok);
          return;
     }

     delete AttributeDelete;
     AttributeDelete = new AttributeDelete_ui(0);
     AttributeDelete->show();
}
// -----------------------------------------------------------------------------

// #brief Show group menager window.
//
// #return No return value.
void MainWin_ui::GroupManagerShow(void)
{
     delete GroupMan;
     GroupMan = new GroupManager_ui;
     GroupMan->show();
}
// -----------------------------------------------------------------------------

// #brief Show attribute menager window.
//
// #return No return value.
void MainWin_ui::AttributeManagerShow(void)
{
     delete AttrManager;
     AttrManager = new AttributeManager_ui;
     AttrManager->show();
}
// -----------------------------------------------------------------------------

// #brief Show table editor window.
//
// #return No return value.
void MainWin_ui::TableEditorShow(void)
{
     delete TableEdit;
     TableEdit = new TableEditor_ui(true, 0);
     TableEdit->show();
}
// -----------------------------------------------------------------------------

// #brief Show table editor window in edit mode.
//
// #return No return value.
void MainWin_ui::TableEditorEdit(void)
{
     if(TableList->IsEmpty())
     {
          const QString str2 = "There is no tables defined yet.";
          QMessageBox::information(NULL, "HQEd", str2, QMessageBox::Ok);
          return;
     }

     // Wyszukiwanie indexu zaznaczonej tabeli
     int tindex = TableList->IndexOfID(xtt_scene->SelectedTableID());
     if(tindex == -1)
          tindex = 0;

     delete TableEdit;
     TableEdit = new TableEditor_ui(false, tindex);
     TableEdit->show();
}
// -----------------------------------------------------------------------------

// #brief Show delete table window.
//
// #return No return value.
void MainWin_ui::DeleteTableShow(void)
{
     if(TableList->IsEmpty())
     {
          const QString str2 = "There is no tables defined yet.";
          QMessageBox::information(NULL, "HQEd", str2, QMessageBox::Ok);
          return;
     }

     // Wyszukiwanie indexu zaznaczonej tabeli
     int tindex = TableList->IndexOfID(xtt_scene->SelectedTableID());
     if(tindex == -1)
          tindex = 0;

     delete DelTable;
     DelTable = new DeleteTable_ui(tindex, 0);
     DelTable->show();
}
// -----------------------------------------------------------------------------

// #brief Performs tables autoarrangment procedure
//
// #return No return value.
void MainWin_ui::tablesArrangment(void)
{
     xtt_scene->AdaptSceneSize();
     TableList->tablesLayout();
     xtt_scene->AdaptSceneSize();
}
// -----------------------------------------------------------------------------

// #brief Show cell editor window.
//
// #return No return value.
void MainWin_ui::ShowCellEditor(void)
{
     if(TableList->IsEmpty())
     {
          const QString str2 = "There is no tables defined yet.";
          QMessageBox::information(NULL, "HQEd", str2, QMessageBox::Ok);
          return;
     }

     // Wyszukiwanie indexu zaznaczonej tabeli
     int tindex = TableList->IndexOfID(xtt_scene->SelectedTableID());
     if(tindex == -1)
          tindex = 0;

     delete CellEdit;
     CellEdit = new CellEditor_ui(tindex, 0);
     CellEdit->show();
}
// -----------------------------------------------------------------------------

// #brief Show connetion editor window.
//
// #return No return value.
void MainWin_ui::ShowConnEditor(void)
{
     // Wyszukiwanie indexu zaznaczonej tabeli
     int tindex = TableList->IndexOfID(xtt_scene->SelectedTableID());
     if(tindex == -1)
          tindex = 0;

     delete ConnEdit;
     ConnEdit = new ConnectionEditor_ui(true, tindex);
     ConnEdit->show();
}
// -----------------------------------------------------------------------------

// #brief Show connetion editor window in edit mode.
//
// #return No return value.
void MainWin_ui::ShowConnEditorInEditMode(void)
{
     if(ConnectionsList->Count() == 0)
     {
          const QString str2 = "There is no connections defined yet.";
          QMessageBox::information(NULL, "HQEd", str2, QMessageBox::Ok);
          return;
     }

     // Wyszukiwanie indexu zaznaczonego polaczenia
     int tindex = ConnectionsList->IndexOfID(xtt_scene->SelectedConnectionID());
     if(tindex == -1)
          tindex = 0;

     delete ConnEdit;
     ConnEdit = new ConnectionEditor_ui(false, tindex);
     ConnEdit->show();
}
// -----------------------------------------------------------------------------

// #brief Show delete connetion window.
//
// #return No return value.
void MainWin_ui::ShowConnectionDeleteWindow(void)
{
     if(ConnectionsList->Count() == 0)
     {
          const QString str2 = "There is no connections defined yet.";
          QMessageBox::information(NULL, "HQEd", str2, QMessageBox::Ok);
          return;
     }

     // Wyszukiwanie indexu zaznaczonej tabeli
     int conn_ind = ConnectionsList->IndexOfID(xtt_scene->SelectedConnectionID());
     if(conn_ind == -1)
          conn_ind = 0;
     QString tid = ConnectionsList->Connection(conn_ind)->SrcTable();
     QString rid = ConnectionsList->Connection(conn_ind)->SrcRow();

     int tindex = TableList->IndexOfID(tid);
     int rindex = TableList->IndexOfID(tid);
     if(tindex == -1)
     {
          QMessageBox::critical(NULL, "HQEd", "Table with ID = " + tid + " not exist.", QMessageBox::Ok);
          return;
     }
     // Wyszukiwanie wiersza o podanym ID
     for(unsigned int r=0;r<TableList->Table(tindex)->RowCount();r++)
          if(TableList->Table(tindex)->Row(r)->RowId() == rid)
          {
               rindex = r;
               break;
          }
     if(rindex == -1)
     {
          QMessageBox::critical(NULL, "HQEd", "Row with ID = " + rid + " not exist in table with ID = " + tid + ".", QMessageBox::Ok);
          return;
     }

     delete DelConn;
     DelConn = new DeleteConnection_ui(tindex, rindex, conn_ind, 0);
     DelConn->show();
}
// -----------------------------------------------------------------------------

// #brief Show connetion menager window.
//
// #return No return value.
void MainWin_ui::ShowConnectionManager(void)
{
     if(TableList->IsEmpty())
     {
          const QString str2 = "There is no tables defined yet.";
          QMessageBox::information(NULL, "HQEd", str2, QMessageBox::Ok);
          return;
     }

     // Wyszukiwanie indexu zaznaczonej tabeli
     int tindex = TableList->IndexOfID(xtt_scene->SelectedTableID());
     if(tindex == -1)
          tindex = 0;

     delete ConnManager;
     ConnManager = new ConnectionManager_ui(tindex, 0);
     ConnManager->show();
}
// -----------------------------------------------------------------------------

// #brief Open type manager
//
// #return No return value.
void MainWin_ui::typeManagerShow(void)
{
     delete TypeManager;
     TypeManager = new TypeManager_ui;
     TypeManager->show();
}
// -----------------------------------------------------------------------------

// #brief Change angle editor.
//
// #param _newValue New angle.
// #return No return value.
void MainWin_ui::angleEditorChange(int _newValue)
{
     dial->setValue(_newValue);
     if(appMode == ARD_MODE)
          fCurrentARDscene->Rotate(_newValue);
     if(appMode == XTT_MODE)
          xtt_scene->Rotate(_newValue);
}
// -----------------------------------------------------------------------------

// #brief Reset model rotate.
//
// #return No return value.
void MainWin_ui::resetRotate(void)
{
     dial->setValue(0);
}
// -----------------------------------------------------------------------------

// #brief Show list of unused attributes.
//
// #return No return value.
void MainWin_ui::ShowUnusedAttributes(void)
{
     if(AttributeGroups->IsEmpty())
     {
          const QString str2 = "There is no attributes defined yet.";
          QMessageBox::information(NULL, "HQEd", str2, QMessageBox::Ok);
          return;
     }

     delete UnusedAttr;
     UnusedAttr = new UnusedAtrtribute_ui;
     UnusedAttr->show();
}
// -----------------------------------------------------------------------------

// #brief Show program information.
//
// #return No return value.
void MainWin_ui::ShowAboutWin(void)
{
     //delete AboutWin;
     //AboutWin = new About_ui;
     AboutWin->show();
}
// -----------------------------------------------------------------------------

// #brief Check ARD model changes.
//
// #return True when model is changed, otherwise false.
bool MainWin_ui::ARD_Changed(void)
{
     return ardAttributeList->changed() || ardPropertyList->changed() || ardDependencyList->changed();
}
// -----------------------------------------------------------------------------

// #brief Check XTT model changes.
//
// #return True when model is changed, otherwise false.
bool MainWin_ui::XTT_Changed(void)
{
     return AttributeGroups->Changed() || ConnectionsList->Changed() || TableList->Changed();
}
// -----------------------------------------------------------------------------

// #brief Set status of modyfication ARD model.
//
// #param _c Status of modyfication.
// #return No return value.
void MainWin_ui::setARD_Changed(bool _c)
{
     ardAttributeList->setChanged(_c);
     ardPropertyList->setChanged(_c);
     ardDependencyList->setChanged(_c);
}
// -----------------------------------------------------------------------------

// #brief Set status of modyfication XTT model.
//
// #param _c Status of modyfication.
// #return No return value.
void MainWin_ui::setXTT_Changed(bool _c)
{
     ConnectionsList->setChanged(_c);
     TableList->setChanged(_c);
     AttributeGroups->setChanged(_c);
}
// -----------------------------------------------------------------------------

// #brief Create new ARD project.
//
// #return 0 - Cancel, 1 - Yes, -1 - No.
int MainWin_ui::CreateNewARDProject(void)
{
     QApplication::restoreOverrideCursor();

     int res = 1;
     // if(ARD_Changed())
     // {
          // switch(QMessageBox::question(NULL, "HQEd", "Current project has been changed. Do you want to save changes?", QMessageBox::Yes | QMessageBox::No | QMessageBox::Cancel))
          // {
               // case QMessageBox::Yes:
                    // saveToXTTML1_0();
                    // res = 1;
                    // break;
               // case QMessageBox::No:
                    // res = -1;
                    // break;
               // case QMessageBox::Cancel:
                    // return 0;
                    // break;
               // default:
                    // return 0;
          // }
     // }

     QApplication::setOverrideCursor(Qt::WaitCursor);
     fArdFileName = "New project";
     setWindowTitle("ARD Editor - [" + fArdFileName + "]");
     setARD_Changed(false);

     // Kasowanie list elementow
     ard::clear();
     isARDFile = false;

     QApplication::restoreOverrideCursor();

     return res;
}
// -----------------------------------------------------------------------------

// #brief Create new XTT project.
//
// #param _keepAttributes Determines whether leave argument list.
// #return 0 - Cancel, 1 - Yes, -1 - No.
int MainWin_ui::CreateNewXTTProject(bool _keepAttributes)
{
     QApplication::restoreOverrideCursor();

     int res = 1;
     if(XTT_Changed())
     {
          switch(QMessageBox::question(NULL, "HQEd", "Current project has been changed. Do you want to save changes?", QMessageBox::Yes | QMessageBox::No | QMessageBox::Cancel))
          {
               case QMessageBox::Yes:
                    save();
                    res = 1;
                    break;
               case QMessageBox::No:
                    res = -1;
                    break;
               case QMessageBox::Cancel:
                    return 0;
                    break;
               default:
                    return 0;
          }
     }

     QApplication::setOverrideCursor(Qt::WaitCursor);
     fXttFileName = "New project";
     setWindowTitle("HQEd - [" + fXttFileName + "]");
     setXTT_Changed(false);

     xtt_scene->setSelectedConnectionID("");
     xtt_scene->setSelectedTableID("");
     xtt_runModelMessagesDockWindowMessagesList->deleteAll();
     xtt_errorsList->deleteAll();

     xtt::clear(_keepAttributes);
     isXTTFile = false;

     QApplication::restoreOverrideCursor();

     return res;
}
// -----------------------------------------------------------------------------

// #brief Create new project.
//
// #return No return value.
void MainWin_ui::NewClick(void)
{
     if(currentMode() == ARD_MODE)
          CreateNewARDProject();
     if(currentMode() == XTT_MODE)
          CreateNewXTTProject(false);
}
// -----------------------------------------------------------------------------

// #brief Create new project with the same attributes.
//
// #return No return value.
void MainWin_ui::NewWhithOldAttributiesClick(void)
{
     CreateNewXTTProject(true);
}
// -----------------------------------------------------------------------------

// #brief Open XTT file dialog.
//
// #return No return value.
void MainWin_ui::openXTTFileDialog(void)
{
     // Ustalanie nazwy pliku
     QString fileName = QFileDialog::getOpenFileName(this,
                                 tr("Open project"),
                                 QDir::currentPath(),
                                 tr("HML files (*.hml);;XTTML files (*.xttml);;ATTML files (*.attml);;All files (*)"));

     openFile(fileName);
}
// -----------------------------------------------------------------------------

// #brief Open ARD file dialog.
//
// #return No return value.
void MainWin_ui::openARDFileDialog(void)
{
     // Ustalanie nazwy pliku
     QString fileName = QFileDialog::getOpenFileName(this,
                                 tr("Open project"),
                                 QDir::currentPath(),
                                 tr("HML files (*.hml);;ARDML files (*.ardml);;All files (*)"));

     openFile(fileName);
}
// -----------------------------------------------------------------------------

// #brief Open file.
//
// #param _fileName File name to open.
// #return No return value.
void MainWin_ui::openFile(QString _fileName)
{
     // Gdy nie wybrano zandego pliku
     if(_fileName.isEmpty())
          return;

     if(!QFile::exists(_fileName))
     {
          QMessageBox::critical(this, "HQEd", "File " + _fileName + " does not exists.");
          return;
     }

     // Czytanie pliku
     QFile file(_fileName);
     if(!file.open(QFile::ReadOnly | QFile::Text))
     {
          QMessageBox::critical(this, tr("HQEd"), tr("Cannot read file %1:\n%2.").arg(_fileName).arg(file.errorString()));
          return;
     }

     QString msg = "", format = "";
     QApplication::setOverrideCursor(Qt::WaitCursor);
     int result = vas_in_XML(&file, msg, format);
     QApplication::restoreOverrideCursor();

     if(result == VAS_XML_UNKNOWN_DEVICE)
          QMessageBox::critical(this, "HQEd", "Loading error: device is not specified.");
     if(result == VAS_XML_DEVICE_NOT_READABLE)
          QMessageBox::critical(this, "HQEd", "Loading error: device is not readable.");
     if(result == VAS_XML_FORMAT_ERROR)
          QMessageBox::critical(this, "HQEd", "Loading error: " + hqed::retrieveMessageParts(msg));
     if(result == VAS_XML_FORMAT_NOT_SUPPORTED)
          QMessageBox::critical(this, "HQEd", "Loading error: the file format is not supported.");
     if(result == VAS_XML_VERSION_NOT_SUPPORTED)
          QMessageBox::critical(this, "HQEd", "Loading error: the xml version is not supported or not specified.");
     if(result == VAS_XML_XTT_SYNTAX_ERROR)
          hqed::showMessages(msg);
     if(result == VAS_XML_NOTHING_DONE)
          return;
          
     // Information about loading finish
     //if(result == VAS_XML_SUCCES)       // When no errors occur
     QMessageBox::information(this, "HQEd", "Loading finished.");

     // Gdy plik typu xttml
     if(format == "xttml2.0")
          onLoad_XTTML_2_0_file(_fileName);

     // Gdy plik typu ardml
     if(format == "ardml1.0")
          onLoad_ARDML_1_0_file(_fileName);

     // Gdy plik typu hml1.0
     if(format == "hml1.0")
          onLoad_HML_1_0_file(_fileName);

     // Gdy plik typu hml2.0
     if(format == "hml2.0")
          onLoad_HML_2_0_file(_fileName, msg);
}
// -----------------------------------------------------------------------------

// #brief Function that is called when XTTML 2.0 file has been loaded
//
// #param __sourceName - the name of the source
// #return No return value.
void MainWin_ui::onLoad_XTTML_2_0_file(QString __sourceName)
{
     setCurrentMode(XTT_MODE);
     fXttFileName = __sourceName;
     setWindowTitle("HQEd - [" + fXttFileName + "]");
     isXTTFile = true;

     // sprawdzenie modelu 2x
     xtt::fullVerification();
     xtt::fullVerification();

     // Usuniecie elementu ktory podaza za wskaznikiem myszki
     xtt_scene->resetNewItem();

     // Uaktualnienie rozmiarow sceny
     xtt_scene->update();
     xtt_scene->AdaptSceneSize();
     TableList->tablesLayout();
     xtt_scene->AdaptSceneSize();
}
// -----------------------------------------------------------------------------

// #brief Function that is called when ARDML 1.0 file has been loaded
//
// #param __sourceName - the name of the source
// #return No return value.
void MainWin_ui::onLoad_ARDML_1_0_file(QString __sourceName)
{
     onLoad_HML_1_0_file(__sourceName);
}
// -----------------------------------------------------------------------------

// #brief Function that is called when HML 1.0 file has been loaded
//
// #param __sourceName - the name of the source
// #return No return value.
void MainWin_ui::onLoad_HML_1_0_file(QString __sourceName)
{
     setCurrentMode(ARD_MODE);
     fArdFileName = __sourceName;
     setWindowTitle("HQEd - [" + fArdFileName + "]");
     isARDFile = true;

     ardLevelList->createBaseLevel();

     // Uaktualnienie rozmiarow sceny
     ard_thpScene->AdaptSceneSize();
     ard_thpScene->update();

     setARD_Changed(false);
}
// -----------------------------------------------------------------------------

// #brief Function that is called when HML 2.0 file has been loaded
//
// #param __sourceName - Reference to section with file contants.
// #param __msgs - the reading messages.
// #return No return value.
void MainWin_ui::onLoad_HML_2_0_file(QString __sourceName, QString __msgs)
{
     if(__msgs.contains("HASARD"))
          onLoad_ARDML_1_0_file(__sourceName);
     if(__msgs.contains("HASXTT"))
          onLoad_XTTML_2_0_file(__sourceName);
}
// -----------------------------------------------------------------------------

// #brief Save file as.
//
// #param varsion Filter file version.
// #return No return value.
void MainWin_ui::saveAs(int version)
{
     QStringList filters;
     filters << "HML 2.0 files (*.hml)";

     // Ustalanie nazwy pliku
     QString fileName = QFileDialog::getSaveFileName(this,
                                 tr("Save project as"),
                                 QDir::currentPath(),
                                 filters.at(version-2));

     // Gdy nie wybrano zandego pliku
     if(fileName.isEmpty())
          return;

     isXTTFile = true;

     if(version == 2)
          saveTo_HML_2_0(fileName);
}
// -----------------------------------------------------------------------------

// #brief Save changes.
//
// #return No return value.
void MainWin_ui::save(void)
{
     saveTo_HML_2_0(fXttFileName);
}
// -----------------------------------------------------------------------------

// #brief Executes action of saving file to hml version 2.0.
//
// #return No return value.
void MainWin_ui::saveAs_HML_2_0(void)
{
     saveAs(2);
}
// -----------------------------------------------------------------------------

// #brief Save file to format xttml 2.0.
//
// #param __fileName - the destination file name
// #return No return value.
void MainWin_ui::saveTo_HML_2_0(QString __fileName)
{
     if(!isXTTFile)
     {
          saveAs(2);
          return;
     }

     // Changing the file extesion
     QString fName = __fileName;
     QFileInfo fi(fName);
     QString ext = fi.suffix();
     if(ext.toLower() != "hml")
     {
          fName.remove(ext, Qt::CaseInsensitive);
          fName += "hml";
     }

     // Zapis danych do pliku
     QFile file(fName);
     if(!file.open(QFile::WriteOnly | QFile::Text))
     {
         QMessageBox::warning(this, tr("HQEd"), tr("Cannot write file %1:\n%2.").arg(fName).arg(file.errorString()));
         return;
     }

     QApplication::setOverrideCursor(Qt::WaitCursor);
     int result = vas_out_HML_2_0(&file, VAS_XML_MODEL_PARTS_XTT_TYPES | VAS_XML_MODEL_PARTS_XTT_ATTRS | VAS_XML_MODEL_PARTS_XTT_TABLS | VAS_XML_MODEL_PARTS_XTT_STATS);
     QApplication::restoreOverrideCursor();

     if(result == VAS_XML_UNKNOWN_DEVICE)
          QMessageBox::critical(this, "HQEd", "Saving error: device is not specified.");
     if(result == VAS_XML_DEVICE_NOT_WRITABLE)
          QMessageBox::critical(this, "HQEd", "Saving error: device is not writable.");

     // Zapisanie stanu statusu modyfikacji modelu XTT
     setXTT_Changed(result != VAS_XML_SUCCES);

     fXttFileName = fName;
     setWindowTitle("HQEd - [" + fXttFileName + "]");
}
// -----------------------------------------------------------------------------

// #brief Export attributes to HML 2.0 file.
//
// #return No return value.
void MainWin_ui::exportAttributesToHML20file(void)
{
     QStringList filters;
     filters << "HML 2.0 files (*.hml)";

     // Ustalanie nazwy pliku
     QString fileName = QFileDialog::getSaveFileName(this,
                                 tr("Export atrributes to HML 2.0 format"),
                                 QDir::currentPath(),
                                 filters.at(0));

     // Gdy nie wybrano zandego pliku
     if(fileName.isEmpty())
          return;

     // Export code
     QFile file(fileName);
     if(!file.open(QFile::WriteOnly | QFile::Text))
     {
         QMessageBox::warning(this, tr("HQEd"), tr("Cannot write file %1:\n%2.").arg(fXttFileName).arg(file.errorString()));
         return;
     }

     QApplication::setOverrideCursor(Qt::WaitCursor);
     int result = vas_out_HML_2_0(&file, VAS_XML_MODEL_PARTS_XTT_TYPES | VAS_XML_MODEL_PARTS_XTT_ATTRS);
     QApplication::restoreOverrideCursor();

     if(result == VAS_XML_UNKNOWN_DEVICE)
          QMessageBox::critical(this, "HQEd", "Saving error: device is not specified.");
     if(result == VAS_XML_DEVICE_NOT_WRITABLE)
          QMessageBox::critical(this, "HQEd", "Saving error: device is not writable.");
}
// -----------------------------------------------------------------------------

// #brief Export scene to svg file.
//
// #return No return value.
void MainWin_ui::exportSceneToSVGfile(void)
{
#if QT_VERSION >= 0x040300

     QStringList filters;
     filters      << "SVG files (*.svg)";

     // Ustalanie nazwy pliku
     QString fileName = QFileDialog::getSaveFileName(this,
                                 tr("Export scene picture"),
                                 QDir::currentPath(),
                                 filters.at(0));
     if(fileName.isEmpty())
          return;

     QFile dev(fileName);
     if(!dev.open(QIODevice::WriteOnly /*| QIODevice::Text*/))
     {
          QMessageBox::critical(this, "HQEd", "Export error: cannot open file.");
          return;
     }
     QGraphicsScene* scene;

     // Gdy tryb ARD
     if(appMode == ARD_MODE)
     {
          ard_modelScene->AdaptSceneSize();
          scene = ard_modelScene;
     }

     // Gdy tryb XTT
     if(appMode == XTT_MODE)
     {
          xtt_scene->AdaptSceneSize();
          scene = xtt_scene;
     }

     int result = vasSVG(&dev, scene);
     if(result == VAS_PICTURE_UNKNOWN_DEVICE)
          QMessageBox::critical(this, "HQEd", "Export error: device is not specified.");
     if(result == VAS_PICTURE_DEVICE_NOT_WRITABLE)
          QMessageBox::critical(this, "HQEd", "Export error: device is not writable.");

#else
     QMessageBox::information(this, "HQEd", "No SVG supported");
#endif
}
// -----------------------------------------------------------------------------

// #brief Export scene as a picture.
//
// #param _format File format.
// #return No return value.
void MainWin_ui::exportSceneAsPicture(QString _format)
{
     QStringList formats, hints, filters;
     formats << "BMP" << "JPG" << "PNG" << "PPM" << "XBM" << "XPM";
     hints << "Windows Bitmap" << "Joint Photographic Experts Group" << "Portable Network Graphics" << "Portable Pixmap" << "X11 Bitmap" << "X11 Pixmap";
     int imageCompression = 100-Settings->xttImageCompression();
     if(appMode == ARD_MODE)
          imageCompression = 100-Settings->ardImageCompression();

     QString msgTitle = "HQEd";

     int fi = formats.indexOf(_format.toUpper());
     if(fi == -1)
     {
          QMessageBox::information(this, msgTitle, "Unknown file format " + _format + ".");
          return;
     }

     filters << hints.at(fi) + " files (*." + _format.toLower() + ")";

     // Ustalanie nazwy pliku
     QString fileName = "";
     fileName = QFileDialog::getSaveFileName(this,
                       tr("Export scene picture"),
                       QDir::currentPath(),
                       filters.at(0));

     if(fileName.isEmpty())
          return;

     QFile dev(fileName);
     if(!dev.open(QIODevice::WriteOnly))
     {
          QMessageBox::critical(this, "HQEd", "Export error: cannot open file.");
          return;
     }
     QGraphicsScene* scene;

     // Gdy tryb ARD
     if(appMode == ARD_MODE)
     {
          ard_modelScene->AdaptSceneSize();
          scene = ard_modelScene;
     }

     // Gdy tryb XTT
     if(appMode == XTT_MODE)
     {
          xtt_scene->AdaptSceneSize();
          scene = xtt_scene;
     }

     int result = vasPicture(&dev, scene, _format, imageCompression);
     if(result == VAS_PICTURE_UNKNOWN_FORMAT)
          QMessageBox::critical(this, "HQEd", "Export error: the given format is not supported.");
     if(result == VAS_PICTURE_NOT_SAVED)
          QMessageBox::critical(this, "HQEd", "Export error: saving error.");
     if(result == VAS_PICTURE_UNKNOWN_DEVICE)
          QMessageBox::critical(this, "HQEd", "Export error: device is not specified.");
     if(result == VAS_PICTURE_DEVICE_NOT_WRITABLE)
          QMessageBox::critical(this, "HQEd", "Export error: device is not writable.");
}
// -----------------------------------------------------------------------------

// #brief Export scene to png file.
//
// #return No return value.
void MainWin_ui::exportSceneToPNGfile(void)
{
     exportSceneAsPicture("PNG");
}
// -----------------------------------------------------------------------------

// #brief Export scene to bmp file.
//
// #return No return value.
void MainWin_ui::exportSceneToBMPfile(void)
{
     exportSceneAsPicture("BMP");
}
// -----------------------------------------------------------------------------

// #brief Export scene to jpg file.
//
// #return No return value.
void MainWin_ui::exportSceneToJPGfile(void)
{
     exportSceneAsPicture("JPG");
}
// -----------------------------------------------------------------------------

// #brief Export scene to ppm file.
//
// #return No return value.
void MainWin_ui::exportSceneToPPMfile(void)
{
     exportSceneAsPicture("PPM");
}
// -----------------------------------------------------------------------------

// #brief Export scene to xbm file.
//
// #return No return value.
void MainWin_ui::exportSceneToXBMfile(void)
{
     exportSceneAsPicture("XBM");
}
// -----------------------------------------------------------------------------

// #brief Export scene to xpm file.
//
// #return No return value.
void MainWin_ui::exportSceneToXPMfile(void)
{
     exportSceneAsPicture("XPM");
}
// -----------------------------------------------------------------------------

// #brief Export tph scene to svg file.
//
// #return No return value.
void MainWin_ui::exportTPHtoSVGfile(void)
{
#if QT_VERSION >= 0x040300

     QStringList filters;
     filters      << "SVG files (*.svg)";

     // Ustalanie nazwy pliku
     QString fileName = QFileDialog::getSaveFileName(this,
                                 tr("Export scene picture"),
                                 QDir::currentPath(),
                                 filters.at(0));
     if(fileName.isEmpty())
          return;

     QFile dev(fileName);
     if(!dev.open(QIODevice::WriteOnly | QIODevice::Text))
     {
          QMessageBox::critical(this, "HQEd", "Export error: cannot open file.");
          return;
     }
     QGraphicsScene* scene;

     ard_thpScene->AdaptSceneSize();
     scene = ard_thpScene;

     int result = vasSVG(&dev, scene);
     if(result == VAS_PICTURE_UNKNOWN_DEVICE)
          QMessageBox::critical(this, "HQEd", "Export error: device is not specified.");
     if(result == VAS_PICTURE_DEVICE_NOT_WRITABLE)
          QMessageBox::critical(this, "HQEd", "Export error: device is not writable.");

#else
     QMessageBox::information(this, "HQEd", "No SVG supported");
#endif
}
// -----------------------------------------------------------------------------

// #brief Export tph as picture.
//
// #param _format File format.
// #return No return value.
void MainWin_ui::exportTPHasPicture(QString _format)
{
     QStringList formats, hints, filters;
     formats << "BMP" << "JPG" << "PNG" << "PPM" << "XBM" << "XPM";
     hints << "Windows Bitmap" << "Joint Photographic Experts Group" << "Portable Network Graphics" << "Portable Pixmap" << "X11 Bitmap" << "X11 Pixmap";
     char* cFormats[] = {"BMP", "JPG", "PNG", "PPM", "XBM", "XPM"};
     int imageCompression = 100-Settings->ardImageCompression();

     int fi = formats.indexOf(_format.toUpper());
     if(fi == -1)
     {
          QMessageBox::information(this, "HQEd", "Unknown file format " + _format + ".");
          return;
     }

     filters << hints.at(fi) + " files (*." + _format.toLower() + ")";

     // Ustalanie nazwy pliku
     QString fileName = "";
     fileName = QFileDialog::getSaveFileName(this,
                       tr("Export scene picture"),
                       QDir::currentPath(),
                       filters.at(0));

     if(fileName == "")
          return;

     // Obszar sceny do zrzutu
     QRect srect;

     // Poprawienie rozmiaru sceny, tak aby plik svg mial odpowiedni rozmiar

     // Gdy tryb ARD
     ard_thpScene->AdaptSceneSize();

     // Zapisanie wymiarow sceny
     srect.setX((int)ard_modelScene->sceneRect().x());
     srect.setY((int)ard_modelScene->sceneRect().y());
     srect.setWidth((int)ard_modelScene->sceneRect().width());
     srect.setHeight((int)ard_modelScene->sceneRect().height());

     // Tworzenie pliku graficznego
     QPainter painter;
     QPixmap* pixmap = new QPixmap(srect.width(), srect.height());

     painter.begin(pixmap);

     painter.setPen(QColor(Qt::black));
     painter.setBrush(QBrush(Qt::white, Qt::SolidPattern));
     painter.drawRect(pixmap->rect());

     ard_thpScene->render(&painter);

     painter.end();

     if(!pixmap->save(fileName, cFormats[fi], imageCompression))
          QMessageBox::information(this, "HQEd", "File not saved correctly.");
}
// -----------------------------------------------------------------------------

// #brief Export tph scene to png file.
//
// #return No return value.
void MainWin_ui::exportTPHtoPNGfile(void)
{
     exportTPHasPicture("PNG");
}
// -----------------------------------------------------------------------------

// #brief Export tph scene to bmp file.
//
// #return No return value.
void MainWin_ui::exportTPHtoBMPfile(void)
{
     exportTPHasPicture("BMP");
}
// -----------------------------------------------------------------------------

// #brief Export tph scene to jpg file.
//
// #return No return value.
void MainWin_ui::exportTPHtoJPGfile(void)
{
     exportTPHasPicture("JPG");
}
// -----------------------------------------------------------------------------

// #brief Export tph scene to ppm file.
//
// #return No return value.
void MainWin_ui::exportTPHtoPPMfile(void)
{
     exportTPHasPicture("PPM");
}
// -----------------------------------------------------------------------------

// #brief Export tph scene to xbm file.
//
// #return No return value.
void MainWin_ui::exportTPHtoXBMfile(void)
{
     exportTPHasPicture("XBM");
}
// -----------------------------------------------------------------------------

// #brief Export tph scene to xpm file.
//
// #return No return value.
void MainWin_ui::exportTPHtoXPMfile(void)
{
     exportTPHasPicture("XPM");
}
// -----------------------------------------------------------------------------

// #brief Performs XTT+ model generation based on ARD+ model
//
// #return No return value.
void MainWin_ui::generateXTT(void)
{
     QString msgs[] = {"XTT generation succes",
                       "No ARD levels are defined",
                       "The last ARD level is not finalized"
                         };

     QApplication::setOverrideCursor(Qt::WaitCursor);
     int res = hqed::generateXTT();
     QApplication::restoreOverrideCursor();

     if(res > 0)
     {
          QMessageBox::critical(this, "HQEd", msgs[res]);
          return;
     }

     setCurrentMode(XTT_MODE);
     QMessageBox::information(this, "HQEd", msgs[res]);
     tablesArrangment();
}
// ----------------------------------------------------------------------------

// #brief Performs PROLOG code generation
//
// #return No return value.
void MainWin_ui::generatePROLOG(void)
{
     QString deffn = fXttFileName;
     if(deffn.contains("."))
          deffn.remove(QRegExp("\\.(xttml|hml)\\b"));

     QString plFileName = QFileDialog::getSaveFileName(this, tr("Generate To File"),
                              deffn,
                              tr("*.pl (*.pl)"));

     if(plFileName.isEmpty())
          return;

     QFile file(plFileName);
     if(!file.open(QIODevice::WriteOnly | QIODevice::Text))
     {
          QMessageBox::critical(this, tr("HQEd"), tr("Cannot read file %1:\n%2.").arg(plFileName).arg(file.errorString()));
          return;
     }

     QString msg;
     int result = vas_out_SWI_Prolog(&file, msg);

     if(result == VAS_SWI_PROLOG_UNKNOWN_DEVICE)
          QMessageBox::critical(this, "HQEd", "Saving error: device is not specified.");
     if(result == VAS_SWI_PROLOG_DEVICE_NOT_WRITABLE)
          QMessageBox::critical(this, "HQEd", "Saving error: device is not writable.");
     if(result == VAS_SWI_PROLOG_ERROR)
          hqed::showMessages(msg);
}
// ----------------------------------------------------------------------------

// #brief Exectue mode.
//
// #return No return value.
void MainWin_ui::ExecuteModel(void)
{
     xtt::run();
}
// ----------------------------------------------------------------------------
