/**
 * \file	StartScreen_ui.cpp
 * \author	Krzysztof Kaczor kinio4444@gmail.com
 * \date 	14.01.2008 
 * \version	1.0
 * \brief	This file defines class arranges window creating content of cell.
 * \note This file is a part of HQEd. It has been develped within the HeKatE Project.
 * HQEd is a free software that you can redistribute and/or modify
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License or 
 * (at your option) any later version.
 * You should have received a copy of the GNU General Public License with HQEd.
 * \note HQEd is distributed in hope that will be useful.
 * It has NO ANY WARRANTY, neither implied warranty of
 * MERCHANTABILITY nor FITNESS FOR A PARTICULAR PURPOSE. 
 * \note Copyright (C) 2006-9 by the HeKatE Project.
 * \see GNU General Public License for more details http://www.gnu.org/licenses/
 * \see HeKatE project http://hekate.ia.agh.edu.pl
 */

#include <QMessageBox>
#include <QtGui>
#include <QDialog>
#include <QObject>
#include <QKeyEvent>

#include "../namespaces/ns_hqed.h"

#include "MainWin_ui.h"
#include "GLStartWidget.h"
#include "StartScreen_ui.h"
// -----------------------------------------------------------------------------

StartScreen_ui* StartScreen;
// -----------------------------------------------------------------------------

// #brief Constructor StartScreen_ui class.
//
// #param parent Pointer to parent object.
StartScreen_ui::StartScreen_ui(QWidget*)
{
    setupUi(this);
    
    setWindowModality(Qt::ApplicationModal);
    setWindowFlags(Qt::SplashScreen | Qt::FramelessWindowHint);
    hqed::centerWindow(this);
    
    int dWidth = QApplication::desktop()->screenGeometry().width();
    int dHeight = QApplication::desktop()->screenGeometry().height();
    
    setGeometry((dWidth >> 1) - (dHeight >> 2), dHeight >> 2, dHeight >> 1, dHeight >> 1);
    
    glScreen = new GLStartWidget(this);
    glScreen->setCopyArea(QRect(((dWidth >> 1) - (dHeight >> 2))+1, (dHeight >> 2)+1, (dHeight >> 1)-2, (dHeight >> 1)-2));
    glScreen->resize(size());
    connect(glScreen, SIGNAL(runApp()), this, SLOT(runApplication()));
}
// -----------------------------------------------------------------------------

// #breif Destructor About_ui class.
StartScreen_ui::~StartScreen_ui(void)
{
	delete glScreen;
}
// -----------------------------------------------------------------------------

// #brief Function is triggered when key is pressed.
//
// #param event Pointer to event object.
// #return No return value.
void StartScreen_ui::keyPressEvent(QKeyEvent* event)
{
	glScreen->keyPressEvent(event);
}
// -----------------------------------------------------------------------------

// #brief Run application.
//
// #return No return value.
void StartScreen_ui::runApplication(void)
{
	close();
	delete glScreen;
	MainWin->show();
}
// -----------------------------------------------------------------------------
