/*
 *	   $Id: AttributeManager_ui.cpp,v 1.25 2009-05-13 10:28:17 kinio Exp $
 *	
 *     Implementation by Krzysztof Kaczor <kinio4444@gmail.com>
 *
 *     Copyright (C) 2006-9 by the HeKatE Project
 *
 *     HQEd has been develped within the HeKatE Project,
 *     see http://hekate.ia.agh.edu.pl
 *
 *     This file is part of HQEd.
 *
 *     HQEd is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     HQEd is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with HQEd.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <QMessageBox>
#include <QtGui>
#include <QDialog>
#include <QObject>
#include <QKeyEvent>

#include "SelectGroup_ui.h"
#include "AttributeManager_ui.h"
#include "AttributeEditor_ui.h"
#include "SelectGroup_ui.h"

#include "../../namespaces/ns_hqed.h"
         
#include "../../M/XTT/XTT_Table.h"
#include "../../M/XTT/XTT_TableList.h"
#include "../../M/XTT/XTT_AttributeGroups.h"

#include "../../M/hSet.h"
#include "../../M/hType.h"
#include "../../M/hFunction.h"
#include "../../M/hMultipleValue.h"
// -----------------------------------------------------------------------------

AttributeManager_ui* AttrManager;
// -----------------------------------------------------------------------------

// #brief Constructor AttributeManager_ui class.
//
// #param parent Pointer to parent object.
AttributeManager_ui::AttributeManager_ui(QWidget*)
{
    setupUi(this);
    setWindowIcon(QIcon(":/all_images/images/mainico.png")); 
    hqed::centerWindow(this);

    connect(pushButton, SIGNAL(clicked()), this, SLOT(AddNewAttribute()));
    connect(pushButton_2, SIGNAL(clicked()), this, SLOT(EditAttribute()));
    connect(pushButton_3, SIGNAL(clicked()), this, SLOT(DeleteClick()));
    connect(pushButton_5, SIGNAL(clicked()), this, SLOT(NewGroupSelect()));
    connect(pushButton_6, SIGNAL(clicked()), this, SLOT(ReadAttributes()));
    connect(pushButton_7, SIGNAL(clicked()), this, SLOT(selectClick()));
    connect(treeWidget, SIGNAL(itemClicked(QTreeWidgetItem*, int)), this, SLOT(TreeClick(QTreeWidgetItem*, int)));
    setWindowFlags(Qt::WindowMaximizeButtonHint);
    treeWidget->insertTopLevelItems(0, AttributeGroups->CreateTree());

     _type = NULL;                           
     _setClass = -1;                         
     _sreq = false; 
     _currentAttribute = NULL;       
     _useFilter = false; 
     
    _selectedAttribute = NULL;
    fGroupIndex = 0;
    ReadAttributes();

    // signals/slots mechanism in action
    //connect(pushButton, SIGNAL(clicked()), this, SLOT(DeleteClick()));
    //connect(pushButton_3, SIGNAL(clicked()), this, SLOT(SelectGroupClick()));
}
// -----------------------------------------------------------------------------

// #brief Function that sets the window mode
//
// #param *__type - pointer to the type of attribute
// #param __setClass - the class of defined set (0-set, 1-domain, 2-decision context, 3-conditional context)
// #param __singleRequired - determines if the multiplicity of argument value must be single (true) or multiple (false)
// #param *__currentAttribute - the pointer to the current attribute in the expression
// #param *__cell - the pointer to the edited cell.
// #param __useFilter - determines whether given filter must be used
// #return true if the mode is correct otherwise false
bool AttributeManager_ui::setMode(hType* __type, int __setClass, bool __singleRequired, XTT_Attribute* __currentAttribute, XTT_Cell* __cell, bool __useFilter)
{
     // Checking for mode correctness
     if(__type == NULL)
          return false;
     if((__setClass != 0) && (__setClass != 1) && (__setClass != 2) && (__setClass != 3))
          return false;

     setWindowModality(Qt::ApplicationModal);
     
     _type = __type;                           
     _setClass = __setClass;       // 0 - coditional context, 2 - action context                         
     _sreq = __singleRequired; 
     _cell = __cell;       
     _currentAttribute = __currentAttribute;       
     _useFilter = __useFilter; 

     // Searching for the attribute
     if(_currentAttribute != NULL)
     {
          XTT_AttributeList* dest_list = AttributeGroups->AttributeGroup(_currentAttribute);
          if(dest_list != NULL)
               fGroupIndex = (unsigned int)AttributeGroups->IndexOfID(dest_list->ID());
     }
     
     ReadAttributes();
     
     return true;
}
// -----------------------------------------------------------------------------

// #brief Function is triggered when you click the tree object.
//
// #param item Pointer to tree object.
// #param col Number of column that was clicked.
// #return No return value.
void AttributeManager_ui::TreeClick(QTreeWidgetItem* item, int col)
{
     // Zapisanie indexu kliknietej grupy
     fGroupIndex = AttributeGroups->IndexOfName(item->text(col));
     ReadAttributes();
}
// -----------------------------------------------------------------------------

// #brief Function reads information about attributes belnogs to the group.
//
// #return No return value.
void AttributeManager_ui::ReadAttributes(void)
{
     tableWidget->setRowCount(0);

     // Sprawdzanie czy index jest poprawny
     if((int)fGroupIndex == -1)
     {
          const QString content = "There is no group selected.";
          QMessageBox::information(NULL, "HQEd", content, QMessageBox::Ok);
          return;
     }

     // Wczytywanie informacji
     XTT_Table* cellParent = TableList->findCell(_cell);
     QList<XTT_Attribute*> *calist = NULL, 
                           *aalist = NULL;
     QList<XTT_Attribute*> parentAtts;
     
     if(cellParent != NULL)
     {
          calist = cellParent->ContextAtributes(1);
          aalist = cellParent->ContextAtributes(4);
          parentAtts = hqed::mergeListContents(*calist, *aalist);
          delete calist;
          delete aalist;
     }
     int row_selection = -1;
     for(unsigned int i=0;i<AttributeGroups->Group(fGroupIndex)->Count();i++)
     {
          if(_useFilter)
          {
               int type_characteristics = hFunction::typeFunctionRepresentation(_type);
               if(((hFunction::typeFunctionRepresentation(AttributeGroups->Group(fGroupIndex)->Item(i)->Type()) & type_characteristics) == 0) ||      // If incompatible types 
                   ((_sreq) && (_sreq == AttributeGroups->Group(fGroupIndex)->Item(i)->multiplicity())) ||                                            // If single is required and multiplicity is incopatible. This condition allow to select single attribute in case we want general.
                   ((cellParent != NULL) && (!parentAtts.contains(AttributeGroups->Group(fGroupIndex)->Item(i))))                                     // If table doesnt contain an attribute
                 )  
                         continue;
                         
               if(AttributeGroups->Group(fGroupIndex)->Item(i) == _currentAttribute)
                    row_selection = tableWidget->rowCount();
          }
          
          int row_index = tableWidget->rowCount();
          tableWidget->setRowCount(tableWidget->rowCount()+1);
          
          QString type_name = "Type name: " + AttributeGroups->Group(fGroupIndex)->Item(i)->Type()->name();
          if(hpTypesInfo.ptn.indexOf(AttributeGroups->Group(fGroupIndex)->Item(i)->Type()->name()) == -1)
               type_name += "\nBase type: " + hpTypesInfo.ptn.at(hpTypesInfo.iopti(AttributeGroups->Group(fGroupIndex)->Item(i)->Type()->baseType()));

          QString val = "";
          hMultipleValueResult mvr = AttributeGroups->Group(fGroupIndex)->Item(i)->Value()->value();
          
          if(!(bool)mvr)
               val = (QString)mvr;
          if((bool)mvr)
               val = mvr.toSet(false)->toString(); //mvr.toSet()->toString() + " = " + mvr.toSet()->toVals();
               
          QTableWidgetItem* _name = new QTableWidgetItem(AttributeGroups->Group(fGroupIndex)->Item(i)->Name());
          QTableWidgetItem* _aronym = new QTableWidgetItem(AttributeGroups->Group(fGroupIndex)->Item(i)->Acronym());
          QTableWidgetItem* _desc = new QTableWidgetItem(AttributeGroups->Group(fGroupIndex)->Item(i)->Description());
          QTableWidgetItem* _class = new QTableWidgetItem(XTT_Attribute::classString(AttributeGroups->Group(fGroupIndex)->Item(i)->Class()));
          QTableWidgetItem* _type = new QTableWidgetItem(type_name);
          QTableWidgetItem* _conrepr = new QTableWidgetItem(AttributeGroups->Group(fGroupIndex)->Item(i)->ConstraintsRepresentation());
          QTableWidgetItem* _val = new QTableWidgetItem(val);

          tableWidget->setItem(row_index, 0, _name);
          tableWidget->setItem(row_index, 1, _aronym);
          tableWidget->setItem(row_index, 2, _desc);
          tableWidget->setItem(row_index, 3, _class);
          tableWidget->setItem(row_index, 4, _type);
          tableWidget->setItem(row_index, 5, _conrepr);
          tableWidget->setItem(row_index, 6, _val);
     }
     
     if(row_selection > -1)
          tableWidget->setRangeSelected(QTableWidgetSelectionRange(row_selection, 0, row_selection, tableWidget->columnCount()-1), true);
}
// -----------------------------------------------------------------------------

// #brief Function shows edit attribute window.
//
// #return No return value.
void AttributeManager_ui::AddNewAttribute(void)
{
     delete AttributeEditor;
     AttributeEditor = new AttrEditor(true, fGroupIndex);

     connect(AttributeEditor, SIGNAL(ClickOK()), this, SLOT(ReadAttributes()));
     
     AttributeEditor->show();
}
// -----------------------------------------------------------------------------

// #brief Function shows edit attribute window.
//
// #return No return va
void AttributeManager_ui::EditAttribute(void)
{
     // Wczytywanie indexu atrybutu
     if(tableWidget->currentRow() == -1)
     {
          QMessageBox::information(NULL, "HQEd", "There is no row selected.", QMessageBox::Ok);
          return;
     }

     QString name = tableWidget->item(tableWidget->currentRow(), 0)->text();
     int index = AttributeGroups->Group(fGroupIndex)->IndexOfName(name);

     delete AttributeEditor;
     AttributeEditor = new AttrEditor(false, fGroupIndex, index);

     connect(AttributeEditor, SIGNAL(ClickOK()), this, SLOT(ReadAttributes()));

     AttributeEditor->show();
}
// -----------------------------------------------------------------------------

// #brief Function deletes attribute that was chosen.
//
// #return No return value.
void AttributeManager_ui::DeleteClick(void)
{
     // Kiedy nie jest zaznaczony zaden wiersz
     if((int)fGroupIndex == -1)
          return;

     // Wczytywanie indexu atrybutu
     if(tableWidget->currentRow() == -1)
     {
          QMessageBox::information(NULL, "HQEd", "There is no row selected.", QMessageBox::Ok);
          return;
     }

     QString name = tableWidget->item(tableWidget->currentRow(), 0)->text();
     int index = AttributeGroups->Group(fGroupIndex)->IndexOfName(name);

     // Gdy nie znaleziono
     if(index == -1)
     {
          QMessageBox::critical(NULL, "HQEd", "Can't find selected attribute.", QMessageBox::Ok);
          return;
     }

     // Potwierdzenie usuniecia
     const QString content = "Do you realy want to delete attribute named " + name + "?";
     if(QMessageBox::warning(NULL, "HQEd", content, QMessageBox::Yes | QMessageBox::No) == QMessageBox::No)
          return;

     // Inaczej kasujemy obiekt
     AttributeGroups->Group(fGroupIndex)->Delete(index);

     // Odswiezenie widoku
     ReadAttributes();

     // Odswiezenie widoku tabel
     TableList->RefreshAll();
}
// -----------------------------------------------------------------------------

// #brief Function shows select group window.
//
// #return No return value.
void AttributeManager_ui::NewGroupSelect(void)
{
     // Kiedy nie jest zaznaczony zaden wiersz
     if((int)fGroupIndex == -1)
          return;

     // Wczytywanie indexu atrybutu
     if(tableWidget->currentRow() == -1)
     {
          QMessageBox::information(NULL, "HQEd", "There is no row selected.", QMessageBox::Ok);
          return;
     }
     
     QString name = tableWidget->item(tableWidget->currentRow(), 0)->text();
     int fAttrIndex = AttributeGroups->Group(fGroupIndex)->IndexOfName(name);

     // Gdy nie znaleziono
     if(fAttrIndex == -1)
     {
          QMessageBox::critical(NULL, "HQEd", "Can't find selected attribute.", QMessageBox::Ok);
          return;
     }
     
     delete SelGroup;
     SelGroup = new SelectGroup_ui;
     connect(SelGroup, SIGNAL(ResultOK()), this, SLOT(NewGroupChange()));
     SelGroup->show();
}
// -----------------------------------------------------------------------------

// #brief Function is triggered when you change the group of attribute.
//
// #return No return value.
void AttributeManager_ui::NewGroupChange(void)
{
     QString name = tableWidget->item(tableWidget->currentRow(), 0)->text();
     int fAttrIndex = AttributeGroups->Group(fGroupIndex)->IndexOfName(name);

     // Gdy nie znaleziono
     if(fAttrIndex == -1)
     {
          QMessageBox::critical(NULL, "HQEd", "Can't find selected attribute.", QMessageBox::Ok);
          return;
     }

     int pindex = AttributeGroups->IndexOfName(SelGroup->Result());
     if(pindex == -1)
          return;

     // Sprawdzanie czy podana nazwa juz nie istnieje
     QString atrrNewName = name;                                                               // Nazwa atrybutu
     int sgindex = fGroupIndex;                                                                // Index starej grupy
     int dgindex = pindex;                                                                     // Index nowej grupy


     if((sgindex == -1) || (dgindex == -1))
     {
          const QString content = "Incorrect source/destination group.";
          QMessageBox::critical(NULL, "HQEd", content, QMessageBox::Ok);
          return;
     }
     int sgai = fAttrIndex;                                                                    // Index atrybutu w starej grupie
     int dgai = AttributeGroups->Group(dgindex)->IndexOfName(atrrNewName);                     // Index atrybutu w nowej grupie
     int dgaci = AttributeGroups->Group(dgindex)->IndexOfAcronym(atrrNewName);                 // Index atrybutu w nowej grupie o akronimie takim jak nazwa atrybutu

     if(((dgai > -1) && (sgindex == dgindex) && (sgai != dgai)) ||
      ((dgai > -1) && (sgindex != dgindex)) ||
      ((dgaci > -1) && (sgindex == dgindex) && (sgai != dgaci)) ||
      ((dgaci > -1) && (sgindex != dgindex)))
     {
          const QString content = "The name/acronym is already exists in this group.\nPlease give another name.";
          QMessageBox::information(NULL, "HQEd", content, QMessageBox::Ok);
          return;
     }

     // Sprawdzanie czy podany akronim juz nie istnieje
     QString atrrNewAcronym = AttributeGroups->Group(sgindex)->Item(fAttrIndex)->Acronym();    // Nowy akronim atrybutu
     dgai = AttributeGroups->Group(dgindex)->IndexOfAcronym(atrrNewAcronym);                   // Index atrybutu w nowej grupie
     dgaci = AttributeGroups->Group(dgindex)->IndexOfName(atrrNewAcronym);                     // Index atrybutu o nazwie jak akronim aktalnie przenoszonego

     if(((dgai > -1) && (sgindex == dgindex) && (sgai != dgai)) ||
      ((dgai > -1) && (sgindex != dgindex)) ||
      ((dgaci > -1) && (sgindex == dgindex) && (sgai != dgaci)) ||
      ((dgaci > -1) && (sgindex != dgindex)))
     {
          const QString content = "The name/acronym is already exists in this group.\nPlease give another acronym.";
          QMessageBox::information(NULL, "HQEd", content, QMessageBox::Ok);
          return;
     }

     // Gdy nastepuje zmiana grupy to trzeba przeniesc atrybut
     if(sgindex != dgindex)
     {
          // Dodawanie do nowej grupy
          AttributeGroups->Group(dgindex)->Add(AttributeGroups->Group(sgindex)->Item(fAttrIndex));

          // Usuwanie ze starej grupy
          AttributeGroups->Group(sgindex)->Delete(sgai, false);	// Usuwanie atrybutu
     }

     // Odswiezenie widoku
     ReadAttributes();

     // Odswiezenie widoku tabel
     TableList->RefreshAll();
}
// -----------------------------------------------------------------------------

// #brief Function is triggered when the select button is clicked
//
// #return No return value.
void AttributeManager_ui::selectClick(void)
{
     // Kiedy nie jest zaznaczony zaden wiersz
     if((int)fGroupIndex == -1)
          return;

     // Wczytywanie indexu atrybutu
     if(tableWidget->currentRow() == -1)
     {
          QMessageBox::information(NULL, "HQEd", "There is no row selected.", QMessageBox::Ok);
          return;
     }

     QString name = tableWidget->item(tableWidget->currentRow(), 0)->text();
     int index = AttributeGroups->Group(fGroupIndex)->IndexOfName(name);

     // Gdy nie znaleziono
     if(index == -1)
     {
          QMessageBox::critical(NULL, "HQEd", "Can't find selected attribute.", QMessageBox::Ok);
          return;
     }

     // Inaczej kasujemy obiekt
     _selectedAttribute = AttributeGroups->Group(fGroupIndex)->Item(index);
     
     emit onSelectClose();
     close();
}
// -----------------------------------------------------------------------------

// #brief Function is triggered when key is pressed.
//
// #param event Pointer to event object.
// #return No return value.
void AttributeManager_ui::keyPressEvent(QKeyEvent *event)
{
     switch (event->key())
     {
          case Qt::Key_Escape:
               close();
               break;

          case Qt::Key_Return:
               EditAttribute();
               break;

          default:
               QWidget::keyPressEvent(event);
     }
}
// -----------------------------------------------------------------------------

// #brief Function that returns the attribute selection result
//
// #return the attribute selection result
XTT_Attribute* AttributeManager_ui::selectionResult(void)
{
     return _selectedAttribute;
}
// -----------------------------------------------------------------------------
