/*
 *	   $Id: MyGraphicsScene.cpp,v 1.29 2009-05-13 10:28:18 kinio Exp $
 *	
 *     Implementation by Krzysztof Kaczor <kinio4444@gmail.com>
 *
 *     Copyright (C) 2006-9 by the HeKatE Project
 *
 *     HQEd has been develped within the HeKatE Project,
 *     see http://hekate.ia.agh.edu.pl
 *
 *     This file is part of HQEd.
 *
 *     HQEd is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     HQEd is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with HQEd.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <QMessageBox>
#include <QtGui>
#include <QDialog>
#include <QObject>
#include <QKeyEvent>
#include <QGraphicsScene>
#include <QGraphicsView>

#include "../MainWin_ui.h"
#include "../Settings_ui.h"
#include "MyGraphicsScene.h"
// -----------------------------------------------------------------------------

// #brief Constructor MyGraphicsScene class.
//
// #param oParent Pointer to parent object.
MyGraphicsScene::MyGraphicsScene(QObject* oParent) : QGraphicsScene(oParent)
{	
	setNewItem(NULL);
	
	fCurrentAngle = 0;
	fSelectedConnectionID = -1;
	fSelectedTableID = -1;
}
// -----------------------------------------------------------------------------

// #brief Destructor MyGraphicsScene class.
MyGraphicsScene::~MyGraphicsScene(void)
{
}
// -----------------------------------------------------------------------------

// #brief Function sets object on the scene.
//
// #param _item Pointer to object that is set on the scene.
// #return No return value.
void MyGraphicsScene::setNewItem(QGraphicsItem* _item)
{
	fNewItem = _item;
}
// -----------------------------------------------------------------------------
	
// #brief Function gets object that is set on the scene.
//
// #return NPointer to object that is set on the scene.
QGraphicsItem* MyGraphicsScene::newItem(void)
{
	return fNewItem;
}
// -----------------------------------------------------------------------------

// #brief Function resets pointer to the object that is set on the scene.
//
// #return No return value.
void MyGraphicsScene::resetNewItem(void)
{
	setNewItem(NULL);
}
// -----------------------------------------------------------------------------

// #brief Function is triggered when mouse cursor is moved.
//
// #param mouseEvent Pointer to object with event parameters.
// #return No return value.
void MyGraphicsScene::mouseMoveEvent(QGraphicsSceneMouseEvent* mouseEvent)
{
	// Jezeli mamy jakis nowy przedmiot to jest on priorytetm do tego aby zawsze byl widoczny
	if(fNewItem)
	{
		fNewItem->setPos(mouseEvent->scenePos());
		if(Settings->xttEnabledAutocentering())
			for(int i=0;i<views().count();i++)
				views().at(i)->ensureVisible(fNewItem);
	}
		
	// Jezeli cos chwycilismy myszka
	if((!fNewItem) && (mouseGrabberItem()))
	{
		if(Settings->xttEnabledAutocentering())
			for(int i=0;i<views().count();i++)
				views().at(i)->ensureVisible(mouseGrabberItem());
	}
		
	QGraphicsScene::mouseMoveEvent(mouseEvent);
}
// -----------------------------------------------------------------------------

// #brief Function is triggered when mouse is clicked on the scene.
//
// #param mouseEvent Pointer to object with event parameters.
// #return No return value.
void MyGraphicsScene::mousePressEvent(QGraphicsSceneMouseEvent* mouseEvent)
{
	if(fNewItem)
		resetNewItem();
		
	QGraphicsScene::mousePressEvent(mouseEvent);
}
// -----------------------------------------------------------------------------

// #brief Function is triggered when angle of scene rotation is changed.
//
// #param Angle value.
// #return Angle of scene rotation.
void MyGraphicsScene::Rotate(int angle)
{
	QList<QGraphicsView*> views_list = views();
	
	for(int i=0;i<views_list.count();i++)
		views_list.at(i)->rotate(angle-fCurrentAngle);
		
     fCurrentAngle = angle;
	//rotateAngleEdit->setValue(angle);
}
// -----------------------------------------------------------------------------

// #brief Function zoom in on the scene.
//
// #return No return value.
void MyGraphicsScene::zin(void)
{
	double factor = (Settings->zoomFactorValue(MainWin->currentMode())/100.0)+1.0;
     scaleView(factor);
}
// -----------------------------------------------------------------------------

// #brief Function zoom out on the scene.
//
// #return No return value
void MyGraphicsScene::zout(void)
{
	double factor = (Settings->zoomFactorValue(MainWin->currentMode())/100.0)+1.0;
     scaleView(1/factor);
}
// -----------------------------------------------------------------------------

// #brief Function zoom scene.
//
// #param scaleFactor Zoom value.
// #return Angle of scene rotation.
void MyGraphicsScene::scaleView(qreal scaleFactor)
{
	QList<QGraphicsView*> views_list = views();
	
	for(int i=0;i<views_list.count();i++)
	{
		qreal factor = views_list.at(i)->matrix().scale(scaleFactor, scaleFactor).mapRect(QRectF(0, 0, 1, 1)).width();
		if (factor < 0.007 || factor > 100)
			continue;
		views_list.at(i)->scale(scaleFactor, scaleFactor);
	}
}
// -----------------------------------------------------------------------------

// #brief Function adapts scene size to the size of elements.
//
// #return No return value.
void MyGraphicsScene::AdaptSceneSize(void)
{
     const int Margin = 50;

     QList<QGraphicsItem*> items_list = items();

     float minLeft = 0;
     float minTop = 0;
     float maxLeft = 0;
     float maxTop = 0;

     for(int i=0;i<items_list.size();i++)
     {
          if(minLeft > items_list.at(i)->x())
               minLeft = items_list.at(i)->x();
          if(minTop > items_list.at(i)->y())
               minTop = items_list.at(i)->y();
          if(maxLeft < items_list.at(i)->x())
               maxLeft = items_list.at(i)->x();
          if(maxTop < items_list.at(i)->y())
               maxTop = items_list.at(i)->y();
     }

     minLeft -= Margin;
     minTop -= Margin;
     maxLeft += Margin;
     maxTop += Margin;

     setSceneRect(minLeft, minTop, maxLeft-minLeft, maxTop-minTop);
}
// -----------------------------------------------------------------------------

// #brief Function moves elements on the scene up.
//
// #return No return value.
void MyGraphicsScene::MoveItemsUp(void)
{	
	int moveDistance = Settings->xttMoveElementsDistance();
     setSceneRect(sceneRect().x(), sceneRect().y()+moveDistance, sceneRect().width(), sceneRect().height()-moveDistance);
}
// -----------------------------------------------------------------------------

// #brief Function moves elements on the scene down.
//
// #return No return value.
void MyGraphicsScene::MoveItemsDown(void)
{
	int moveDistance = Settings->xttMoveElementsDistance();
     setSceneRect(sceneRect().x(), sceneRect().y()-moveDistance, sceneRect().width(), sceneRect().height()+moveDistance);
}
// -----------------------------------------------------------------------------

// #brief Function moves elements on the scene right.
//
// #return No return value.
void MyGraphicsScene::MoveItemsRight(void)
{
	int moveDistance = Settings->xttMoveElementsDistance();
     setSceneRect(sceneRect().x()-moveDistance, sceneRect().y(), sceneRect().width()+moveDistance, sceneRect().height());
}
// -----------------------------------------------------------------------------

// #brief Function moves elements on the scene left.
//
// #return No return value.
void MyGraphicsScene::MoveItemsLeft(void)
{
	int moveDistance = Settings->xttMoveElementsDistance();
     setSceneRect(sceneRect().x()+moveDistance, sceneRect().y(), sceneRect().width()-moveDistance, sceneRect().height());
}
// -----------------------------------------------------------------------------
