 /**
 *                                                                             *
 * @file ARD_Attribute.h                                                       *
 * Project name: HeKatE hqed                                                   *
 * $Id: ARD_Attribute.h,v 1.20 2009-05-13 10:28:19 kinio Exp $                  *
 * Author: Krzysztof Kaczor                                                    *
 * Date: 21.12.2007                                                            *
 *                                                                             *
 * Implementation by Krzysztof Kaczor <kinio4444@gmail.com>                    *
 *                                                                             *
 * Copyright (C) 2006-9 by the HeKatE Project                                  *
 *                                                                             *
 * HQEd has been develped within the HeKatE Project,                           *
 * see http://hekate.ia.agh.edu.pl                                             *
 *                                                                             *
 * This file is part of HQEd.                                                  *
 *                                                                             *
 * HQEd is free software: you can redistribute it and/or modify                *
 * it under the terms of the GNU General Public License as published by        *
 * the Free Software Foundation, either version 3 of the License, or           *
 * (at your option) any later version.                                         *
 *                                                                             *
 * HQEd is distributed in the hope that it will be useful,                     *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU General Public License for more details.                                *
 *                                                                             *
 * You should have received a copy of the GNU General Public License           *
 * along with HQEd.  If not, see <http://www.gnu.org/licenses/>.               *
 *                                                                             *
 * Comment: Plik zwiera definicje obiektu bedacego atrybutem, zmienna w        *
 * diagramch ARD, jeeli w pliku ARD atrybut jest zdefiniowany w sposob        *
 * dokladny przy pomocy ATTML, to w tej klasie przehcowujemy tylko nazwe       *
 * natomiast reszte danych w klasie XTT_ATTRIBUTE                              *
 *                                                                             *
 */
 
#ifndef ARD_ATTRIBUTEH
#define ARD_ATTRIBUTEH

// -----------------------------------------------------------------------------

#define ARD_ATT_CONCEPTUAL 0
#define ARD_ATT_PHYSICAL   1
// -----------------------------------------------------------------------------

#include <QtCore/QString>
#include <QDomDocument>
#include <QStringList>
// -----------------------------------------------------------------------------

class XTT_Attribute;
// -----------------------------------------------------------------------------

class ARD_Attribute
{
	
private:

	/// Identyfikator atrybutu
	QString fID;
	
	/// Nazwa atrybutu:
	/// - jezeli jest z wielkiej litery to znaczy ze jest to ztrybut koncepcyjny
	/// - jezeli jest z malej litery to oznacza ze jest to atrybut fizyczny
	QString fName;
	
	/// Opis atrybutu
	QString fAttrDesc;
	
	/// Zawiera wskaznik do obiektu XTT_Attribute, ktory jest tym samym atrybutem tylko dobrze zdefiniowanym, jezeli NULL to nie ma odpowiednika
	XTT_Attribute* fXTTequivalent;
	
	/// Okresla czy atrybut byl edytowany
	bool fChanged;
	
public:

	/// Konstruktor defaultowy
	ARD_Attribute(void);
	
	/// Konstruktor przyjmujacy wartosc ID
	ARD_Attribute(QString);
	
	/// Konstruktor przyjmujacy wartosc ID i nazwe atrybutu
	ARD_Attribute(QString, QString);
	
	/// ----------------- Metody zwracajaca wartosci pol klasy -----------------
	
	/// Zwraca typ atrybutu: ARD_ATT_CONCEPTUAL, ARD_ATT_PHYSICAL
	int type(void);
	
	/// Zwraca ID atrybutu
	QString id(void);
	
	/// Zwraca nazwe atrybutu
	QString name(void);
	
	/// Zwraca opis atrybutu
	QString description(void);
	
	/// Zwraca wskaznik do obiektu XTT_Attribute, ktory jest tym samym atrybutem tylko dobrze zdefiniowanym
	XTT_Attribute* xttEquivalent(void);
	
	/// Zwraca czy dany atrybut byl zmieniany
	bool changed(void);
	
	/// ----------------- Metody ustawiajace wartosci pol klasy -----------------
	
	/// Ustawia nazwe atrybutu
	bool setName(QString);
	
	/// Ustawia opis atrybutu
	void setDescription(QString);
     
     /// \brief Function that sets the ID of the attribute
     ///
     /// \return no values return
	void setId(QString);
	
	/// Ustawia status zmiany atrybutu
	void setChanged(bool);
	
	/// Ustawia ekwiwalnet atrybutu
	void setXTTequivalent(XTT_Attribute*);
     
     /// \brief Function that is called when some imporatant event occurs. It sends the information about event to the others object.
	///
     /// \param *__errmsgs - pointer to the string that contains the error message that are generated on event.
     /// \param __event - the event identifier
     /// \param ... - the list of event params
	/// \return true on succes otherwise false
     bool eventMessage(QString* __errmsgs, int __event, ...);
     
     // ----------------- Metody statyczne -----------------
     
     /// \brief Static function that returns the prefix of the id.
     ///
     /// \return the prefix of the id as string.
     static QString idPrefix(void);
};
// -----------------------------------------------------------------------------
#endif
