 /**
 *                                                                             *
 * @file ARD_AttributeList.h                                                   *
 * Project name: HeKatE hqed                                                   *
 * $Id: ARD_AttributeList.h,v 1.20 2009-05-13 10:28:19 kinio Exp $              *
 * Author: Krzysztof Kaczor                                                    *
 * Date: 22.12.2007                                                            *
 *                                                                             *
 * Implementation by Krzysztof Kaczor <kinio4444@gmail.com>                    *
 *                                                                             *
 * Copyright (C) 2006-9 by the HeKatE Project                                  *
 *                                                                             *
 * HQEd has been develped within the HeKatE Project,                           *
 * see http://hekate.ia.agh.edu.pl                                             *
 *                                                                             *
 * This file is part of HQEd.                                                  *
 *                                                                             *
 * HQEd is free software: you can redistribute it and/or modify                *
 * it under the terms of the GNU General Public License as published by        *
 * the Free Software Foundation, either version 3 of the License, or           *
 * (at your option) any later version.                                         *
 *                                                                             *
 * HQEd is distributed in the hope that it will be useful,                     *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU General Public License for more details.                                *
 *                                                                             *
 * You should have received a copy of the GNU General Public License           *
 * along with HQEd.  If not, see <http://www.gnu.org/licenses/>.               *
 *                                                                             *
 * Comment: Plik zwiera definicje obiektu bedacego pojemnikiem na atrybuty     *
 * ARD                                                                         *
 */
 
#ifndef ARD_ATTRIBUTELISTH
#define ARD_ATTRIBUTELISTH
// -----------------------------------------------------------------------------

#include <QtCore/QString>
#include <QDomDocument>
#include <QStringList>
#include <QList>
// -----------------------------------------------------------------------------

class ARD_Attribute;
class XTT_Attribute;
// -----------------------------------------------------------------------------

class ARD_AttributeList : public QList<ARD_Attribute*>
{

private:
     
     /// Okresla status czy zawartosc pojemnika byla zmieniona
     bool fChanged;
     
public:

     /// Konstruktor domyslny
     ARD_AttributeList(void);
     
     /// Destruktor
     ~ARD_AttributeList(void);
     
     /// ----------------- Metody ustawiajace wartosci pol klasy -----------------
     
     /// Ustawia status zmiany na zadana wartosc
     void setChanged(bool);
     
     /// ----------------- Metody operacyjne -----------------
     
     /// Oproznia pojemnik
     void flush(void);
     
     /// Dodaje nowy atrybut
     ARD_Attribute* add(void);
     
     /// Dodaje nowy atrybut
     void add(ARD_Attribute*);
     
     /// Usuwa wybrany atrybut
     void remove(int);
     
     /// Wyszukuje atrybuty na podstawie nazwy
     int indexOfName(QString);
     
     /// Wyszukuje atrybuty po ID
     int indexOfID(QString);
     
     /// \brief Functions that finds and returns the unique id
     ///
     /// \return the unique id as string
     QString findUniqID(void);
     
     /// \brief Functions that creates the ARD attributes using XTT attributes
     ///
     /// \return true on succes, false on error
     bool xttAtts2ardAtts(void);
     
     /// Sortuje atrybuty po nazwie
     void sort(void);
     
     /// Zwraca czy atrybuty byly modyfikowane
     bool changed(void);
     
     /// \brief Function that is called when some imporatant event occurs. It sends the information about event to the others object.
	///
     /// \param *__errmsgs - pointer to the string that contains the error message that are generated on event.
     /// \param __event - the event identifier
     /// \param ... - the list of event params
	/// \return true on succes otherwise false
     bool eventMessage(QString* __errmsgs, int __event, ...);
     
     /// Wcztuje dane z pliku w formacie HML w wersji 1.0
     bool fromHML_1_0(QDomElement&);
};
// -----------------------------------------------------------------------------

extern ARD_AttributeList* ardAttributeList;
// -----------------------------------------------------------------------------
#endif
