 /**
 *                                                                             *
 * @file ARD_Dependency.h                                                      *
 * Project name: HeKatE hqed                                                   *
 * $Id: ARD_Dependency.h,v 1.20 2009-05-13 10:28:19 kinio Exp $                 *
 * Author: Krzysztof Kaczor                                                    *
 * Date: 26.12.2007                                                            *
 *                                                                             *
 * Implementation by Krzysztof Kaczor <kinio4444@gmail.com>                    *
 *                                                                             *
 * Copyright (C) 2006-9 by the HeKatE Project                                  *
 *                                                                             *
 * HQEd has been develped within the HeKatE Project,                           *
 * see http://hekate.ia.agh.edu.pl                                             *
 *                                                                             *
 * This file is part of HQEd.                                                  *
 *                                                                             *
 * HQEd is free software: you can redistribute it and/or modify                *
 * it under the terms of the GNU General Public License as published by        *
 * the Free Software Foundation, either version 3 of the License, or           *
 * (at your option) any later version.                                         *
 *                                                                             *
 * HQEd is distributed in the hope that it will be useful,                     *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU General Public License for more details.                                *
 *                                                                             *
 * You should have received a copy of the GNU General Public License           *
 * along with HQEd.  If not, see <http://www.gnu.org/licenses/>.               *
 *                                                                             *
 * Comment: Plik zwiera definicje obiektu bedacego zaleznosci pomiedzy         *
 * wlasciowsciami w diagramie ARD.                                             *
 *                                                                             *
 */
 
#ifndef ARD_DEPENDENCYH
#define ARD_DEPENDENCYH
// -----------------------------------------------------------------------------

#include <QtCore/QString>
#include <QStringList>
// -----------------------------------------------------------------------------

class ARD_Property;
// -----------------------------------------------------------------------------

class ARD_Dependency
{
	
private:

	/// Identyfikator zaleznosci
	QString fID;
	
	///	Wskaznik do obiektu niezaleznego
	ARD_Property* fIndependent;
	
	///	Wskaznik do obiektu zaleznego
	ARD_Property* fDependent;
	
	/// Okresla czy zaleznosc byla edytowana
	bool fChanged;
	
public:

	/// Konstruktor defaultowy
	ARD_Dependency(void);
	
	/// Konstruktor przyjmujacy wartosc ID
	ARD_Dependency(QString);
	
	/// ----------------- Metody zwracajaca wartosci pol klasy -----------------
	
	/// Zwraca identyfikator obiektu
	QString id(void);
	
	/// Zwraca status zmiany obiektu
	bool changed(void);
	
	/// Zwraca wskaznik do obiektu niezaleznego
	ARD_Property* independent(void);
	
	/// Zwraca wskaznik do obiektu zaleznego
	ARD_Property* dependent(void);
	
	/// ----------------- Metody ustawiajace wartosci pol klasy -----------------
	
	///	Ustawia status zmiany obiektu
	void setChanged(bool);
     
     /// \brief Sets the id of the object
     ///
     /// \param __id - id of the object
     /// \return no values returns
	void setID(QString __id);
	
	///	Ustawia wskaznik do obiektu niezaleznego
	bool setIndependent(ARD_Property*);
	
	///	Ustawia wskaznik do obiektu zaleznego
	bool setDependent(ARD_Property*);
     
     /// \brief Function that is called when some imporatant event occurs. It sends the information about event to the others object.
	///
     /// \param *__errmsgs - pointer to the string that contains the error message that are generated on event.
     /// \param __event - the event identifier
     /// \param ... - the list of event params
	/// \return true on succes otherwise false
     bool eventMessage(QString* __errmsgs, int __event, ...);
     
     // ----------------- Metody statyczne -----------------
     
     /// \brief Static function that returns the prefix of the id.
     ///
     /// \return the prefix of the id as string.
     static QString idPrefix(void);
	
};
// -----------------------------------------------------------------------------
#endif
