 /**
 *                                                                             *
 * @file ARD_Level.h                                                           *
 * Project name: HeKatE hqed                                                   *
 * $Id: ARD_Level.h,v 1.22 2009-05-13 10:28:19 kinio Exp $                      *
 * Author: Krzysztof Kaczor                                                    *
 * Date: 03.01.2008                                                            *
 *                                                                             *
 * Implementation by Krzysztof Kaczor <kinio4444@gmail.com>                    *
 *                                                                             *
 * Copyright (C) 2006-9 by the HeKatE Project                                  *
 *                                                                             *
 * HQEd has been develped within the HeKatE Project,                           *
 * see http://hekate.ia.agh.edu.pl                                             *
 *                                                                             *
 * This file is part of HQEd.                                                  *
 *                                                                             *
 * HQEd is free software: you can redistribute it and/or modify                *
 * it under the terms of the GNU General Public License as published by        *
 * the Free Software Foundation, either version 3 of the License, or           *
 * (at your option) any later version.                                         *
 *                                                                             *
 * HQEd is distributed in the hope that it will be useful,                     *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU General Public License for more details.                                *
 *                                                                             *
 * You should have received a copy of the GNU General Public License           *
 * along with HQEd.  If not, see <http://www.gnu.org/licenses/>.               *
 *                                                                             *
 * Comment: Plik zwiera definicje obiektu bedacego pojemnikiem na zaleznosci   *
 * i wlasciwosci na danym poziomie projektowania.                              *
 *                                                                             *
 */
 
#ifndef ARD_LEVELH
#define ARD_LEVELH
// -----------------------------------------------------------------------------

#include <QtCore/QString>
#include <QStringList>
#include <QList>
// -----------------------------------------------------------------------------

class ARD_Property;
class ARD_PropertyList;
class ARD_Dependency;
class ARD_DependencyList;

class gARDlevel;
// -----------------------------------------------------------------------------

class ARD_Level
{
private:
	
	/// Numer levelu
	int fLevel;
	
	/// Lista wlasnosci
	QList<ARD_Property*>* fProperties;
	
	/// Lista zaleznosci
	QList<ARD_Dependency*>* fDependencies;
	
	/// Okresla status czy zawartosc pojemnika byla zmieniona
	bool fChanged;
	
	/// Graficzna reprezentacja poziomu
	gARDlevel* fGlevel;
	
	/// Wyszukuje rodzicow dla danej wlasnosci, funckja jest prywatna ze wzgledu na jej rekurencyjnosc i niebezpieczenstwo
	/// podczas nieumiejetnego wywolania
	void findParents(QString, QList<ARD_Property*>*);
	
public:

	/// Konstruktor domyslny
	ARD_Level(void);
	
	/// Konstruktor ustawiajacy wartosc levelu
	ARD_Level(int);
	
	/// Destruktor
	~ARD_Level(void);
	
	/// ---------------- Metody zwracajace wartosci pol klasy ----------------
	
	/// Zwraca czy zelznosci byly modyfikowane
	bool changed(void);
	
	/// Zwraca numer levelu
	int level(void);
	
	/// Zwraca wskaznik do listy wlasnosci na tym poziomie
	QList<ARD_Property*>* properties(void);
	
	/// Zwraca wskaznik do zaleznosci wlasnosci na tym poziomie
	QList<ARD_Dependency*>* dependencies(void);
	
	/// Zwraca wskaznik do obiektu bedacego graficzna reprezentacja poziomu
	gARDlevel* gLevel(void);
	
	/// ---------------- Metody ustawiajace wartosci pol klasy ----------------
	
	/// Ustawia numer levelu
	void setLevel(int);
	
	/// Ustawia status zmiany na zadana wartosc
	void setChanged(bool);
	
	/// -------------- Metody operacyjne klasy  dla zaleznosci --------------
	
	/// Oproznia pojemnik
	void flush(void);
     
     /// Wyszukuje i zwraca liste zaleznosci ktore zaleza tylko od danej
	QList<ARD_Dependency*>* indexOfDependentOfOnly(ARD_Property*);
	
	/// Wyszukuje i zwraca liste zaleznosci o podanych wartoscich zrodla i przeznaczenia
	QList<ARD_Dependency*>* indexOfinout(ARD_Property*, ARD_Property*);
	
	/// Wyszukuje i zwraca liste zaleznosci o podanych wartoscich zrodla i przeznaczenia
	QList<ARD_Dependency*>* indexOfinout(QString, QString);
	
	/// Wyszukuje zelznosc po ID
	int indexOfDepID(QString);
	
	/// Dodaje nowa zaleznosc
	bool addDependency(ARD_Dependency*);
	
	/// Usuwa zaleznosc
	bool removeDependency(ARD_Dependency*);
	
	/// Usuwa zaleznosc o podanym indexie
	bool removeDependency(int);
     
     /// Usuwa zaleznosci pode w liscie
	bool removeDependencies(QList<ARD_Dependency*>*);
	
	/// --------------- Metody operacyjne klasy  dla wlasnosci ---------------
	
	/// Wyszukuje obiekty zawierajace atrybut o podanej nazwie
	QList<ARD_Property*>* listOfAttributeName(QString);
	
	/// Wyszukuje obiekty zawierajace atrybut o podanym id
	QList<ARD_Property*>* listOfAttributeID(QString);
	
	/// Wyszukuje obiekty po ID
	int indexOfPrpID(QString);
	
	/// Dodaje nowa wlasnosc
	bool addProperty(ARD_Property*);
	
	/// Usuwa wlasnosc
	bool removeProperty(ARD_Property*);
	
	/// Usuwa wlasnosc o podanym indexie
	bool removeProperty(int);
	
	/// Uaktualnie poziom na podstawie podanego
	bool update(ARD_Level*, bool);
     
     /// \brief Function that is called when some imporatant event occurs. It sends the information about event to the others object.
	///
     /// \param *__errmsgs - pointer to the string that contains the error message that are generated on event.
     /// \param __event - the event identifier
     /// \param ... - the list of event params
	/// \return true on succes otherwise false
     bool eventMessage(QString* __errmsgs, int __event, ...);
	
	/// ----------------------- Metody operacyjne klasy -----------------------
	
	/// Rysuje level na scenie
	QRectF paint(void);
	
	/// Wyszukuje rodzina dla danej wlasnosci
	QList<ARD_Property*>* findFamily(QString);
};
// -----------------------------------------------------------------------------
#endif
