 /**
 *                                                                             *
 * @file ARD_LevelList.h                                                       *
 * Project name: HeKatE hqed                                                   *
 * $Id: ARD_LevelList.h,v 1.20 2009-05-13 10:28:20 kinio Exp $                  *
 * Author: Krzysztof Kaczor                                                    *
 * Date: 04.01.2008                                                            *
 *                                                                             *
 * Implementation by Krzysztof Kaczor <kinio4444@gmail.com>                    *
 *                                                                             *
 * Copyright (C) 2006-9 by the HeKatE Project                                  *
 *                                                                             *
 * HQEd has been develped within the HeKatE Project,                           *
 * see http://hekate.ia.agh.edu.pl                                             *
 *                                                                             *
 * This file is part of HQEd.                                                  *
 *                                                                             *
 * HQEd is free software: you can redistribute it and/or modify                *
 * it under the terms of the GNU General Public License as published by        *
 * the Free Software Foundation, either version 3 of the License, or           *
 * (at your option) any later version.                                         *
 *                                                                             *
 * HQEd is distributed in the hope that it will be useful,                     *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU General Public License for more details.                                *
 *                                                                             *
 * You should have received a copy of the GNU General Public License           *
 * along with HQEd.  If not, see <http://www.gnu.org/licenses/>.               *
 *                                                                             *
 * Comment: Plik zwiera definicje obiektu bedacego pojemnikiem na obiekty      *
 * bedace kolejnymi poziomami ARD                                              *
 */
 
#ifndef ARD_LEVELLISTH
#define ARD_LEVELLISTH
// -----------------------------------------------------------------------------

#include <QtCore/QString>
#include <QStringList>
#include <QList>
// -----------------------------------------------------------------------------

class ARD_Level;
class ARD_Property;
class ARD_Dependency;
class ARD_Attribute;
class XTT_Attribute;
class gARDlevelList;
// -----------------------------------------------------------------------------

class ARD_LevelList : public QList<ARD_Level*>
{

private:
	
	/// Okresla status czy zawartosc pojemnika byla zmieniona
	bool fChanged;
	
	/// Tresc ostatniego bledu
	QString fError;
	
	/// Obiekt bedacy graficzna reprezentacja poziomow
	gARDlevelList* fGlevelList;
	
public:

	/// Konstruktor domyslny
	ARD_LevelList(void);
	
	/// Destruktor
	~ARD_LevelList(void);
	
	/// ----------------- Metody ustawiajace wartosci pol klasy -----------------
	
	/// Ustawia status zmiany na zadana wartosc
	void setChanged(bool);
	
	/// ----------------- Metody zwracajace wartosci pol klasy -----------------
	
	/// Zwraca status zmiany pojemnika
	bool changed(void);
	
	/// Zwraca wskaznik do obiektu bedacego graficzna reprezentacja poziomow
	gARDlevelList* gLevelList(void);
	
	/// Zwraca tresc ostatniego bledu obiektu
	QString error(void);
	
	/// ----------------- Metody operacyjne -----------------
		
	/// Oproznia pojemnik
	void flush(void);
	
	/// Dodaje nowy poziom
	ARD_Level* add(void);
	
	/// Dodaje nowy poziom
	void add(ARD_Level*);
	
	/// Usuwa wybrany poziom
	void remove(int);
	
	/// Odtworz poziomy
	void buildLevels(int);
	
	/// Tworzy pierwszy poziom
	void createBaseLevel(void);
     
     /// \brief Function that is called when some imporatant event occurs. It sends the information about event to the others object.
	///
     /// \param *__errmsgs - pointer to the string that contains the error message that are generated on event.
     /// \param __event - the event identifier
     /// \param ... - the list of event params
	/// \return true on succes otherwise false
     bool eventMessage(QString* __errmsgs, int __event, ...);
};
// -----------------------------------------------------------------------------

extern ARD_LevelList* ardLevelList;
// -----------------------------------------------------------------------------
#endif
