 /**
 *                                                                             *
 * @file ARD_Property.cpp                                                      *
 * Project name: HeKatE hqed                                                   *
 * $Id: ARD_Property.cpp,v 1.23 2009-05-13 10:28:20 kinio Exp $                 *
 * Author: Krzysztof Kaczor                                                    *
 * Date: 23.12.2007                                                            *
 *                                                                             *
 * Implementation by Krzysztof Kaczor <kinio4444@gmail.com>                    *
 *                                                                             *
 * Copyright (C) 2006-9 by the HeKatE Project                                  *
 *                                                                             *
 * HQEd has been develped within the HeKatE Project,                           *
 * see http://hekate.ia.agh.edu.pl                                             *
 *                                                                             *
 * This file is part of HQEd.                                                  *
 *                                                                             *
 * HQEd is free software: you can redistribute it and/or modify                *
 * it under the terms of the GNU General Public License as published by        *
 * the Free Software Foundation, either version 3 of the License, or           *
 * (at your option) any later version.                                         *
 *                                                                             *
 * HQEd is distributed in the hope that it will be useful,                     *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU General Public License for more details.                                *
 *                                                                             *
 * You should have received a copy of the GNU General Public License           *
 * along with HQEd.  If not, see <http://www.gnu.org/licenses/>.               *
 *                                                                             *
 * Comment: Plik zawiera definicje metod klasy ARD_Property                    *
 *                                                                             *
 */

#include <QMessageBox>
#include <QtGui>
#include <QDialog>
#include <QObject>
#include <QStringList>

#include "../../C/MainWin_ui.h"

#include "../XTT/XTT_Attribute.h"
#include "../../namespaces/ns_hqed.h"

#include "../../C/ARD/ardGraphicsScene.h"
#include "../../V/ARD/gARDnode.h"
#include "../../V/ARD/gTPHnode.h"
#include "ARD_DependencyList.h"
#include "ARD_Attribute.h"
#include "ARD_AttributeList.h"
#include "ARD_Property.h"
// -----------------------------------------------------------------------------

/**
 * Konstruktor defaultowy klasy
 * @returns Brak zwracanych wartosci
*/
ARD_Property::ARD_Property(void)
{
	fID = "";
	fTPHparent = "root";
	fChanged = false;
	fGraphicTPHnode = new gTPHnode(this);
}
// -----------------------------------------------------------------------------

/**
 * Konstruktor defaultowy klasy
 * @param QString _id - Identyfikator obiektu
 * @returns Brak zwracanych wartosci
*/
ARD_Property::ARD_Property(QString _id)
{
	fID = _id;
	fTPHparent = "root";
	fChanged = false;
	
	fGraphicTPHnode = new gTPHnode(this);
	MainWin->ard_thpScene->addItem(fGraphicTPHnode);
}
// -----------------------------------------------------------------------------

/**
 * Destruktor klasy
 * @returns Brak zwracanych wartosci
*/
ARD_Property::~ARD_Property(void)
{
	delete fGraphicTPHnode;
}
// -----------------------------------------------------------------------------

/**
 * Zwraca identyfikator obiektu
 * @returns Zwraca identyfikator obiektu, jezeli zero to id jest niezdefiniowane
*/
QString ARD_Property::id(void)
{
	return fID;
}
// -----------------------------------------------------------------------------

/**
 * Zwaraca identyfikator rodzica w diagramie TPH
 * @returns Zwaraca identyfikator rodzica w diagramie TPH, jezeli "root" to ROOT
*/
QString ARD_Property::TPHparent(void)
{
	return fTPHparent;
}
// -----------------------------------------------------------------------------

/**
 * Zwaraca czy dany obiekt byl modyfikowany
 * @returns Zwaraca czy dany obiekt byl modyfikowany
*/
bool ARD_Property::changed(void)
{
	return fChanged;
}
// -----------------------------------------------------------------------------

/**
 * Zwaraca obiekt ktory graficznie reprezentuje dany obiekt na diagramie TPH
 * @returns obiekt ktory graficznie reprezentuje dany obiekt na diagramie TPH
*/
gTPHnode* ARD_Property::graphicTPHnode(void)
{
	return fGraphicTPHnode;
}
// -----------------------------------------------------------------------------

/**
 * Zwaraca czy dany obiekt moze byc poddany dalszej transformacji (finalizacji, podzialowi)
 * @returns Zwaraca true jezeli tak, inaczej false
*/
bool ARD_Property::canBeTransformated(void)
{
	if((size() == 1) && (at(0)->type() == ARD_ATT_PHYSICAL))
		return false;
		
	return true;
}
// -----------------------------------------------------------------------------

/**
 * Ustawia wartosc statusu zmiany obiektu
 * @param bool _val - nowa wartosc statusu
 * @returns Brak zwracanych wartosci
*/
void ARD_Property::setChanged(bool _val)
{
	fChanged = _val;
}
// -----------------------------------------------------------------------------

/**
 * Ustawia wartosc identyfikatora
 * @param QString _parent - identyfikator obiektu
 * @returns brak zwracanych wartosci
*/
void ARD_Property::setID(QString _id)
{
	fID = _id;
	fChanged = true;
}
// -----------------------------------------------------------------------------

/**
 * Ustawia wartosc identyfikatora rodzica w diagramie TPH
 * @param QString _parent - identyfikator rodzica w diagramie TPH
 * @returns brak zwracanych wartosci
*/
void ARD_Property::setTPHparent(QString _parent)
{
	fTPHparent = _parent;
	fChanged = true;
}
// -----------------------------------------------------------------------------

/**
 * Dodaje nowy atrybut do obiektu
 * @param _newAttrID - identyfikator atrybutu, musi istniec w liscie atrybutow
 * @returns true jezeli uda sie dodoac, jezeli nie ma atryburu o podanym ID, lub jezeli juz atrybut jest w obiekcie to false
*/
bool ARD_Property::add(QString _newAttrID)
{
	// Wyszukiwanie w liscie atrybutow
	int index = ardAttributeList->indexOfID(_newAttrID);
	if(index == -1)
		return false;
		
	return add(ardAttributeList->at(index));
}
// -----------------------------------------------------------------------------

/**
 * Dodaje nowy atrybut do obiektu
 * @param ARD_Attribute* _newAttrPtr - wskaznik do atrybutu, musi istniec w liscie atrybutow
 * @returns true jezeli uda sie dodoac, jezeli nie ma atryburu o podanym adresie, lub jezeli juz atrybut jest w obiekcie to false
*/
bool ARD_Property::add(ARD_Attribute* _newAttrPtr)
{
	// Sprawdzanie czy juz nie ma w obiekcie
	if(contains(_newAttrPtr))
		return false;
	
	// Jezeli wskaznik na nic nie wskazuje
	if(_newAttrPtr == NULL)
		return false;
		
	// Sprawdzanie czy istnieje w liscie atrybutow
	if(!ardAttributeList->contains(_newAttrPtr))
		return false;
		
	// Dodanie obiektu
	append(_newAttrPtr);	
	
	// Zapisanie statusu zmiany
	fChanged = true;
	
	return true;
}
// -----------------------------------------------------------------------------

/**
 * Wyszukuje atrybuty na podstawie nazwy
 * @param QString _name - nazwa atrybutu
 * @returns index atrybutu o podanej nazwie, jezeli nie ma takiego atrybutu to -1
*/
int ARD_Property::indexOfName(QString _name)
{
	for(int i=0;i<size();++i)
		if(at(i)->name() == _name)
			return i;
			
	return -1;
}
// -----------------------------------------------------------------------------

/**
 * Wyszukuje atrybuty na podstawie identyfikatora
 * @param unsigned int _id - wartosc identyfikatora
 * @returns index atrybutu o podanym identyfikatorze, jezeli nie ma takiego atrybutu to -1
*/
int ARD_Property::indexOfID(QString _id)
{
	for(int i=0;i<size();++i)
		if(at(i)->id() == _id)
			return i;
			
	return -1;
}
// -----------------------------------------------------------------------------

/**
 * Sortuje atrybuty rosnaco wg nazwy
 * @returns brak zwraanych wartosci
*/
void ARD_Property::sort(void)
{
	for(int i=0;i<size();++i)
		for(int j=0;j<size()-i-1;++j)
			if(at(j)->name() > at(j+1)->name())
				swap(j, j+1);
				
	// Zapisanie statusu zmiany
	fChanged = true;
}
// -----------------------------------------------------------------------------

// #brief Static function that returns the prefix of the id.
//
// #return the prefix of the id as string.
QString ARD_Property::idPrefix(void)
{
     return "prp_";
}
// -----------------------------------------------------------------------------

// #brief Function that is called when some imporatant event occurs. It sends the information about event to the others object.
//
// #param *__errmsgs - pointer to the string that contains the error message that are generated on event.
// #param __event - the event identifier
// #param ... - the list of event params
// #return true on succes otherwise false
bool ARD_Property::eventMessage(QString* __errmsgs, int __event, ...)
{
     va_list argsptr;
     va_start(argsptr, __event);
     
     bool res = true;
     
     if(__event == XTT_SIGNAL_ATTRIBUTE_CHANGED)
          return res;
          
     va_end(argsptr);
          
     return true;
}
// -----------------------------------------------------------------------------
