 /**
 *                                                                             *
 * @file ARD_PropertyList.h                                                    *
 * Project name: HeKatE hqed                                                   *
 * $Id: ARD_PropertyList.h,v 1.20 2009-05-13 10:28:20 kinio Exp $               *
 * Author: Krzysztof Kaczor                                                    *
 * Date: 25.12.2007                                                            *
 *                                                                             *
 * Implementation by Krzysztof Kaczor <kinio4444@gmail.com>                    *
 *                                                                             *
 * Copyright (C) 2006-9 by the HeKatE Project                                  *
 *                                                                             *
 * HQEd has been develped within the HeKatE Project,                           *
 * see http://hekate.ia.agh.edu.pl                                             *
 *                                                                             *
 * This file is part of HQEd.                                                  *
 *                                                                             *
 * HQEd is free software: you can redistribute it and/or modify                *
 * it under the terms of the GNU General Public License as published by        *
 * the Free Software Foundation, either version 3 of the License, or           *
 * (at your option) any later version.                                         *
 *                                                                             *
 * HQEd is distributed in the hope that it will be useful,                     *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU General Public License for more details.                                *
 *                                                                             *
 * You should have received a copy of the GNU General Public License           *
 * along with HQEd.  If not, see <http://www.gnu.org/licenses/>.               *
 *                                                                             *
 * Comment: Plik zwiera definicje obiektu bedacego pojemnikiem na obiekty      *
 * ARD                                                                         *
 */
 
#ifndef ARD_PROPERTYLISTH
#define ARD_PROPERTYLISTH
// -----------------------------------------------------------------------------

#include <QtCore/QString>
#include <QStringList>
#include <QDomDocument>
#include <QList>
// -----------------------------------------------------------------------------

#define PROPERTY_IS_TPH_LEAF 0
#define PROPERTY_IS_NOT_TPH_LEAF 1
#define PROPERTY_IS_NOT_EXIST 2
// -----------------------------------------------------------------------------

class ARD_Property;
class ARD_Attribute;
class XTT_Attribute;

class gTPHtree;
class gARDtree;
// -----------------------------------------------------------------------------

class ARD_PropertyList : public QList<ARD_Property*>
{

private:
     
     /// Okresla status czy zawartosc pojemnika byla zmieniona
     bool fChanged;
     
     /// Zmienna odpowiadajaca za zarzadzanie diagramem ARD
     gARDtree* ARDdiagram;
     
     /// Zmienna odpowiadajaca za zarzadzanie diagramem TPH
     gTPHtree* TPHdiagram;
     
     /// Tresc ostatniego bledu
     QString fError;
     
public:

     /// Konstruktor domyslny
     ARD_PropertyList(void);
     
     /// Destruktor
     ~ARD_PropertyList(void);
     
     /// ----------------- Metody ustawiajace wartosci pol klasy -----------------
     
     /// Ustawia status zmiany na zadana wartosc
     void setChanged(bool);
     
     /// ----------------- Metody zwracajace wartosci pol klasy -----------------
     
     /// Zwraca wskaznik do obiektu rysujacego diagram TPH
     gTPHtree* tphTree(void) const;
     
     /// Zwraca tresc ostatniego bledu obiektu
     QString error(void);
     
     /// ----------------- Metody operacyjne -----------------
          
     /// Oproznia pojemnik
     void flush(void);
     
     /// Dodaje nowy atrybut
     ARD_Property* add(void);
     
     /// Dodaje nowy atrybut
     void add(ARD_Property*);
     
     /// Usuwa wybrany atrybut
     void remove(int);
     
     /// Wyszukuje obiekty zawierajace atrybut o podanej nazwie
     QList<ARD_Property*>* listOfAttributeName(QString);
     
     /// Wyszukuje obiekty zawierajace atrybut o podanym id
     QList<ARD_Property*>* listOfAttributeID(QString);
     
     /// Wyszukuje liste potomkow danego obiektu
     QList<ARD_Property*>* childList(ARD_Property*);
     
     /// Zwraca wskaznik do rodzica danej wlasnosci w diagramie tph
     ARD_Property* tphParent(ARD_Property*);
     
     /// Wyszukuje obiekty liscie diagramu TPH
     QList<ARD_Property*>* listOfNotTransformatedProperties(QString _attName = "");
     
     /// Zwraca liste lisci diagramu TPH
     QList<ARD_Property*>* listOfLeafs(void);
     
     /// \brief Functions that finds and returns the unique id
     ///
     /// \return the unique id as string
     QString findUniqID(void);
     
     /// Wyszukuje obiekty po ID
     int indexOfID(QString);
     
     /// Wyszukuje liczbe obiketow zadeklarowanych jako root'y w diagramie tph
     int rootsCount(void);
     
     /// Wyszukuje obiket bedacy root'em w diagramie tph
     int indexOfRoot(void);
     
     /// Zwraca czy atrybuty byly modyfikowane
     bool changed(void);
     
     /// Ustawia wartosc identyfikatora obiektu
     bool setPropertyID(ARD_Property* _property, QString _id);
     
     /// Ustawia wartosc rodzica obiektu w diagramie tph
     bool setTPHparent(ARD_Property*, ARD_Property*);
     
     /// Sprawdza czy dany obiekt jest lisciem w diagramie TPH
     int isLeaf(ARD_Property*);
     
     /// \brief Function that is called when some imporatant event occurs. It sends the information about event to the others object.
	///
     /// \param *__errmsgs - pointer to the string that contains the error message that are generated on event.
     /// \param __event - the event identifier
     /// \param ... - the list of event params
	/// \return true on succes otherwise false
     bool eventMessage(QString* __errmsgs, int __event, ...);
     
     /// Wcztuje dane z pliku w formacie ARDML w wersji tymczasowej
     bool fromARDML_wv(QStringList*);
     
     /// Wcztuje dane z pliku w formacie ARDML w wersji tymczasowej
     bool fromARDML_wv(QDomElement&);
     
     /// Wcztuje dane z pliku w formacie HML w wersji 1.0
     bool fromHML_1_0(QDomElement&);
     
     /// \brief Function loads all data from hml 2.0 format.
	///
	/// \param __root - reference to the parent element
     /// \param __msg - reference to the error messages
	/// \return the result code:
     /// \li 0 - on success
     /// \li 1 - on error
	int in_HML_2_0(QDomElement& __root, QString& __msg);
};
// -----------------------------------------------------------------------------

extern ARD_PropertyList* ardPropertyList;
// -----------------------------------------------------------------------------
#endif
