/*
 *        $Id: XTT_Table.cpp,v 1.28 2009-05-13 10:28:20 kinio Exp $
 *     
 *     Implementation by Krzysztof Kaczor <kinio4444@gmail.com>
 *
 *     Copyright (C) 2006-9 by the HeKatE Project
 *
 *     HQEd has been develped within the HeKatE Project,
 *     see http://hekate.ia.agh.edu.pl
 *
 *     This file is part of HQEd.
 *
 *     HQEd is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     HQEd is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with HQEd.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <QMessageBox>
#include <QtGui>
#include <QDialog>
#include <QObject>

#include <stdarg.h>

#include "../hType.h"
#include "../../namespaces/ns_hqed.h"
#include "../../C/XTT/ErrorsListWidget.h"
#include "../../C/MainWin_ui.h"
#include "../../C/Settings_ui.h"
#include "../../V/XTT/GTable.h"
#include "XTT_AttributeGroups.h"
#include "XTT_ConnectionsList.h"
#include "XTT_Table.h"
#include "XTT_TableList.h"
//---------------------------------------------------------------------------

// #brief Constructor XTT_Table class.
XTT_Table::XTT_Table(void)
{
     fNextRowId = NULL;
     fTableID = "";
     fTitle = "";
     fDescription = "";
     fnContextSize = new unsigned int[CONTEXT_COUNT];
     for(int i=0;i<CONTEXT_COUNT;i++)
          fnContextSize[i] = 0;
     fContextAtributes = NULL;
     fRowCount = 0;
     fRows = NULL;
     fPoint.setX(0);
     fPoint.setY(0);
     fWidth = NULL;
     fHeight = NULL;
     fChanged = false;
     fLevel = 0;
     fParent = 0;
     fTableType = 1;
     fAbstractColID = NULL;     
     fNextAbstractID = 1;
     fNextCellId = 0;
     fHeaderHeight = Settings->xttDefaultHeaderHeight();
     fGTable = new GTable(this);
}
//---------------------------------------------------------------------------

// #brief Constructor XTT_Table class.
// #param _tid Table id.
// #param _ptrid Pointer to the next table id.
XTT_Table::XTT_Table(unsigned int _tid, unsigned int* _prid)
{
     fNextRowId = _prid;
     fTableID = idPrefix() + QString::number(_tid);
     fTitle = "";
     fDescription = "";
     fnContextSize = new unsigned int[CONTEXT_COUNT];
     for(int i=0;i<CONTEXT_COUNT;i++)
          fnContextSize[i] = 0;
     fContextAtributes = NULL;
     fRowCount = 0;
     fRows = NULL;
     fPoint.setX(0);
     fPoint.setY(0);
     fWidth = NULL;
     fHeight = NULL;
     fChanged = false;
     fLevel = 0;
     fParent = 0;
     fTableType = 1;
     fAbstractColID = NULL;
     fNextAbstractID = 1;
     fNextCellId = 0;
     fHeaderHeight = Settings->xttDefaultHeaderHeight();
     fGTable = new GTable(this);
}
//---------------------------------------------------------------------------

// #brief Destructor XTT_Table class.
XTT_Table::~XTT_Table(void)
{    
     // Clear selection
     if(fGTable != NULL)
          fGTable->unselectAllElements();
          
     while(fRowCount > 0)
     {
          DeleteRow(0);
     }

     // Usuwanie zaalokowanych obiektow
     if(ColCount())
     {
          delete [] fContextAtributes;
          delete [] fWidth;
          delete [] fHeight;
     }
     
     delete [] fnContextSize;

     if(fRows != NULL)
          delete [] fRows;
          
     if(fGTable != NULL)
          delete fGTable;
}
//---------------------------------------------------------------------------

// #brief Function retrieves number of column.
//
// #return Number of column.
int XTT_Table::ColCount(void)
{
     int cc = 0;
     for(int i=0;i<CONTEXT_COUNT;++i)
          cc += fnContextSize[i];
          
     return cc;
}
//---------------------------------------------------------------------------

// #brief Function retrieves size of the context.
//
// #param _cid Context id.
// #return Size of the context. When context id is incorrect, function returns -1.
int XTT_Table::ContextSize(unsigned int _cindex)
{
     // Sprawdzanie indexu
     if((_cindex < 1) || (_cindex > CONTEXT_COUNT))
          return -1;

     return (int)fnContextSize[_cindex-1];
}
//---------------------------------------------------------------------------

// #brief Function retrieves last not empty context.
//
// #return Last not empty context.
int XTT_Table::LastNotEmptyContext(void)
{
     int res = CONTEXT_COUNT-1;
     while((res >= 0) && (fnContextSize[res] == 0))
     {
          res--;
     }

     return res+1;
}
//---------------------------------------------------------------------------

// #brief Function retrieves pointer to the attribute.
//
// #param _index Index of the attribute.
// #return Pointer to the attribute. When context index is incorrect, function returns -1.
XTT_Attribute* XTT_Table::ContextAtribute(unsigned int _index)
{
     // Sprawdzanie indexu
     if((((int)_index) < 0) || (_index >= (unsigned int)ColCount()))
          return NULL;

     return fContextAtributes[_index];
}
//---------------------------------------------------------------------------

// #brief Returns list of attributes in given context
//
// #param unsigned int _ctx - context identifier
// #param __uniq - indicates if the list must contain onlu uniques pointers
// #return list of attributes in given context, if context identifier is incorrect returs empty list
QList<XTT_Attribute*>* XTT_Table::ContextAtributes(unsigned int _ctx, bool __uniq)
{
     QList<XTT_Attribute*>* res = new QList<XTT_Attribute*>;
     
     if((_ctx < 1) || (_ctx > CONTEXT_COUNT))
          return res;
          
     int sindex = 0;
     for(unsigned int c=1;c<_ctx;++c)
          sindex += fnContextSize[c-1];
          
     for(unsigned int c=0;c<fnContextSize[_ctx-1];++c)
          if((!__uniq) || (res->indexOf(fContextAtributes[sindex+c]) == -1))
               res->append(fContextAtributes[sindex+c]);
     
     return res;
}
//---------------------------------------------------------------------------

// #brief Function retrieves pointer to the row of the table.
//
// #param _index Row index.
// #return Pointer to the row of the table.
XTT_Row* XTT_Table::Row(unsigned int _index)
{
     // Sprawdzanie indexu
     if((((int)_index) < 0) || (_index >= fRowCount))
          return NULL;

     return fRows[_index];
}
//---------------------------------------------------------------------------

// #brief Function determines whether the table was modyfied.
//
// #return Decision whether the table was modyfied.
bool XTT_Table::Changed(void)
{
     for(unsigned int i=0;i<fRowCount;i++)
          if(fRows[i]->Changed())
               return true;
          
     return fChanged;
}
//---------------------------------------------------------------------------

// #brief Function sets title of the table.
//
// #param _title Title of the table.
// #return No return value.
void XTT_Table::setTitle(QString _title)
{
     fTitle = _title;
     fChanged = true;
     
     connectionsDiagnostics();
}
//---------------------------------------------------------------------------

// #brief Function sets ID of the table.
//
// #param __id - new table identifier
// #return No return value.
void XTT_Table::setID(QString __id)
{    
     fTableID = __id;
     
     // Updateing list of in connections
     QList<XTT_Connections*>* ic = inConnectionsList();
     for(int i=0;i<ic->size();++i)
          ic->at(i)->setDesTable(fTableID);
     delete ic;
     
     // Updateing list of out connections
     QList<XTT_Connections*>* oc = outConnectionsList();
     for(int i=0;i<oc->size();++i)
          oc->at(i)->setSrcTable(fTableID);
     delete oc;
}
//---------------------------------------------------------------------------

// #brief Function sets describtion of the table.
//
// #param _desc Describtion of the table.
// #return No return value.
void XTT_Table::setDescription(QString _desc)
{
     fDescription = _desc;
     fChanged = true;
}
//---------------------------------------------------------------------------

// #brief Function sets coordinate of the table.
//
// #param _point Coordinate of the table.
// #return Always true.
bool XTT_Table::setLocalization(QPoint _point)
{
     fPoint = _point;
     fChanged = true;
     fGTable->RePosition();
     
     // Odswiezenie polaczen
     ConnectionsList->RepaintConnectionRelatedWithTable(fTableID);
     
     return true;
}
//---------------------------------------------------------------------------

// #brief Function sets type of the table.
//
// #param _type Type of the table.
// #return No return value.
void XTT_Table::setTableType(unsigned int _tt)
{ 
     fTableType = _tt;
     fChanged = true;
     connectionsDiagnostics();
}
//---------------------------------------------------------------------------

// #brief Function sets level of nesting of the table.
//
// #param _level Level of nesting of the table.
// #return No return value.
void XTT_Table::setLevel(unsigned int _l)
{ 
     fLevel = _l; 
     fChanged = true;
}
//---------------------------------------------------------------------------

// #brief Function sets parent id.
//
// #param _pid Parent id.
// #return No return value.
void XTT_Table::setParent(unsigned int _p)
{ 
     fParent = _p;
     fChanged = true;
}
//---------------------------------------------------------------------------

// #brief Function sets decision whether the table was modyfied.
//
// #param _b Decision whether the table was modyfied.
// #return No return value.
void XTT_Table::setChanged(bool _changed)
{ 
     for(unsigned int i=0;i<fRowCount;i++)
          fRows[i]->setChanged(_changed);
          
     fChanged = _changed;
}
//---------------------------------------------------------------------------

// #brief Function sets height of the header.
//
// #param _height Height of the header.
// #return No return value.
void XTT_Table::setHeaderHeight(unsigned int _hh)
{ 
     fHeaderHeight = _hh; 
     fChanged = true;
}
//---------------------------------------------------------------------------

// #brief Function retrieves width of the table
//
// #return Width of the table.
unsigned int XTT_Table::Width(void)
{
     unsigned int res = 0;
     for(int i=0;i<ColCount();i++)
          res += fWidth[i];

     // Na podwojna linie
     /*
     // Kiedy istnieje granica pomiedzy conditional i assert lub retract
     if((fnContextSize[0] > 0) && ((fnContextSize[1] > 0) || (fnContextSize[2] > 0)))
          res += 2;

     // Kiedy istnieje granica pomiedzy action i assert lub retract
     if((fnContextSize[3] > 0) && ((fnContextSize[1] > 0) || (fnContextSize[2] > 0)))
          res += 2;

     // Kiedy tylko wystepuja konteksty conditional i action
     if((fnContextSize[1] > 0) && (fnContextSize[1] == 0) && (fnContextSize[2] == 0) && (fnContextSize[3] > 0))
          res += 2;  */

     return res;
}
//---------------------------------------------------------------------------

// #brief Function retrieves height of the table.
//
// #return Height of the table.
unsigned int XTT_Table::Height(void)
{
     unsigned int res = fHeaderHeight;
     for(unsigned int i=0;i<fRowCount;i++)
          res += fHeight[i];

     return res;
}
//---------------------------------------------------------------------------

// #brief Function retrieves width of the column.
//
// #param _index Index of the column.
// #return Width of the column.
unsigned int XTT_Table::ColWidth(unsigned int _index)
{
     // Sprawdzanie indexu
     if((((int)_index) < 0) || (_index >= (unsigned int)ColCount()))
          return 0;

     return fWidth[_index];
}
//---------------------------------------------------------------------------

// #brief Function retrieves height of the row of the table.
//
// #param _index Index of the row.
// #return Height of the row of the table.
unsigned int XTT_Table::RowHeight(unsigned int _index)
{
     // Sprawdzanie indexu
     if((((int)_index) < 0) || (_index >= fRowCount))
          return 0;

     return fHeight[_index];
}
//---------------------------------------------------------------------------

// #brief Function sets width of the column.
//
// #param _index Index of the column.
// #param _width Width of the column.
// #return Always true.
bool XTT_Table::setColWidth(unsigned int _index, unsigned int _width)
{
     // Sprawdzanie indexu
     if((((int)_index) < 0) || (_index >= (unsigned int)ColCount()))
          return false;

     fWidth[_index] = _width;
     fChanged = true;

     return true;
}
//---------------------------------------------------------------------------

// #brief Function sets height of the row.
//
// #param _index Index of the row.
// #param _height Height of the table.
// #return Always true.
bool XTT_Table::setRowHeight(unsigned int _index, unsigned int _height)
{
     // Sprawdzanie indexu
     if((((int)_index) < 0) || (_index >= fRowCount))
          return false;

     fHeight[_index] = _height;
     fChanged = true;

     return true;
}
//---------------------------------------------------------------------------

// #brief Function sets context of the attributes.
//
// #param _cindex Index of the column
// #param _pattr Pointer to the attribute.
// #return True when pointer to the attribute is correct, otherwise function returns false.
bool XTT_Table::setContextAtributes(unsigned int _colIndex, XTT_Attribute* _newAttr)
{
     if((((int)_colIndex) < 0) || (_colIndex >= (unsigned int)ColCount()))
          return false;
     //if(_newAttr == NULL)
     //     return false;

     try
     {
          fContextAtributes[_colIndex] = _newAttr;
          for(unsigned int i=0;i<fRowCount;i++)
               fRows[i]->Cell(_colIndex)->setAttrType(_newAttr);
     }
     catch(...)
     {
          return false;
     }

     // Ze sie zmienilo
     fChanged = true;

     return true;
}
//---------------------------------------------------------------------------

// #brief Function changes attribute of the cell.
//
// #param _context Index of the context.
// #param _cindex Index of the column
// #param _pattr Pointer to the attribute.
// #return True when pointer to the attribute is correct, otherwise function returns false.
bool XTT_Table::setContextAtributes(unsigned int _context, unsigned int _colIndex, XTT_Attribute* _newAttr)
{
     // Jezeli kontekst jest niepoprawnie okreslony
     if((_context < 1) || (_context > CONTEXT_COUNT))
          return false;

     // Jezeli index kolumny jest niepopranwnie okreslony
     if((((int)_colIndex) < 0) || (_colIndex >= fnContextSize[_context]))
          return false;

     // Jezeli wskaznik nie poazujacy na cos konkretnego :)
     if(_newAttr == NULL)
          return false;

     // Wyszukanie docelowej kolumny
     int col_ind = 0;
     for(unsigned int i=0;i<(_context-1);i++)
          col_ind += fnContextSize[i];
     col_ind += _colIndex;


     // Ustawienie nowego atrybutu, w razie jakis bledow zwracamy false
     try
     {
          for(unsigned int i=0;i<fRowCount;i++)
               fRows[i]->Cell(col_ind)->setAttrType(_newAttr);
     }
     catch(...)
     {
          return false;
     }

     // Ze sie zmienilo
     fChanged = true;

     return true;
}
//---------------------------------------------------------------------------

// #brief Function retrieves context of the column.
//
// #param _cindex Index of the column.
// #return Context of the column. When index is incorrect function returns -1.
int XTT_Table::ColContext(int _cindex)
{
     // Sprawdzanie indexu
     if((_cindex < 0) || (_cindex >= ColCount()))
          return -1;

     // Wyszukiwanie kontekstu
     int res = 0;
     for(int i=0;i<CONTEXT_COUNT;i++)
     {
          res += fnContextSize[i];
          if(res > _cindex)
               return i+1;
     }

     return -1;
}
//---------------------------------------------------------------------------

// #brief Function retrieves container index of the column.
//
// #param _context Context of the column.
// #param _index Index of the column.
// #return Container index of hte column.
int XTT_Table::ColIndex(unsigned int _context, unsigned int _index)
{
     // Jezeli kontekst jest niepoprawnie okreslony
     if((_context < 1) || (_context > CONTEXT_COUNT))
          return -1;

     // Jezeli index kolumny jest niepopranwnie okreslony
     if((((int)_index) < 0) || (_index >= fnContextSize[_context]))
          return -1;

     // Sumowanie kolumn z poprzedzajacych konykstow
     int res = 0;
     for(unsigned int i=0;i<(_context-1);i++)
          res += fnContextSize[i];

     // Przesuniecie o index
     res += _index;

     return res;
}
//---------------------------------------------------------------------------

// #brief Function retrieves changes two columns with each other.
//
// #param _index1 Index of the first column.
// #param _index2 Index of the second column.
// #return True when everything is ok, otherwise function returns false.
bool XTT_Table::Exchange(unsigned int _index1, unsigned int _index2)
{
     // Sprawdzanie indexu
     if((((int)_index1) < 0) || (_index1 >= (unsigned int)ColCount()))
          return false;

     // Sprawdzanie indexu
     if((((int)_index2) < 0) || (_index2 >= (unsigned int)ColCount()))
          return false;

     // Zamienianie poszczegolnych komorek
     bool ok = true;
     for(unsigned int i=0;i<fRowCount;i++)
          ok = ok && fRows[i]->Exchange(_index1, _index2);

     // Zmiana identyfikatorow
     unsigned int tid = fAbstractColID[_index1];
     fAbstractColID[_index1] = fAbstractColID[_index2];
     fAbstractColID[_index2] = tid;

     // Ze sie zmienilo
     fChanged = true;
     
     // Odswiezenie widoku tabeli
     fGTable->RePosition();

     return ok;
}
//---------------------------------------------------------------------------

// #brief Function adds new row to the table.
//
// #return No return value.
void XTT_Table::AddRow(void)
{
     // Alokacja tymczasowej tablicy
     XTT_Row** tmp = new XTT_Row*[fRowCount+1];
     unsigned int* htmp = new unsigned int[fRowCount+1];

     // Kopiowanie wskaznikow
     for(unsigned int i=0;i<fRowCount;i++)
     {
          tmp[i] = fRows[i];
          htmp[i] = fHeight[i];
     }

     // Skasowanie strej za malej tablicy elementow
     if(fRowCount)
     {
          delete [] fHeight;
          delete [] fRows;
     }

     // Zapisanie nowej lokalizacji
     fRows = tmp;
     fHeight = htmp;

     // Utworzenie nowego elementu
     fRows[fRowCount] = new XTT_Row(fTableID, ++(*fNextRowId), &fNextCellId);
     fHeight[fRowCount] = Settings->xttDefaultRowHeight();

     // Ustawienie komorek w nowym wierszu
     for(unsigned int i=0;i<(unsigned int)ColCount();i++)
     {
          fRows[fRowCount]->Insert(i, ColContext(i));
          fRows[fRowCount]->Cell(i)->setAttrType(fContextAtributes[i]);
     }

     // Powiekszenie pojemnika
     fRowCount++;
     
     fGTable->ReCreate();
     fGTable->Update();

     // Ze sie zmienilo
     fChanged = true;
}
//---------------------------------------------------------------------------

// #brief Function removes row from the table and all connections connected to this row.
//
// #param _index Index of the row.
// #return True when everything is ok, otherwise function returns false.
bool XTT_Table::DeleteRow(unsigned int _index)
{
     // Alokacja tymczasowej tablicy
     XTT_Row** tmp = new XTT_Row*[fRowCount-1];
     unsigned int* htmp = new unsigned int[fRowCount-1];

     // Kopiowanie wskaznikow
     int tmp_i = 0;
     for(unsigned int i=0;i<fRowCount;i++)
          if(i != _index)
          {
               tmp[tmp_i] = fRows[i];
               htmp[tmp_i++] = fHeight[i];
          }

     // Skasowanie strej za malej tablicy elementow
     if(fRowCount)
     {
          delete fRows[_index];
          delete [] fHeight;
          delete [] fRows;
     }

     // Zapisanie nowej lokalizacji
     fRows = tmp;
     fHeight = htmp;

     // Powiekszenie pojemnika
     fRowCount--;

     // Ze sie zmienilo
     fChanged = true;
     
     // Odsiezenie widoku
     TableList->Refresh(TableList->IndexOfID(fTableID));

     return true;
}
//---------------------------------------------------------------------------

// #brief Function inserts new column to the table.
//
// #param _index Index of the column.
// #return True when index is correct, otherwise function returns false.
bool XTT_Table::InsertCol(unsigned int _index)
{
     // Sprawdzanie indexu
     if((((int)_index) < 0) || (_index > (unsigned int)ColCount()))
          return false;

     // Alokacja pamieci na nowa szerokosc
     unsigned int* wtmp = new unsigned int[ColCount()+1];
     unsigned int* aidtmp = new unsigned int[ColCount()+1];
     XTT_Attribute** ftmp = new XTT_Attribute*[ColCount()+1];

     // Kopiowanie szerokosci
     int tmp_i = 0;
     for(int i=0;i<=ColCount();i++)
          if(i != (int)_index)
          {
               wtmp[i] = fWidth[tmp_i];
               aidtmp[i] = fAbstractColID[tmp_i];
               ftmp[i] = fContextAtributes[tmp_i++];
          }

     // Skasowanie starej tablicy szerokosci
     if(ColCount())
     {
          delete [] fWidth;
          delete [] fContextAtributes;
          delete [] fAbstractColID;
     }

     // Zapisanie nowej okalizacji tablicy
     fWidth = wtmp;
     fContextAtributes = ftmp;
     fAbstractColID = aidtmp;

     // Nowa wartosc domyslna szerokosci
     fWidth[_index] = Settings->xttDefaultColumnWidth();
     fAbstractColID[_index] = fNextAbstractID++;
     fContextAtributes[_index] = NULL;

     // Dodawanie nowej kolumny
     int ctx = LastNotEmptyContext();
     if(_index < (unsigned int)ColCount())
          ctx = ColContext(_index);
     for(unsigned int i=0;i<fRowCount;i++)
     {
          fRows[i]->Insert(_index, ctx);
          fRows[i]->Cell(_index)->setAttrType(fContextAtributes[_index]);
     }
          
     // Powiekszenie kontekstu
     fnContextSize[ctx-1]++;

     // Ze sie zmienilo
     fChanged = true;
     
     return true;
}
//---------------------------------------------------------------------------

// #brief Function inserts new row to the table.
//
// #param _tidx Index of the row of the table.
// #return True when index is ok, otherwise function returns false.
bool XTT_Table::insertRow(int _index)
{
     // Jezeli ondex wstawiania jest niepoprawny
     if((_index < 0) || (_index > (int)fRowCount))     
          return false;
          
     // Jezeli wszystko ok to:
     
     // Dodajemy wiersz
     AddRow();
     
     // Przesuwamy go na zadana pozycje
     for(int ci=(int)fRowCount-1;ci>_index;ci--)
          ExchangeRows(ci, ci-1);
     
     // Odsiezenie widoku
     TableList->Refresh(TableList->IndexOfID(fTableID));
          
     return true;
}
//---------------------------------------------------------------------------

// #brief Function removes column from the table.
//
// #param _index Index of the column.
// #return True when index is correct, otherwise function returns false.
bool XTT_Table::DeleteCol(unsigned int _index)
{
     // Sprawdzanie indexu
     if((((int)_index) < 0) || (_index >= (unsigned int)ColCount()))
          return false;

     // Zapisanie ilosc kolumn
     int cc = ColCount();

     // Pomniejszenie kontekstu
     fnContextSize[ColContext(_index)-1]--;

     // Alokacja pamieci na nowa szerokosc
     unsigned int* wtmp = new unsigned int[cc-1];
     unsigned int* aidtmp = new unsigned int[cc-1];
     XTT_Attribute** ftmp = new XTT_Attribute*[cc-1];

     // Kopiowanie szerokosci
     int tmp_i = 0;
     for(int i=0;i<cc;i++)
          if(i != (int)_index)
          {
               wtmp[tmp_i] = fWidth[i];
               aidtmp[tmp_i] = fAbstractColID[i];
               ftmp[tmp_i++] = fContextAtributes[i];
          }

     // Skasowanie starej tablicy szerokosci
     if(cc)
     {
          delete [] fWidth;
          delete [] fContextAtributes;
          delete [] fAbstractColID;
     }

     // Zapisanie nowej okalizacji tablicy
     fWidth = wtmp;
     fContextAtributes = ftmp;
     fAbstractColID = aidtmp;

     // Usuwanie kolumny
     for(unsigned int i=0;i<fRowCount;i++)
          fRows[i]->Delete(_index);

     // Ze sie zmienilo
     fChanged = true;
     
     // Odsiezenie widoku
     TableList->Refresh(TableList->IndexOfID(fTableID));
     
     return true;
}
//---------------------------------------------------------------------------

// #brief Function inserts column of the new context.
//
// #param _ctx Index of the context.
// #return True when column was inserted. False when context of the column already exists.
bool XTT_Table::InsertNewContext(unsigned int _ctx)
{
     // Sprawdzanie indexu
     if((_ctx < 1) || (_ctx > CONTEXT_COUNT))
          return false;

     if(fnContextSize[_ctx-1] > 0)
          return false;

     // Wyszukiwanie indexu wstawienia
     unsigned int _index = 0;
     for(unsigned int i=0;i<(_ctx-1);i++)
          _index += fnContextSize[i];

     // Alokacja pamieci na nowa szerokosc
     unsigned int* wtmp = new unsigned int[ColCount()+1];
     unsigned int* aidtmp = new unsigned int[ColCount()+1];
     XTT_Attribute** ftmp = new XTT_Attribute*[ColCount()+1];

     // Kopiowanie szerokosci
     int tmp_i = 0;
     for(int i=0;i<=ColCount();i++)
          if(i != (int)_index)
          {
               wtmp[i] = fWidth[tmp_i];
               aidtmp[i] = fAbstractColID[tmp_i];
               ftmp[i] = fContextAtributes[tmp_i++];
          }

     // Skasowanie starej tablicy szerokosci
     if(ColCount())
     {
          delete [] fWidth;
          delete [] fContextAtributes;
          delete [] fAbstractColID;
     }
     
     // Zapisanie nowej lokalizacji tablicy
     fWidth = wtmp;
     fContextAtributes = ftmp;
     fAbstractColID = aidtmp;

     // Nowa wartosc domyslna szerokosci
     fWidth[_index] = Settings->xttDefaultColumnWidth();
     fAbstractColID[_index] = fNextAbstractID++;
     fContextAtributes[_index] = NULL;

     // Dodawanie nowej kolumny
     for(unsigned int i=0;i<fRowCount;i++)
     {
          fRows[i]->Insert(_index, _ctx);
          fRows[i]->Cell(_index)->setAttrType(fContextAtributes[_index]);
     }

     // Powiekszenie kontekstu
     fnContextSize[_ctx-1]++;

     // Ze sie zmienilo
     fChanged = true;

     return true;
}
//---------------------------------------------------------------------------

// #brief Function moves column in other place.
//
// #param _from Index of the source column.
// #param _to Index fo the destiny column.
// #return True when everything is ok, otherwise function returns false.
bool XTT_Table::Move(unsigned int from, unsigned int to)
{
     // Sprawdzanie indexow
     if((((int)from) < 0) || (from >= (unsigned int)ColCount()))
          return false;
     if((((int)to) < 0) || (to >= (unsigned int)ColCount()))
          return false;
     if(from == to)
          return true;

     // Przyrost
     int dx = 1;
     if(from > to)
          dx = -1;

     // Przenoszenie
     bool ok = true;
     for(unsigned int r=0;r<fRowCount;r++)                               // Dla kazdego wiersza
     {
          // Wskaznik do komorki zrodlowej
          XTT_Cell* ptr = fRows[r]->Cell(from);
          
          for(unsigned int i=from;i!=to;i+=dx)
               ok = ok && fRows[r]->setCell(i, fRows[r]->Cell(i+dx));
          ok = ok && fRows[r]->setCell(to, ptr);
     }

     // Przenoszenie ID i atrybutow
     unsigned int tmp_col_id = fAbstractColID[from];
     XTT_Attribute* tmp_c_a = fContextAtributes[from];
     for(unsigned int i=from;i!=to;i+=dx)
     {
          fAbstractColID[i] = fAbstractColID[i+dx];
          fContextAtributes[i] = fContextAtributes[i+dx];
     }
     fAbstractColID[to] = tmp_col_id;
     fContextAtributes[to] = tmp_c_a;

     return ok;
}
//---------------------------------------------------------------------------

// #brief Function sets context for the column.
//
// #param _cidx Index of the column.
// #param _ctx Context.
// #return No return value.
void XTT_Table::setColContext(unsigned int _col_index, unsigned int _ctx)
{
     // Sprawdzanie indexow
     if((((int)_col_index) < 0) || (_col_index >= (unsigned int)ColCount()))
          return;
     if((_ctx < 1) || (_ctx > CONTEXT_COUNT))
          return;

     fnContextSize[ColContext(_col_index)-1]--;
     fnContextSize[_ctx-1]++;

     for(unsigned int i=0;i<fRowCount;i++)
          fRows[i]->Cell(_col_index)->setContext(_ctx);
}
//---------------------------------------------------------------------------

// #brief Function sets size of the context.
//
// #param _ctx Index of the context.
// #param _size Size of the context.
// #return No return value.
void XTT_Table::setContextSize(unsigned int _ctx, unsigned int _size)
{
     if((_ctx < 1) || (_ctx > CONTEXT_COUNT))
          return;

     fnContextSize[_ctx-1] = _size;
}
//---------------------------------------------------------------------------

// #brief Function retrieves index of column.
//
// #param _cid Column id.
// #return Index of the column. When id is incorrect function returns -1.
int XTT_Table::IndexOfColId(unsigned int _a_c_id)
{
     for(int i=0;i<ColCount();i++)
          if(fAbstractColID[i] == _a_c_id)
               return i;

     return -1;
}
//---------------------------------------------------------------------------

// #brief Function retrieves index of column that conatin the given cell.
//
// #param __ptr - pointer to the cell.
// #return Index of the column. When table doesn't contain given cell, function returns -1.
int XTT_Table::indexOfCol(XTT_Cell* __ptr)
{
     for(unsigned int i=0;i<RowCount();++i)
     {
          int ri = fRows[i]->indexOfCell(__ptr);
          if(ri > -1)
               return ri;
     }
     return -1;
}
//---------------------------------------------------------------------------

// #brief Function retrieves column id.
//
// #param _index Index of the column.
// #return Column id.
int XTT_Table::AbstractColID(unsigned int _index)
{
     if((((int)_index) < 0) || (_index >= (unsigned int)ColCount()))
          return -1;

     return fAbstractColID[_index];
}
//---------------------------------------------------------------------------

// #brief Function retrieves index of the table.
//
// #param id Attribute id.
// #return Index of the table. When id is incorrect function returns -1.
int XTT_Table::IndexOfAttributeID(QString id)
{
     for(int i=0;i<ColCount();i++)
          if(fContextAtributes[i]->Id() == id)
               return i;

     return -1;
}
//---------------------------------------------------------------------------

// #brief Function retrieves index of the attribute in given context.
//
// #param _aid Attribute id.
// #param __ctx - context
// #return Index of attribute in the table, but within given context
int XTT_Table::IndexOfAttributeID(QString _aid, int __ctx)
{
     if((__ctx < 1) || (__ctx > CONTEXT_COUNT))
          return -1;
          
     int start = 0;
     int end = 0;
     for(int i=1;i<__ctx;++i)
          start += ContextSize(i);
     end = start + ContextSize(__ctx);
     
     for(int i=start;i<end;i++)
          if(fContextAtributes[i]->Id() == _aid)
               return i;
      
     return -1;
}
//---------------------------------------------------------------------------

// #brief Function exchanges two rows.
//
// #param _index1 Index of the firs row.
// #param _index2 Index of the second row.
// #param _keepConnections Determines whether connections should be refreshed.
// #return True when indexes are ok, otherwise function returns false.
bool XTT_Table::ExchangeRows(unsigned int _index1, unsigned int _index2, bool _keepConnections)
{
     if((((int)_index1) < 0) || (_index1 >= fRowCount))
          return false;
     if((((int)_index2) < 0) || (_index2 >= fRowCount))
          return false;
     
     // Zamiana wskaznikow do wierszy
     XTT_Row* tmp = fRows[_index1];
     fRows[_index1] = fRows[_index2];
     fRows[_index2] = tmp;
     
     // Zamiana wysokosci wierszy
     unsigned int tmph = fHeight[_index1];
     fHeight[_index1] = fHeight[_index2];
     fHeight[_index2] = tmph;
     
     // Jezeli nalezy odswiezyc polaczenia
     if(_keepConnections)
          ConnectionsList->RepaintConnectionRelatedWithTable(fTableID);
     
     // Sprawdzenie poprawnosci polaczen
     connectionsDiagnostics();
     
     // Odswiezenie widoku tabeli
     fGTable->RePosition();

     return true;
}
//---------------------------------------------------------------------------

// #brief Function retrieves index of the firs row that has any connections.
//
// #return Index of the firs row that has any connections.
int XTT_Table::firstConnectionRowIndex(void)
{     
     int res = -1;
     
     for(unsigned int i=0;i<RowCount();i++)
     {
          QList<XTT_Connections*>* conn_list = Row(i)->InConnection();
          if(conn_list->size() > 0)
               res = i;
          delete conn_list;
          
          if(res > -1)
               return res;
     }
     
     return res;
}
//---------------------------------------------------------------------------

// #brief Function retrieves index of the row.
//
// #param _rid Row id.
// #return Index of the row.
int XTT_Table::indexOfRowID(QString _id)
{
     for(unsigned int i=0;i<RowCount();++i)
          if(fRows[i]->RowId() == _id)
               return i;
               
     return -1;
}
//---------------------------------------------------------------------------

// #brief Function retrieves index of the row that contains the given cell.
//
// #param __ptr - pointer to the cell
// #return Index of the row.
int XTT_Table::indexOfRow(XTT_Cell* __ptr)
{
     for(unsigned int i=0;i<RowCount();++i)
          if(fRows[i]->indexOfCell(__ptr) > -1)
               return i;
               
     return -1;
}
//---------------------------------------------------------------------------

// #brief Function retrieves pointer to the list of input connections of the table.
//
// #return Pointer to the list of input connections of the table.
QList<XTT_Connections*>* XTT_Table::inConnectionsList(void)
{
     return ConnectionsList->listOfInConnections(fTableID);
}
//---------------------------------------------------------------------------

// #brief Function retrieves pointer to the list of output connections of the table.
//
// #return Pointer to the list of output connections of the table.
QList<XTT_Connections*>* XTT_Table::outConnectionsList(void)
{
     return ConnectionsList->listOfOutConnections(fTableID);
}
//---------------------------------------------------------------------------

// #brief Function checks correctness of connections of the table.
//
// #param *__error_msgs - pointer to the string where the error messages will be stored
// #return true if ok, otherwise false
bool XTT_Table::connectionsDiagnostics(QString* __error_msgs)
{
     bool res = true;
     QString errors = "";

     // Okresla czy wsrod polaczen przychodzacych znajduje sie chociaz jedno polaczenie nie etykietowane, jezeli tak
     // to wszystkie polaczenia bez wzgledu na etykiete moga zostac uruchomione
     bool existsNotLabeled = false;

     // Lista wszystkich wchodzacych polaczen
     QList<XTT_Connections*>* _iclist = new QList<XTT_Connections*>;

     // Lista polaczen znalezionych w danej iteracji
     QList<XTT_Connections*>* currentRowInConnections = NULL;
     QList<XTT_Connections*>* currentRowOutConnections = NULL;

     for(unsigned int i=0;i<RowCount();++i)
     {
          // Continue Main Loop
          bool cml = false;

          // Wyszukanie listy polaczen przychodzacych do aktualnego wiersza
          currentRowInConnections = Row(i)->InConnection();
          currentRowOutConnections = Row(i)->OutConnection();

          (*_iclist) << (*currentRowInConnections);

          // Jezeli tabela jest tabela incjalizacyjna to z danego wiersza moze wychodzic
          // conajwyzej jedno polaczenie, w innym przypadku nie wiadomo ktore wybrac
          if((fTableType == 0) && (currentRowOutConnections->size() > 1))
          {
               QString _Error = "Ambiguous connections. The row can has only one outgoing connection.";
               if(Row(i)->Length() > 0)
                    errors += hqed::createErrorString(fTableID, Row(i)->Cell(Row(i)->Length()-1)->CellId(), "\n" + _Error, WARNING_TYPE, 1);
                    
               res = false;
               cml = true;
          }

          // Jezeli nie ma jeszcze zadnych polaczen wchodzacych a istneija polaczenia wychodzace
          // to jezeli tabela jest inicjalizacyjna to ok
          if((fTableType == 0) && (currentRowOutConnections->size() == 1))
               cml = true;

          // Jezeli nie ma jeszcze zadnych polaczen wchodzacych a istneija polaczenia wychodzace
          // to jezeli tabela nie jest inicjalizacyjna to ostrzezenie
          if((fTableType != 0) && (_iclist->size() == 0) && (currentRowOutConnections->size() == 1))
          {
               QString _Error = "There is no possibility on use connection \"" + currentRowOutConnections->at(0)->Label() + "\", unless this table type is \"Initialize\".";
               if(Row(i)->Length() > 0)
                    errors += hqed::createErrorString(fTableID, Row(i)->Cell(Row(i)->Length()-1)->CellId(), "\n" + _Error, WARNING_TYPE, 1);
               
               res = false;
               cml = true;
          }

          // Jezeli nie ma jeszcze zadnych polaczen wchodzacych to kazdy row jest niedostepny
          bool iaccesiblerow = ((fTableType != 0) && (_iclist->size() == 0));
          if(iaccesiblerow)
          {
               QString _Error = "Inaccessible row.";
               if(Row(i)->Length() > 0)
                    errors += hqed::createErrorString(fTableID, Row(i)->Cell(Row(i)->Length()-1)->CellId(), "\n" + _Error, WARNING_TYPE, 5);
               
               res = false;
               cml = true;
          }

          // Sprawdzanie czy wsrod polaczen wychodzacych nie ma dwoch o takich samych etykietach
          for(int foc=0;foc<currentRowOutConnections->size() && (!existsNotLabeled) && (!cml);++foc)
               for(int soc=foc+1;soc<currentRowOutConnections->size();++soc)
                    if(currentRowOutConnections->at(foc)->Label() == currentRowOutConnections->at(soc)->Label())
                    {
                         QString _Error = "Ambiguous connections, there are two connections with the same label \"" + currentRowOutConnections->at(soc)->Label() + "\".";
                         
                         // Dodanie informacji ostrzezenia
                         if(Row(i)->Length() > 0)
                              errors += hqed::createErrorString(fTableID, Row(i)->Cell(Row(i)->Length()-1)->CellId(), "\n" + _Error, WARNING_TYPE, 1);
                         res = false;
                    }

          // Sprawdzanie czy znajduje sie polaczenie nie etykietowane, jezeli tak to nie ma sensu
          // dalej sprawdzac pod tym katem
          for(int ic=0;(ic<currentRowInConnections->size()) && (!existsNotLabeled) && (!cml);++ic)
               if(currentRowInConnections->at(ic)->Label() == "")
                    existsNotLabeled = true;

          // Jezeli istnieja polaczenia nie etykietowane, oraz z ktoregos wiersza wychodzi wiecej niz jedno polaczenie
          // to w przypadku uzycia takiego polaczenia nie etykietowanego nie jest wiadomo ktore z tych wychodzacych
          // polaczen trzeba uzyc
          if((existsNotLabeled) && (currentRowOutConnections->size() > 1))
          {
               QString _Error = "Ambiguous connections, which one should be used ? Check not labeled connections.";
               if(Row(i)->Length() > 0)
                    errors += hqed::createErrorString(fTableID, Row(i)->Cell(Row(i)->Length()-1)->CellId(), "\n" + _Error, WARNING_TYPE, 1);
                         
               res = false;
               cml = true;
          }

          // Jezeli istneije polaczenie bez etykiety to nie ma sensu sprawdzac dalej bool
          // kazde polaczenie ma szanse byc uruchomione
          if(existsNotLabeled)
               continue;

          /*
          // Sprawdzanie polaczen, czy kazde moze byc uruchomione
          for(int c=0;c<currentRowOutConnections->size() && (!existsNotLabeled) && (!cml);++c)
          {
               // Jezeli jest to polaczenie nie etykietowane to moze byc uzyte 
               // bez wzgledu na etykiete polaczenia wejsciowego
               if(currentRowOutConnections->at(c)->Label() == "")
                    cml = true;
                    
               // Jezeli jest to polaczenie etykietowane to sprawdzamy czy istneije polaczenie 
               // przychodzace o tej etykiecie i nie jest to to samo polaczenie (loop)
               for(int ini=0;ini<_iclist->size();++ini)
                    if((_iclist->at(ini) != currentRowOutConnections->at(c)) && (_iclist->at(ini)->Label() == currentRowOutConnections->at(c)->Label()))
                         cml = true;
                         
               if(!cml)
               {
                    QString _Error = "Probably impossible execution of connection \"" + currentRowOutConnections->at(c)->Label() + "\".";
                    if(Row(i)->Length() > 0)
                         MainWin->xtt_errorsList->add(fTableID, Row(i)->Cell(Row(i)->Length()-1)->CellId(), _Error, WARNING_TYPE, 1);
               }
          }*/

          delete currentRowInConnections;
          delete currentRowOutConnections;
     }
     
     if(__error_msgs != NULL)
          *__error_msgs += errors;
     
     delete _iclist;
     
     return res;
}
//---------------------------------------------------------------------------

// #brief Function that returns if the row object definition is correct
//
// #param __error_message - the reference to the error message(s)
// #return true if yes otherwise false
bool XTT_Table::tableVerification(QString& __error_message)
{
     bool res = true;
     
     // Checking the compatybility of an attribute class with the column context
     QList<XTT_Attribute*>* cca = ContextAtributes(1);
     QList<XTT_Attribute*>* dca = ContextAtributes(4);
     
     // The initial table can't have conditional part
     if((cca->size() > 0) && (TableType() == 0))
     {
          QString _Error = "Initial table cannot have the conditional part.";
          __error_message += hqed::createErrorString(fTableID, "", _Error, ERROR_TYPE, 3);
          res = false;
     }
     
     for(int i=0;i<cca->size();++i)
     {
          // The conditional context can not contain the WriteOlny WO attribute
          if(cca->at(i)->Class() == XTT_ATT_CLASS_WO)
          {
               QString _Error = "Incorrect use attribute \'" + cca->at(i)->Name() + "\' in conditional context. The " + XTT_Attribute::classString(XTT_ATT_CLASS_WO) + " attribute can not be used in this context.";
               __error_message += hqed::createErrorString(fTableID, QString::number(i), _Error, ERROR_TYPE, 4);
               res = false;
          }
     }
     
     for(int i=0;i<dca->size();++i)
     {
          // The action context can not contain the ReadOlny RO attribute at least this is initial table
          if((dca->at(i)->Class() == XTT_ATT_CLASS_RO) && (fTableType != 0))
          {
               QString _Error = "Incorrect use attribute \'" + dca->at(i)->Name() + "\' in decision context. The " + XTT_Attribute::classString(XTT_ATT_CLASS_RO) + " attribute can not be used in this context.";
               __error_message += hqed::createErrorString(fTableID, QString::number(cca->size()+i), _Error, ERROR_TYPE, 4);
               res = false;
          }
     }
     
     delete cca;
     delete dca;

     for(unsigned int a=0;a<fRowCount;a++)
          res = fRows[a]->rowVerification(__error_message) && res;

     res = connectionsDiagnostics(&__error_message) && res;
     
     return res;
}
//---------------------------------------------------------------------------

// #brief Function that updates int all the types the pointer to the new type
//
// #param hType* __old - the value of the old pointer
// #param hType* __new - the value of the new pointer
// #return true on succes otherwise false
bool XTT_Table::updateTypePointer(hType* __old, hType* __new)
{
     bool res = true;
     
     for(unsigned int a=0;a<fRowCount;a++)
          res = fRows[a]->updateTypePointer(__old, __new) && res;
     
     return res;
}
//---------------------------------------------------------------------------

// #brief Function that is called when some imporatant event occurs. It sends the information about event to the others object.
//
// #param *__errmsgs - pointer to the string that contains the error message that are generated on event.
// #param __event - the event identifier
// #param ... - the list of event params
// #return true on succes otherwise false
bool XTT_Table::eventMessage(QString* __errmsgs, int __event, ...)
{
     va_list argsptr;
     va_start(argsptr, __event);
     
     bool res = true;
     
     if(__event == XTT_SIGNAL_ATTRIBUTE_CHANGED)
          return res;
          
     for(unsigned int a=0;a<fRowCount;a++)
          res = fRows[a]->eventMessage(__errmsgs, __event, argsptr) && res;
          
     va_end(argsptr);
          
     return true;
}
// -----------------------------------------------------------------------------

// #brief Function that maps the rows to the prolog code.
//
// #return The prolog code that represents the rules as string.
QString XTT_Table::rulesToProlog(void)
{    
     QString result = "";
     
     for(unsigned int r=0;r<fRowCount;++r)
          result += fRows[r]->toProlog() + "\n";
     
     return result;
}
//---------------------------------------------------------------------------

// #brief Function that maps the table schema to the prolog code.
//
// #return The prolog code that represents the schema of the table as string.
QString XTT_Table::schemaToProlog(void)
{
     QString result = "";
     QString condSchema = "";
     QString actnSchema = "";
     
     // creating schema
     QList<XTT_Attribute*>* ac = ContextAtributes(1, true);
     QList<XTT_Attribute*>* cc = ContextAtributes(4, true);
     
     for(int i=0;i<ac->size();++i)
     {
          condSchema += hqed::mcwp(ac->at(i)->Name());
          if(i < (ac->size()-1))
               condSchema += ",";
     }
     for(int i=0;i<cc->size();++i)
     {
          actnSchema += hqed::mcwp(cc->at(i)->Name());
          if(i < (cc->size()-1))
               actnSchema += ",";
     }
     
     delete ac;
     delete cc;
     
     // creating string schema
     result += "xschm " + hqed::mcwp(Title()) + ": ";
     result += "[" + condSchema + "] ==> [" + actnSchema + "].";
     
     return result;
}
//---------------------------------------------------------------------------

// #brief Function that maps the FACT tables to the prolog states code.
//
// #return The prolog code that represents the states of the tables as string.
QString XTT_Table::statesToProlog(void)
{
     if(TableType() != 0)
          return "";
          
     QString result = "";
     
     for(unsigned int r=0;r<fRowCount;++r)
          result += fRows[r]->statesToProlog();
     
     return result;
}
//---------------------------------------------------------------------------

// #brief Static function that returns the prefix of the id.
//
// #return the prefix of the id as string.
QString XTT_Table::idPrefix(void)
{
     return "tab_";
}
//---------------------------------------------------------------------------

// #brief Static function that returns the prefix of the id of state table.
//
// #return the state prefix of the id as string.
QString XTT_Table::stateIdPrefix(void)
{
     return "sts_";
}
//---------------------------------------------------------------------------

// #brief Function reads data from file in XTTML 1.0 format.
//
// #param root Reference to the section that concerns this attribute.
// #return True when data is ok, otherwise function returns false.
bool XTT_Table::FromXTTML_2_0(QDomElement& root)      
{
     // Resetowanie ilosci kontekstow
     fnContextSize[0] = 0;
     fnContextSize[1] = 0;
     fnContextSize[2] = 0;
     fnContextSize[3] = 0;

     bool res = true;
     
     if(root.tagName().toLower() != "xtt_table")
          return false;

     // Reszta zakladamy ze jest ok
     
     QStringList tableTypes;
     tableTypes << "initial" << "middle" << "halt";

     // Okresla ktra wartosc wczytujemy z wartosci indexowanych, jak np fContextAtributes, lub fnContextSize
     int index;

     // Wczytywanie parametrow tagu glownego
     QString tableType = root.attribute("table_type", "#NAN#");
     QString tableTitle = root.attribute("name", "#NAN#");
     
     // Usuwanie spacji z wczytanych wartosci
     for(;tableType.contains(" ");)
          tableType = tableType.remove(tableType.indexOf(" "), 1);
     for(;tableTitle.contains(" ");)
          tableTitle = tableTitle.remove(tableTitle.indexOf(" "), 1);
     
     // Oba parametry sa wymagane wiec sprawdzanie czy wystepuja
     if((tableType == "") || (tableType == "#NAN#"))
     {
          QMessageBox::critical(NULL, "HQEd", "Missing table parameter \"table_type\" at line " + QString::number(root.lineNumber()) + ".", QMessageBox::Ok);
          return false;
     }
     
     // Jezeli okreslono typ tabeli to trzeba sprawdzic czy poprawnie
     if(!tableTypes.contains(tableType, Qt::CaseInsensitive))
     {
          QMessageBox::critical(NULL, "HQEd", "Incorrect table type: \"" + tableType + "\" at line " + QString::number(root.lineNumber()) + ".", QMessageBox::Ok);
          return false;
     }
     
     // Sprawdzanie podania nazwy tabeli
     if((tableTitle == "") || (tableTitle == "#NAN#"))
     {
          QMessageBox::critical(NULL, "HQEd", "Missing table parameter \"name\" at line " + QString::number(root.lineNumber()) + ".", QMessageBox::Ok);
          return false;
     }
     
     fTitle = tableTitle;
     fTableType = tableTypes.indexOf(tableType.toLower());
     
     // "<row"
     QDomElement DOMrow = root.firstChildElement("row");
     while(!DOMrow.isNull())
     {
          // Kiedy linia konczaca linie danych
          // Dodajemy nowy wiersz
          AddRow();

          // Wczytujemy dane z pliku
          res = res && fRows[fRowCount-1]->FromXTTML_2_0(DOMrow);
          
          // Wczytanie kolejnego wiersza
          DOMrow =  DOMrow.nextSiblingElement("row");
     }

     // "<description>"
     QString tDesc = root.firstChildElement("description").text();
     if(!tDesc.isEmpty())
          fDescription = tDesc;

     // "<context_size"
     QDomElement DOMcs = root.firstChildElement("context_size");
     if(!DOMcs.isNull())
     {
          QString sConditional = DOMcs.attribute("conditional", "#NAN#");
          QString sAssert = DOMcs.attribute("assert", "#NAN#");
          QString sRetract = DOMcs.attribute("retract", "#NAN#");
          QString sAction = DOMcs.attribute("action", "#NAN#");
          
          // Podanie nie jest wymagane, jezeli parametr nie jest podany to przyjmujemy wielkosc za 0
          // Usuwanie spacji z wczytanych wartosci
          for(;sConditional.contains(" ");)
               sConditional = sConditional.remove(sConditional.indexOf(" "), 1);
          for(;sAssert.contains(" ");)
               sAssert = sAssert.remove(sAssert.indexOf(" "), 1);
          for(;sRetract.contains(" ");)
               sRetract = sRetract.remove(sRetract.indexOf(" "), 1);
          for(;sAction.contains(" ");)
               sAction = sAction.remove(sAction.indexOf(" "), 1);
               
          int iConditional = 0;
          int iAssert = 0;
          int iRetract = 0;
          int iAction = 0;
               
          // Proby konwersji jezeli podano
          if((sConditional != "") && (sConditional != "#NAN#"))
          {
               bool ok;
               iConditional = sConditional.toInt(&ok);
               if((!ok) || (iConditional < 0))
               {
                    QMessageBox::information(NULL, "HQEd", "Incorrect table \"" + fTitle + "\" conditional context size: \"" + sConditional + "\" at line " + QString::number(DOMcs.lineNumber()) + ".\nContext will be set as zero.", QMessageBox::Ok);
                    iConditional = 0;
               }
          }
          if((sAssert != "") && (sAssert != "#NAN#"))
          {
               bool ok;
               iAssert = sAssert.toInt(&ok);
               if((!ok) || (iAssert < 0))
               {
                    QMessageBox::information(NULL, "HQEd", "Incorrect table \"" + fTitle + "\" assert context size: \"" + sAssert + "\" at line " + QString::number(DOMcs.lineNumber()) + ".\nContext will be set as zero.", QMessageBox::Ok);
                    iAssert = 0;
               }
          }
          if((sRetract != "") && (sRetract != "#NAN#"))
          {
               bool ok;
               iRetract = sRetract.toInt(&ok);
               if((!ok) || (iRetract < 0))
               {
                    QMessageBox::information(NULL, "HQEd", "Incorrect table \"" + fTitle + "\" retract context size: \"" + sRetract + "\" at line " + QString::number(DOMcs.lineNumber()) + ".\nContext will be set as zero.", QMessageBox::Ok);
                    iRetract = 0;
               }
          }
          if((sAction != "") && (sAction != "#NAN#"))
          {
               bool ok;
               iAction = sAction.toInt(&ok);
               if((!ok) || (iAction < 0))
               {
                    QMessageBox::information(NULL, "HQEd", "Incorrect table \"" + fTitle + "\" action context size: \"" + sAction + "\" at line " + QString::number(DOMcs.lineNumber()) + ".\nContext will be set as zero.", QMessageBox::Ok);
                    iAction = 0;
               }
          }
          
          fnContextSize[0] = iConditional;
          fnContextSize[1] = iAssert;
          fnContextSize[2] = iRetract;
          fnContextSize[3] = iAction;
          
          // Suma kontekstow jest rowna ilosci kolumn, teraz sprawdzamy czy tak jest rzeczywiscie
          // NAJPIERW MUSZA ZOSTAC WCZYTANE WIERSZE     
          for(int r=0;r<(int)fRowCount;r++)
          {
               if(((int)fRows[r]->Length()) != ColCount())
               {
                    //QMessageBox::critical(NULL, "HQEd", "To long/short row " + QString::number(r+1) + ". Row length was declared as " + QString::number(ColCount()) + ". Error in table \"" + fTitle + "\".", QMessageBox::Ok);
                    return false;
               }
          }
          
          // Jezeli wszystko ok to przystepujemy do alokacji pamieci na pozostale parametry
          fWidth = new unsigned int[ColCount()];
          fAbstractColID = new unsigned int[ColCount()];
          fContextAtributes = new XTT_Attribute*[ColCount()];

          // Ustawianie wartosci domyslnych kolumn i ich kontekstow
          for(int i=0;i<ColCount();i++)
          {
               fWidth[i] = Settings->xttDefaultColumnWidth();
               fAbstractColID[i] = fNextAbstractID++;
               setColContext(i, ColContext(i));
          }
     }

     // "<column_attributes>"
     QDomElement DOMcolattrs = root.firstChildElement("column_attributes");
     QDomElement DOMcolwidths = root.firstChildElement("col_widths");
     QDomElement DOMrowheights = root.firstChildElement("nrow_heights");

     // Wczytanie atrybutow dla kolumn
     index = 0;
     QDomElement DOMca = DOMcolattrs.firstChildElement("ca");
     while(!DOMca.isNull())
     {
          // Sprawdzanie czy zgadza sie ilosc kolumn w kazdym wierszu z suma wielkosci kontekstow
          int ctx_sum = ColCount();
          if(index >= ctx_sum)
          {
               QMessageBox::information(NULL, "HQEd", "Too many columns are defined. Error at line " + QString::number(DOMca.lineNumber()) + ".", QMessageBox::Ok);
               return false;
          }
          
          QString path = DOMca.text();
          XTT_Attribute* attr = AttributeGroups->Attribute(path);
          if(attr == NULL)
          {
               QMessageBox::critical(NULL, "HQEd", "Undefined attribute: \"" + path + "\" at line " + QString::number(DOMca.lineNumber()) + ".", QMessageBox::Ok);
               return false;
          }
          
          // Jezeli odnajdziemy dany atrybut to mozemy go dodac
          fContextAtributes[index] = attr;
          
          // Oraz musimy go zapisac w kazdej komorce
          for(unsigned int r=0;r<fRowCount;r++)
               fRows[r]->Cell(index)->setAttrType(attr);
               
          // Powiekszenie indexu wczytywanej wartosci
          index++;
          
          // Wczytanie kolejengo atrybutu
          DOMca = DOMca.nextSiblingElement("ca");
     }

     // Wczytywanie rozmiarow kolumn
     index = 0;
     QDomElement DOMcw = DOMcolwidths.firstChildElement("cw");
     while(!DOMcw.isNull())
     {
          // Mozliwe jest podanie mniejszej ilosci szerokosci niz jest zadeklarowanych kolumn, wtedy szerokosc bedzie ustawiona
          // w pierszych kolumnach od prawej a pozostale szerokosci beda ustawione na 120. Podanie zbyt duzej ilosci wymiarow
          // spowoduje pominiecie tych ostatnich pozycji ktore sa nadmiarowe
          
          
          // Sprawdzanie czy zgadza sie ilosc kolumn w kazdym wierszu z suma wielkosci kontekstow
          int ctx_sum = ColCount();
          if(index >= ctx_sum)
          {
               QMessageBox::information(NULL, "HQEd", "Too many widths of columns are defined. Error at line " + QString::number(DOMcw.lineNumber()) + ".\nThis value wiil be ignored.", QMessageBox::Ok);
               continue;
          }
     
          bool ok_conv;
          QString l = DOMcw.text();
          unsigned int cw = (unsigned int)l.toInt(&ok_conv);
          
          if(!ok_conv)
          {
               QMessageBox::information(NULL, "HQEd", "Incorrect column " + QString::number(index+1) + " width. Value \"" + l + "\" is not integer type. Error at line " + QString::number(DOMcw.lineNumber()) + ".\nWidth will be set as default.", QMessageBox::Ok);
               cw = Settings->xttDefaultColumnWidth();
          }
          
          if(((int)cw) < 0)
          {
               QMessageBox::information(NULL, "HQEd", "Incorrect column " + QString::number(index+1) + " width. Value \"" + l + "\"  can't be less than zero. Error at line " + QString::number(DOMcw.lineNumber()) + ".\nWidth will be set as default.", QMessageBox::Ok);
               cw = Settings->xttDefaultColumnWidth();
          }
          
          fWidth[index] = cw;
          index++;
     
          // Wczytanie kolejengo rozmiaru dla kolumny
          DOMcw = DOMcw.nextSiblingElement("cw");
     }

     // Wczytywanie rozmiarow wierszy
     index = 0;
     QDomElement DOMrh = DOMrowheights.firstChildElement("rh");
     while(!DOMrh.isNull())
     {
          // Sprawdzanie czy liczba wierszy nie zostala przekroczona
          if((unsigned int)index >= fRowCount)
          {
               QMessageBox::information(NULL, "HQEd", "Too many heights of row are defined. Error at line " + QString::number(DOMrh.lineNumber()) + ".\nThis value wiil be ignored.", QMessageBox::Ok);
               continue;
          }
          
          bool ok_conv;
          QString l = DOMrh.text();
          unsigned int rh = (unsigned int)l.toInt(&ok_conv);
          
          if(!ok_conv)
          {
               QMessageBox::information(NULL, "HQEd", "Incorrect row " + QString::number(index+1) + " height. Value \"" + l + "\" is not integer type. Error at line " + QString::number(DOMrh.lineNumber()) + ".\nHeight will be set as 10.", QMessageBox::Ok);
               rh = Settings->xttDefaultRowHeight();
          }
          
          if(((int)rh) < 0)
          {
               QMessageBox::information(NULL, "HQEd", "Incorrect row " + QString::number(index+1) + " height. Value \"" + l + "\" can't be less than zero. Error at line " + QString::number(DOMrh.lineNumber()) + ".\nHeight will be set as 10.", QMessageBox::Ok);
               rh = Settings->xttDefaultRowHeight();
          }
          
          fHeight[index] = rh;
          index++;
     
          // Wczytanie kolejengo rozmiaru dla wiersza
          DOMrh = DOMrh.nextSiblingElement("rh");
     }

     // "<localization"
     QDomElement DOMlocalization = root.firstChildElement("localization");
     if(!DOMlocalization.isNull())
     {
          QString sXPos = DOMlocalization.attribute("xpos", "#NAN#");
          QString sYPos = DOMlocalization.attribute("ypos", "#NAN#");
          QString sHHeight = DOMlocalization.attribute("header_height", "#NAN#");
          QString sLevel = DOMlocalization.attribute("level", "#NAN#");
          QString sParent = DOMlocalization.attribute("parent", "#NAN#");
          
          // Podanie nie jest wymagane, jezeli parametr nie jest podany to przyjmujemy wielkosc za 0
          // Usuwanie spacji z wczytanych wartosci
          for(;sXPos.contains(" ");)
               sXPos = sXPos.remove(sXPos.indexOf(" "), 1);
          for(;sYPos.contains(" ");)
               sYPos = sYPos.remove(sYPos.indexOf(" "), 1);
          for(;sHHeight.contains(" ");)
               sHHeight = sHHeight.remove(sHHeight.indexOf(" "), 1);
          for(;sLevel.contains(" ");)
               sLevel = sLevel.remove(sLevel.indexOf(" "), 1);
          for(;sParent.contains(" ");)
               sParent = sParent.remove(sParent.indexOf(" "), 1);
               
          // Parametry nie sa wymagane, jezeli nie zostana podane to zostana im
          // przypisane nastpujace wartosci domyslne
          int iXPos = 0;
          int iYPos = 0;
          int iHHeight = 20;
          int iLevel = 0;
          int iParent = -1;
               
          // Proby konwersji jezeli podano
          if((sXPos != "") && (sXPos != "#NAN#"))
          {
               bool ok;
               iXPos = sXPos.toInt(&ok);
               if(!ok)
                    QMessageBox::information(NULL, "HQEd", "Incorrect value type. Value \"" + sXPos + "\" is not integer type. Error at line " + QString::number(DOMlocalization.lineNumber()) + ".", QMessageBox::Ok);
               fPoint.setX(iXPos);
          }
          if((sYPos != "") && (sYPos != "#NAN#"))
          {
               bool ok;
               iYPos = sYPos.toInt(&ok);
               if(!ok)
                    QMessageBox::information(NULL, "HQEd", "Incorrect value type. Value \"" + sYPos + "\" is not integer type. Error at line " + QString::number(DOMlocalization.lineNumber()) + ".", QMessageBox::Ok);
               fPoint.setY(iYPos);
          }
          if((sHHeight != "") && (sHHeight != "#NAN#"))
          {
               bool ok;
               iHHeight = sHHeight.toInt(&ok);
               if(!ok)
                    QMessageBox::information(NULL, "HQEd", "Incorrect value type. Value \"" + sHHeight + "\" is not integer type. Error at line " + QString::number(DOMlocalization.lineNumber()) + ".", QMessageBox::Ok);
               fHeaderHeight = (unsigned int)iHHeight;
          }
          if((sLevel != "") && (sLevel != "#NAN#"))
          {
               bool ok;
               iLevel = sLevel.toInt(&ok);
               if(!ok)
                    QMessageBox::information(NULL, "HQEd", "Incorrect value type. Value \"" + sLevel + "\" is not integer type. Error at line " + QString::number(DOMlocalization.lineNumber()) + ".", QMessageBox::Ok);
               fLevel = iLevel;
          }
          if((sParent != "") && (sParent != "#NAN#"))
          {
               bool ok;
               iParent = sParent.toInt(&ok);
               if(!ok)
                    QMessageBox::information(NULL, "HQEd", "Incorrect value type. Value \"" + sParent + "\" is not integer type. Error at line " + QString::number(DOMlocalization.lineNumber()) + ".", QMessageBox::Ok);
               fParent = iParent;
          }
     }

     // Po wczytaniu wszystkiego - weryfikacja wszystkich ustawien
     // Sprawdzanie czy zgadza sie ilosc kolumn w kazdym wierszu z suma wielkosci kontekstow
     int ctx_sum = ColCount();
     
     for(unsigned int r=0;r<fRowCount;r++)
     {
          if((unsigned int)ctx_sum != fRows[r]->Length())
          {
               QMessageBox::critical(NULL, "HQEd", "Incorrect col count in row " + QString::number(r+1) + ". Column count must be equal sum of all contexts size. Error at line: " + QString::number(root.lineNumber()) + ".", QMessageBox::Ok);
               return false;
          }
          
          // Jezeli wiersz jest odpowiednio dlugi to musimy przeanalizowac komorki
          for(unsigned int c=0;c<fRows[r]->Length();c++)
          {      
               // Zapisanie wartosci jezeli komorka jest pusta
               QString curr_content = fRows[r]->Cell(c)->strContent();
               if((!fRows[r]->Cell(c)->Analyze(curr_content)) && (Settings->xttShowErrorCellMessage()))
                    QMessageBox::warning(NULL, "HQEd", fRows[r]->Cell(c)->CellError(), QMessageBox::Ok);
                    
          }
     }

     // Odswiezenie widoku
     delete fGTable;
     fGTable = new GTable(this);
     fGTable->ReCreate();
     fGTable->Update();
     fGTable->RePosition();

     return res;
}
// -----------------------------------------------------------------------------

// #brief Function gets all data in hml 2.0 format.
//
// #param __mode - writing mode
// #li 0 - set mode
// #li 1 - domain mode
// #li 2 - expr mode
// #li 3 - eval mode
// #li 4 - fact2states mode
// #return List of all data in hml 2.0 format.
QStringList* XTT_Table::out_HML_2_0(int __mode)
{
     QStringList* res = new QStringList;
     res->clear();
     
     if(__mode != 4)
     {
          *res << "<table id=\"" + TableId() + "\" name=\"" + Title() + "\">";

          if(!fDescription.isEmpty())
               *res << "\t<desc>" + fDescription + "</desc>";
               
               // Saving table schema
               *res << "\t<schm>";
                    QList<XTT_Attribute*>* cca = ContextAtributes(1, true);
                    if(cca->size() > 0)
                    {
                         *res << "\t\t<precondition>";
                         for(int i=0;i<cca->size();++i)
                              *res << "\t\t\t<attref ref=\"" + cca->at(i)->Id() + "\"/>";
                         *res << "\t\t</precondition>";
                    }
                    
                    QList<XTT_Attribute*>* dca = ContextAtributes(4, true);
                    QList<XTT_Attribute*>* aca = ContextAtributes(5, true);
                    *dca += *aca;            // merging list
                    if(dca->size() > 0)
                    {
                         *res << "\t\t<conclusion>";
                         for(int i=0;i<dca->size();++i)
                              *res << "\t\t\t<attref ref=\"" + dca->at(i)->Id() + "\"/>";
                         *res << "\t\t</conclusion>";
                    }
                    
               *res << "\t</schm>";
               
               // Saving rules
               for(unsigned int i=0;i<RowCount();++i)
               {
                    QStringList* tmp = Row(i)->out_HML_2_0(__mode);
                    for(int j=0;j<tmp->size();++j)
                         *res << "\t" + tmp->at(j);
                    delete tmp;
               }
          
          *res << "</table>";
     }
     
     if(__mode == 4)
     {
          // Saving rules
          for(unsigned int i=0;i<RowCount();++i)
          {
               QStringList* tmp = Row(i)->out_HML_2_0(__mode);
               for(int j=0;j<tmp->size();++j)
                    *res << "\t" + tmp->at(j);
               delete tmp;
          }
     }
     
     return res;
}
// -----------------------------------------------------------------------------

#if QT_VERSION >= 0x040300
// #brief Function saves all data in hml 2.0 format.
//
// #param _xmldoc Pointer to document.
// #param __mode - writing mode
// #li 0 - set mode
// #li 1 - domain mode
// #li 2 - expr mode
// #li 3 - eval mode
// #li 4 - fact2states mode
// #return No retur value.
void XTT_Table::out_HML_2_0(QXmlStreamWriter* _xmldoc, int __mode)
{
     if(__mode != 4)
     {
          _xmldoc->writeStartElement("table");
          _xmldoc->writeAttribute("id", TableId());
          _xmldoc->writeAttribute("name", Title());

          if(!fDescription.isEmpty())
               _xmldoc->writeTextElement("desc", fDescription);
               
               // Saving table schema
               _xmldoc->writeStartElement("schm");
                    QList<XTT_Attribute*>* cca = ContextAtributes(1, true);
                    if(cca->size() > 0)
                    {
                         _xmldoc->writeStartElement("precondition");
                         for(int i=0;i<cca->size();++i)
                         {
                              _xmldoc->writeEmptyElement("attref");
                              _xmldoc->writeAttribute("ref", cca->at(i)->Id());
                         }
                         _xmldoc->writeEndElement();   // precondition
                    }
                    
                    QList<XTT_Attribute*>* dca = ContextAtributes(4, true);
                    QList<XTT_Attribute*>* aca = ContextAtributes(5, true);
                    *dca += *aca;            // merging list
                    if(dca->size() > 0)
                    {
                         _xmldoc->writeStartElement("conclusion");
                         for(int i=0;i<dca->size();++i)
                         {
                              _xmldoc->writeEmptyElement("attref");
                              _xmldoc->writeAttribute("ref", dca->at(i)->Id());
                         }
                         _xmldoc->writeEndElement();   // conclusion
                    }
                    
               _xmldoc->writeEndElement();   // schm
               
               // Saving rules
               for(unsigned int i=0;i<RowCount();++i)
                    Row(i)->out_HML_2_0(_xmldoc, __mode);
          
          _xmldoc->writeEndElement();   // table
     }
     
     if(__mode == 4)
     {
          // Saving rules
          for(unsigned int i=0;i<RowCount();++i)
               Row(i)->out_HML_2_0(_xmldoc, __mode);
     }
}
#endif
// -----------------------------------------------------------------------------

// #brief Function loads all data from hml 2.0 format.
//
// #param __root - reference to the parent element
// #param __msg - reference to the error messages
// #return the result code:
// #li 0 - on success
// #li 1 - on error
int XTT_Table::in_HML_2_0(QDomElement& __root, QString& __msg)
{
     int result = 0;
     QString elementName = "table";
     
     QDomElement DOMtable = __root;
     if(DOMtable.tagName().toLower() != elementName.toLower())
          return 0;
          
     // Reading table data
     QString tid = DOMtable.attribute("id", "");
     QString tname = DOMtable.attribute("name", "");
     QDomElement DOMdesc = DOMtable.firstChildElement("desc");

     if(tid == "")
     {
          __msg += "\n" + hqed::createErrorString("", "", "Undefined table \'" + tname + "\' identifier. The object will not be read." + hqed::createDOMlocalization(DOMtable), 0, 2);
          result++;
          return result;
     }
     if(tname == "")
     {
          __msg += "\n" + hqed::createErrorString("", "", "Undefined table name.\nTable id = \'" + tid + "\'." + hqed::createDOMlocalization(DOMtable), 1, 2);
          result++;
     }

     fTitle = tname;
     fTableID = tid;
     if(!DOMdesc.isNull())
          fDescription = DOMdesc.text();

     // Reading table schema
     QDomElement DOMschm = DOMtable.firstChildElement("schm");
     if(!DOMschm.isNull())
     {
          // Reading precondition part
          QDomElement DOMcond = DOMschm.firstChildElement("precondition");
          if(!DOMcond.isNull())
          {
               QDomElement DOMattref = DOMcond.firstChildElement("attref");
               while(!DOMattref.isNull())
               {
                    QString aref = DOMattref.attribute("ref", "");
                    if(aref == "")
                    {
                         __msg += "\n" + hqed::createErrorString("", "", "Undefined attribute reference \'" + aref + "\'.\nError from \'" + DOMcond.tagName() + "\' part in table \'" + tname + "\'.\nThe reference will be omitted." + hqed::createDOMlocalization(DOMattref), 1, 2);
                         result++;
                         DOMattref = DOMattref.nextSiblingElement("attref");
                         continue;
                    }
                    
                    XTT_Attribute* attr = AttributeGroups->FindAttribute(aref);
                    if(attr == NULL)
                    {
                         __msg += "\n" + hqed::createErrorString("", "", "Attribute reference error. Undefined attribute identifier \'" + aref + "\'.\nError from \'" + DOMcond.tagName() + "\' part in table \'" + tname + "\'.\nThe reference will be omitted." + hqed::createDOMlocalization(DOMattref), 1, 2);
                         result++;
                         DOMattref = DOMattref.nextSiblingElement("attref");
                         continue;
                    }
                    
                    // Adding new column
                    if(ContextSize(1) > 0)
                         InsertCol(ColCount());
                    if(ContextSize(1) == 0)
                         InsertNewContext(1);
                    setContextAtributes(ColCount()-1, attr);      // Setting new attribute
                         
                    DOMattref = DOMattref.nextSiblingElement("attref");
               }
          }

          // Reading decision part
          QDomElement DOMactn = DOMschm.firstChildElement("conclusion");
          if(!DOMactn.isNull())
          {
               QDomElement DOMattref = DOMactn.firstChildElement("attref");
               while(!DOMattref.isNull())
               {
                    QString aref = DOMattref.attribute("ref", "");
                    if(aref == "")
                    {
                         __msg += "\n" + hqed::createErrorString("", "", "Undefined attribute reference \'" + aref + "\'.\nError from \'" + DOMactn.tagName() + "\' part in table \'" + tname + "\'.\nThe reference will be omitted." + hqed::createDOMlocalization(DOMattref), 1, 2);
                         result++;
                         DOMattref = DOMattref.nextSiblingElement("attref");
                         continue;
                    }

                    XTT_Attribute* attr = AttributeGroups->FindAttribute(aref);
                    if(attr == NULL)
                    {
                         __msg += "\n" + hqed::createErrorString("", "", "Attribute reference error. Undefined attribute identifier \'" + aref + "\'.\nError from \'" + DOMactn.tagName() + "\' part in table \'" + tname + "\'.\nThe reference will be omitted." + hqed::createDOMlocalization(DOMattref), 1, 2);
                         result++;
                         DOMattref = DOMattref.nextSiblingElement("attref");
                         continue;
                    }

                    // Adding new column
                    if(ContextSize(4) > 0)
                         InsertCol(ColCount());
                    if(ContextSize(4) == 0)
                         InsertNewContext(4);
                    setContextAtributes(ColCount()-1, attr);      // Setting new attribute

                    DOMattref = DOMattref.nextSiblingElement("attref");
               }
          }
     }

     // Reading rules
     QDomElement DOMrule = DOMtable.firstChildElement("rule");
     while(!DOMrule.isNull())
     {
          // Checking rule data
          QString rid = DOMrule.attribute("id", "");

          if(rid == "")
          {
               __msg += "\n" + hqed::createErrorString("", "", "Undefined rule identifier in table \'" + fTitle + "\'.\nRule index = " + QString::number(RowCount()+1) + ". The object will not be read." + hqed::createDOMlocalization(DOMrule), 0, 2);
               result++;
               DOMrule = DOMrule.nextSiblingElement("rule");
               continue;
          }

          AddRow();
          fRows[fRowCount-1]->in_HML_2_0(DOMrule, __msg);

          DOMrule = DOMrule.nextSiblingElement("rule");
     }

     // Odswiezenie widoku
     delete fGTable;
     fGTable = new GTable(this);
     fGTable->ReCreate();
     fGTable->Update();
     fGTable->RePosition();

     return result;
}
// -----------------------------------------------------------------------------
