/*
 *	   $Id: hDomain.cpp,v 1.10 2009-05-13 10:28:19 kinio Exp $
 *	
 *     Implementation by Krzysztof Kaczor <kinio4444@gmail.com>
 *
 *     Copyright (C) 2006-9 by the HeKatE Project
 *
 *     HQEd has been develped within the HeKatE Project,
 *     see http://hekate.ia.agh.edu.pl
 *
 *     This file is part of HQEd.
 *
 *     HQEd is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     HQEd is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with HQEd.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <QMessageBox>
#include <QtGui>
#include <QDialog>
#include <QObject>
#include <QStringList>

#include "../namespaces/ns_hqed.h"

#include "hType.h"
#include "hValue.h"
#include "hSetItem.h"
#include "hValueResult.h"
#include "hDomain.h"
// -----------------------------------------------------------------------------

// #brief Constructor hSet class.
hDomain::hDomain(void)
{
     _parentType = NULL;
     _orderd = false;
}
// -----------------------------------------------------------------------------

// #brief Constructor hDomain class.
//
// #param hType* __parentType - poiter to the type of data that the set can contain
hDomain::hDomain(hType* __parentType)
{
     _orderd = false;
     setParentType(__parentType);
}
// -----------------------------------------------------------------------------

// #brief Function that creates copy of this object
//
// #param hDomain* __destinationItem - the destination copy localization. If NULL function creates new instace of object
// #return The same pointer as given or if __destinationItem = NULL the poiter to the new object that is a copy of current
hDomain* hDomain::copyTo(hDomain* __destinationItem)
{
     hDomain* res = __destinationItem;
     if(res == NULL)
          res = new hDomain;
          
     res->setOrdered(_orderd);
     res->_numbering = _numbering;
     hSet::copyTo(res);
          
     return res;
}
// -----------------------------------------------------------------------------

// #brief Function that return the number on the given position
//
// #param __index - the index of value
// #return number on the given position
int hDomain::numberEquivalent(int __index)
{
     if((__index < 0) || (__index >= _numbering.size()) || (!ordered()))
          return __index+1;
          
     return _numbering.at(__index);
}
// -----------------------------------------------------------------------------

// #brief Function that return the index of the item that is equivalent to numeric __value
//
// #param __value - the numeric equivalent of item
// #return index of the item that is equivalent to numeric __value
int hDomain::symbolicEquivalent(int __value)
{
     int index = -1;
     if(ordered())
          index = _numbering.indexOf(__value);
     if((!ordered()) || (_numbering.size() == 0))
          index = __value-1;
          
     return index;
}
// -----------------------------------------------------------------------------

// #brief The function that updates the setItems w.r.t. numbering, it renumber all items again only if the type is apropriate.
//
// #return on succes, otherwise false
bool hDomain::updateItemsNumbering(void)
{
     // Checking correctness of the type and domain
     if(_parentType == NULL)
     {
          setLastError("DOMAIN ERROR: Not defined parent set.");
          return false;
     }
          
     if(_parentType->typeClass() != TYPE_CLASS_DOMAIN)
     {
          setLastError("DOMAIN ERROR: Set constraints are not defined as domain.");
          return false;
     }
     
     if((_parentType->baseType() != SYMBOLIC_BASED) || (!ordered()))
     {
          setLastError("DOMAIN ERROR: Numeric representation is availible only for symbolic and ordered domains.");
          return false;
     }
     
     // Renumbering items
     for(int i=0;i<size();++i)
     {
          int dt = at(i)->valuePtr()->definitionType();
          
          at(i)->valuePtr()->setDefinitionType(VALUE_DEFINITION_TYPE_NUM);
          at(i)->valuePtr()->setValue(QString::number(numberEquivalent(i)));
          
          at(i)->valuePtr()->setDefinitionType(dt);
     }
     
     return true;
}
// -----------------------------------------------------------------------------

// #brief The function that updates the numbering using setItems, it creates new number list only if the type is apropriate.
//
// #return on succes, otherwise false
bool hDomain::updateNumbering(void)
{
     // Checking correctness of the type and domain
     if(_parentType == NULL)
     {
          setLastError("DOMAIN ERROR: Not defined parent set.");
          return false;
     }
          
     if(_parentType->typeClass() != TYPE_CLASS_DOMAIN)
     {
          setLastError("DOMAIN ERROR: Set constraints are not defined as domain.");
          return false;
     }
     
     if((_parentType->baseType() != SYMBOLIC_BASED) || (!ordered()))
     {
          setLastError("DOMAIN ERROR: Numeric representation is availible only for symbolic and ordered domains.");
          return false;
     }
     
     // Creating new numbering list
     _numbering.clear();
     for(int i=0;i<size();++i)
     {
          int dt = at(i)->valuePtr()->definitionType();
          at(i)->valuePtr()->setDefinitionType(VALUE_DEFINITION_TYPE_NUM);
          
          hValueResult hvr = at(i)->valuePtr()->value();
          if(!(bool)hvr)
          {
               setLastError("DOMAIN ERROR: " + (QString)hvr);
               return false;
          }
          _numbering.append(((QString)hvr).toInt());
          
          at(i)->valuePtr()->setDefinitionType(dt);
     }
     
     return true;
}
// -----------------------------------------------------------------------------

// #brief Function that sets the parent type of the set
//
// #param hType* __pt - poiter to the type
// #return true on succes, otherwise false
bool hDomain::setParentType(hType* __pt)
{
     if(__pt != NULL)
     {
          if((__pt->baseType() == INTEGER_BASED) || (__pt->baseType() == NUMERIC_BASED))
               _orderd = true;
     }

     return hSet::setParentType(__pt);
}
// -----------------------------------------------------------------------------

// #brief The virtual function that return the index of the first item that contains given value
//
// #param QString __value - the value as string
// #return index of the first item that contains given value
int hDomain::contain(QString __value)
{
     if(_parentType == NULL)
          return setLastError(H_SET_ERROR_NOT_DEFINED_TYPE);
          
     if(_parentType->typeClass() == TYPE_CLASS_DOMAIN)
     {
          if(_parentType->baseType() == SYMBOLIC_BASED)
          {
               for(int i=0;i<size();++i)
               {
                    if(at(i)->type() == SET_ITEM_TYPE_RANGE)
                         return setLastError(H_SET_ERROR_INCORRECT_DOMAIN_ITEM, "\nError item: " + at(i)->toString());
                         
                    hValueResult hvr = at(i)->value();
                    if(!(bool)hvr)
                         return setLastError(H_SET_OTHER_OBJECT_ERROR, hvr.error());
                         
                    if((QString)hvr == __value)
                         return i;
               }
          }
          else
               return hSet::contain(__value);
     }
     
     return -1;
}
// -----------------------------------------------------------------------------

// #brief The functions that checks if the item is in domain
//
// #param hSetItem* __item - pointer to the item
// #return H_SET_TRUE_VALUE if item is in set otherwise H_SET_FALSE_VALUE. On error returns error code.
int hDomain::contain(hSetItem* __item)
{
     return hSet::contain(__item);
}
// -----------------------------------------------------------------------------

// #brief The functions that checks if the set __item is a sub set of this
//
// #param hSet* __item - pointer to the set
// #return H_SET_TRUE_VALUE if item is a sub set of this set otherwise H_SET_FALSE_VALUE. On error returns error code.
int hDomain::contain(hSet* __item)
{
     return hSet::contain(__item);
}
// -----------------------------------------------------------------------------

// #brief Function that returns the string list representation of the items
//
// #return The string list representation of the items
QStringList hDomain::stringItems(void)
{
     QStringList res;
     for(int i=0;i<size();++i)
          res << toString(i);
          
     return res;
}
// -----------------------------------------------------------------------------

// #brief Function that returns the string representation of the set item
//
// #param __index - index of the item
// #return The string representation of the set item
QString hDomain::toString(int __index)
{
     if((__index < 0) || (__index >= size()))
          return "";
     
     if(_parentType == NULL)
     {
          setLastError(H_SET_ERROR_NOT_DEFINED_TYPE);
          return lastError();
     }
     
     if((_parentType->baseType() == SYMBOLIC_BASED) && (ordered()))
          return at(__index)->toString() + "/" + QString::number(numberEquivalent(__index));
          
     return at(__index)->toString();
}
// -----------------------------------------------------------------------------

// #brief Function that returns the string representation of the set
//
// #return The string representation of the set
QString hDomain::toString(void)
{
     QString res = "";
     
     if(_parentType == NULL)
     {
          setLastError(H_SET_ERROR_NOT_DEFINED_TYPE);
          return lastError();
     }
     
     if((_parentType->baseType() == SYMBOLIC_BASED) && (ordered()))
     {
          if(size() > 0)
               res = toString(0);
          
          for(int i=1;i<size();++i)
               res = res + " U " + toString(i);
               
          if(isEmpty())
               res = "{}";
     }
     else
          res = hSet::toString();
          
     return res;
}
// -----------------------------------------------------------------------------

// #brief The function that return the numeric equivalent of the __value. This function can be used only with ordered symbolic domains.
//
// #param __value - the value as string
// #param __exists - the boolean value where the status of exit is stored
// #return numeric equivalent of the __value.
int hDomain::toNumericEquivalent(QString __value, bool& __exists)
{
     __exists = false;
    
     int index = contain(__value);
     if(index == -1)
          return -1;
    
     __exists = true;
    
     return numberEquivalent(index);
}
// -----------------------------------------------------------------------------

// #brief The function that return the String equivalent of the __value. This function can be used only with ordered symbolic domains.
//
// #param __value - the value as integer
// #return string value
hValueResult hDomain::fromNumericEquivalent(int __value)
{
     int index = symbolicEquivalent(__value);
     
     if((index < 0) || (index >= size()))
          return hValueResult("The numeric value \'" + QString::number(__value) + "\' is out of range.", false);
          
     return at(index)->value();
}
// -----------------------------------------------------------------------------

// #brief The function that return the next value (not index) w.r.t. __ref_value. The function can be used only with SYMBOLIC BASED domains
//
// #param __ref_value - the reference value as integer
// #param __exists - reference to the boolean value, that holds the result of function. If true then returned value is correct, otherwise is incorrect
// #return the next value w.r.t. __ref_value
int hDomain::nextNumericValue(int __ref_value, bool& __exists)
{
     __exists = false;
     
     if((_numbering.size() == 0) && (__ref_value >= 1) && (__ref_value < size()))
     {
          return __ref_value+1;
     }
          
     int result = __ref_value;
     for(int i=0;i<_numbering.size();++i)
          if((((result > _numbering.at(i)) && (_numbering.at(i) > __ref_value))) || 
             (_numbering.at(i) > __ref_value) && (result == __ref_value))
             {
               result = _numbering.at(i);
               __exists = true;
             }
               
     return result;
}
// -----------------------------------------------------------------------------

// #brief The function that return the previous value (not index) w.r.t. __ref_value. The function can be used only with SYMBOLIC BASED domains
//
// #param __ref_value - the reference value as integer
// #param __exists - reference to the boolean value, that holds the result of function. If true then returned value is correct, otherwise is incorrect
// #return the previous value w.r.t. __ref_value
int hDomain::prevNumericValue(int __ref_value, bool& __exists)
{
     __exists = false;
     
     if((_numbering.size() == 0) && (__ref_value > 1) && (__ref_value <= size()))
     {
          return __ref_value-1;
     }
          
     int result = __ref_value;
     for(int i=0;i<_numbering.size();++i)
          if((((result < _numbering.at(i)) && (_numbering.at(i) < __ref_value))) || 
             (_numbering.at(i) < __ref_value) && (result == __ref_value))
             {
               result = _numbering.at(i);
               __exists = true;
             }
               
     return result;
}
// -----------------------------------------------------------------------------

// #brief Function that maps the domain to the prolog code.
//
// #param __option - options of the output
// #return The prolog code that represents this domain as string.
QString hDomain::toProlog(int __option)
{
     QString result = "";
     result += "[" + hSet::toProlog() + "]";
     
     if((parentType()->baseType() == SYMBOLIC_BASED) && (ordered()))
          result += ",\n       ordered: yes";
          
     return result;
}
// -----------------------------------------------------------------------------

// #brief Function gets all data in hml 2.0 format.
//
// #return List of all data in hml 2.0 format.
QStringList* hDomain::out_HML_2_0(void)
{
     QStringList* res = new QStringList;
     res->clear();

     QString ord = "no";
     if(_orderd)
          ord = "yes";
          
     QString init_line = "<domain";
     if(parentType()->baseType() == SYMBOLIC_BASED)
          init_line += " ordered=\"" + ord + "\"";
     init_line += ">";

     *res << init_line;
     
     // Zapis elementow
     for(int a=0;a<size();++a)
     {
          QStringList* al = at(a)->out_HML_2_0(1);
          for(int l=0;l<al->size();l++)
               *res << "\t" + al->at(l);
          delete al;
     }
     
     *res << "</domain>";

     return res;
}
// -----------------------------------------------------------------------------

#if QT_VERSION >= 0x040300
// #brief Function saves all data in hml 2.0 format.
//
// #param _xmldoc Pointer to document.
// #return No retur value.
void hDomain::out_HML_2_0(QXmlStreamWriter* _xmldoc)
{
     QString ord = "no";
     if(_orderd)
          ord = "yes";
          
     _xmldoc->writeStartElement("domain");
     if(parentType()->baseType() == SYMBOLIC_BASED)
          _xmldoc->writeAttribute("ordered", ord);

     // Zapis elementow
     for(int a=0;a<size();++a)
          at(a)->out_HML_2_0(_xmldoc, 1);
     
     _xmldoc->writeEndElement();
}
#endif
// -----------------------------------------------------------------------------

// #brief Function loads all data from hml 2.0 format.
//
// #param __root - reference to the parent element
// #param __msg - reference to the error messages
// #return the result code:
// #li 0 - on success
// #li 1 - on error
int hDomain::in_HML_2_0(QDomElement& __root, QString& __msg)
{
     int result = 0;
     QString elementName = "domain";
     QString childElementName = "value";
     QString att1 = "ordered", att1def = "yes";
     
     if(_parentType == NULL)
     {
          __msg += "\n" + hqed::createErrorString("", "", "Critical error: udefined parent type of domain." + hqed::createDOMlocalization(__root), 0, 2);
          return 1;
     }
     
     if(__root.tagName().toLower() != elementName)
          return 0;
          
     // Reading type data
     QString dordered = __root.attribute(att1, att1def);
     
     // Checking for errors
     if((dordered.toLower() != "no") && (dordered.toLower() != "yes"))
     {
          __msg += "\n" + hqed::createErrorString("", "", "Incorrect value of the domain's \'" + att1 + "\' parameter in \'" + _parentType->name() + "\' type.\nRead value: " + dordered + ".\nThe value will be set to \'" + att1def + "\'." + hqed::createDOMlocalization(__root), 1, 2);
          dordered = att1;
          result++;
     }
     _orderd = (dordered.toLower() == "yes");
     
     // Reading values
     _numbering.clear();
     bool useNumbering = ((_parentType->baseType() == SYMBOLIC_BASED) && (ordered()));
     QDomElement DOMvalue = __root.firstChildElement(childElementName);
     while(!DOMvalue.isNull())
     {
          // Checking for numbering
          bool hasNum = DOMvalue.hasAttribute("num");
          useNumbering = useNumbering && hasNum;
          if((hasNum) && (useNumbering))
          {
               bool convStat;
               QString num = DOMvalue.attribute("num", "");
               int inum = num.toInt(&convStat);
               if(!convStat)
               {
                    __msg += "\n" + hqed::createErrorString("", "", "Incorrect value of the \'num\' parameter in domain of \'" + _parentType->name() + "\' type.\nRead value: " + num + ".\nNumbering will be set as default (1,2,3,...)" + hqed::createDOMlocalization(DOMvalue), 1, 2);
                    useNumbering = false;
                    result++;
               }
               if((convStat) && (_numbering.indexOf(inum) > -1))
               {
                    __msg += "\n" + hqed::createErrorString("", "", "Numeric equivalent \'" + num + "\' already exists.\nError from domain of type \'" + _parentType->name() + "\'.\nNumbering will be set as default (1,2,3,...)" + hqed::createDOMlocalization(DOMvalue), 1, 2);
                    useNumbering = false;
                    result++;
               }
               _numbering.append(inum);
          }
          
          if(((DOMvalue.hasAttribute("is")) && (!DOMvalue.hasAttribute("from")) && (!DOMvalue.hasAttribute("to"))) ||
             ((!DOMvalue.hasAttribute("is")) && (DOMvalue.hasAttribute("from")) && (DOMvalue.hasAttribute("to"))))
          {
               hSetItem* si = new hSetItem;
               si->setParent(this);
               add(si);
               result += si->in_HML_2_0(DOMvalue, __msg, NULL);
          }
          else
          {
               QString errorstr = "Incorrect set of attributes.";
               errorstr += "\nThe attribute \'is\' cannot be used with \'from\' and \'to\' simultaneously.";
               errorstr += "\nThe attribute \'from\' must be used together with \'to\'.";
               errorstr += hqed::createDOMlocalization(DOMvalue);
               __msg += "\n" + hqed::createErrorString("", "", errorstr, 0, 2);
               result++;
          }
          
          DOMvalue = DOMvalue.nextSiblingElement(childElementName);
     }
     
     // When numbering is not defined correctly or is unnecessary
     if(!useNumbering)
     {
          _numbering.clear();
          for(int i=0;i<size();++i)
               _numbering.append(i+1);       // numbering starts with 1
     }
     
     return result;
}
// -----------------------------------------------------------------------------
