/*
 *	   $Id: hopretract.cpp,v 1.4 2009-04-07 08:41:19 kinio Exp $
 *	
 *     Implementation by Krzysztof Kaczor <kinio4444@gmail.com>
 *
 *     Copyright (C) 2006-9 by the HeKatE Project
 *
 *     HQEd has been develped within the HeKatE Project,
 *     see http://hekate.ia.agh.edu.pl
 *
 *     This file is part of HQEd.
 *
 *     HQEd is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     HQEd is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with HQEd.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <QMessageBox>
#include <QtGui>
#include <QDialog>
#include <QObject>
#include <QStringList>

#include "../../XTT/XTT_Attribute.h"
#include "../../hSet.h"
#include "../../hType.h"
#include "../../hValue.h"
#include "../../hDomain.h"
#include "../../hSetItem.h"
#include "../../hFunction.h"
#include "../../hTypeList.h"
#include "../../hFunctionList.h"
#include "../../hMultipleValue.h"
#include "../../hFunctionArgument.h"

#include "hopretract.h"
// -----------------------------------------------------------------------------

// #brief Constructor hadd class.
hopretract::hopretract(void)
{
     setupFunction();
}
// -----------------------------------------------------------------------------

// #brief Function that sets up the internal parameters of the function
//
// #return no value returns
void hopretract::setupFunction(void)
{
     _userRepresentation = "retract";
     _prologRepresentation = "retract";
     _internalRepresentation = "void_retract_operator";
     
     _resultType = FUNCTION_TYPE__SINGLE_VALUED | FUNCTION_TYPE__MULTI_VALUED;              
     _returnedTypes = FUNCTION_TYPE__VOID;                              
     _minArgCount = 1;                  
     _maxArgCount = 2;   
     _additionalArgsType = FUNCTION_TYPE__BOOLEAN | FUNCTION_TYPE__INTEGER | FUNCTION_TYPE__NUMERIC | FUNCTION_TYPE__UOSYMBL | FUNCTION_TYPE__OSYMBL | FUNCTION_TYPE__MULTI_VALUED;          
     setArgCount(1);  
     _minArgCount = 2;  
     _additionalArgsType = FUNCTION_TYPE__BOOLEAN | FUNCTION_TYPE__INTEGER | FUNCTION_TYPE__NUMERIC | FUNCTION_TYPE__UOSYMBL | FUNCTION_TYPE__OSYMBL | _resultType;          
     setArgCount(2);      

     // Defining special names for arguments
     _argNames.append("attribute");
     _argNames.append("expression");
     _argNames.append("expression");
     refreshArgumentsNames();
     

     // Defining function description
     _description = "<b>Function name:</b> " + _userRepresentation + "<br>";
     _description += "<b>Function description:</b> Function that retracts the values of the <b>expression</b> set from the <b>attribute</b>.<br>This function can used only with action context.<br>";
     _description += "<b>Returned type:</b> void<br><br>";
     
     _description += "<b>Minimum argument count:</b> " + QString::number(_minArgCount) + "<br>";
     if(_maxArgCount >= 0)
          _description += "<b>Maximum argument count:</b> " + QString::number(_maxArgCount) + "<br>";
     if(_maxArgCount < 0)
          _description += "<b>Maximum argument count:</b> unimportant<br>";
     _description += "<b>Current argument count:</b> " + QString::number(_minArgCount) + "<br>";
     _description += "<b>Argument description:</b><br>";
     _description += "<b>attribute</b> - the attribute assigned to the cell's column.<br>";
     _description += "<b>expression</b> - represents the expression, which calculated values are substracted from the <b>attribute</b>.";
     
     hFunction::registerFunction(funcList);
}
// -----------------------------------------------------------------------------

// #brief Function that creates the new instance of the function object
//
// #param hFunction* - poiter to the destination object
// #return pointer to the new instance of function object
hFunction* hopretract::copyTo(hFunction*)
{
     hopretract* dest = new hopretract;
     hFunction::copyTo(dest);
          
     return dest;
}
// -----------------------------------------------------------------------------

// #brief Function that sets the argument for the function
//
// #param int __arg_index - the index of argument
// #param hFunctionArgument* __arg - the pointer to the argument
// #return success code if all is ok otherwise error code
int hopretract::setArgument(int __arg_index, hFunctionArgument* __arg)
{
     int fsa_res = hFunction::setArgument(__arg_index, __arg);
     if(fsa_res != FUNCTION_ERROR_SUCCESS)
          return fsa_res;
     
     // When the first argument (here attribute) is going to be change by the user
     if(__arg_index == 0)
     {
          if(argCount() < 2)
               return setLastError(FUNCTION_ERROR_ARG_INDEX_OUT_OF_RANGE);
               
          _args->at(0)->setName("attribute");
          _args->at(1)->setType(_args->at(0)->type() | FUNCTION_TYPE__SINGLE_VALUED | FUNCTION_TYPE__MULTI_VALUED);
          _args->at(1)->findCompatibleTypes(typeList);
          _args->at(1)->setValueParentType(_args->at(0)->value()->parentType(), true);
     }

     return FUNCTION_ERROR_SUCCESS;
}
// -----------------------------------------------------------------------------

// #brief Function that sets the value of the required type.
//
// #param hType* __required_type - the value of required type
// #return the result of compatybility this function with required type
int hopretract::setRequiredType(hType* __required_type)
{
     int hfsrtres = hFunction::setRequiredType(__required_type);
     if(hfsrtres != FUNCTION_ERROR_SUCCESS)
          return hfsrtres;
          
     //_additionalArgsType = hFunction::typeFunctionRepresentation(__required_type);
     //_additionalArgsType |= FUNCTION_TYPE__SINGLE_VALUED | FUNCTION_TYPE__MULTI_VALUED;

     //for(int i=0;i<argCount();++i)
     //     _args->at(i)->setType(_additionalArgsType);
     
     refreshArgumentCompatibleTypesLists();
     return FUNCTION_ERROR_SUCCESS;
}
// -----------------------------------------------------------------------------

// #brief Function that returns if the function object definition is correct
//
// #param __error_message - the reference to the error message(s)
// #return true if yes otherwise false
bool hopretract::functionVerification(QString& __error_message)
{
     if(!hFunction::functionVerification(__error_message))
          return false;
          
     // Checking the correctness of the first attribute
     return (_args->at(0)->value()->type() == MULTIPLE_VALUE_TYPE_ATTRIBUTE);
}
// -----------------------------------------------------------------------------

// #brief Function that returns the result of the fnction caculation. This is pure virtual function.
//
// #return the hSet object as the result of the caculation.
hMultipleValue* hopretract::calculate(void)
{
     /*
     // Checking arguments
     if(arg_value.toLower() == XTT_Attribute::_not_definded_operator_.toLower())
          return new hMultipleValue(MULTIPLE_VALUE_TYPE_NOT_DEFINED);
     if(arg_value.toLower() == XTT_Attribute::_any_operator_.toLower())
          return new hMultipleValue(MULTIPLE_VALUE_TYPE_ANY_VALUE);*/
          
     // Checking if the first argument has got an attribute type
     if(_args->at(0)->value()->type() != MULTIPLE_VALUE_TYPE_ATTRIBUTE)
     {
          setLastError(QString(FUNCTION_ERROR_STRING) + _userRepresentation + " error: the first argument is not defined as attribute.");
          return NULL;
     }
     
     /* This is checked in the tableVerification function */
     /*
     // Checking if the attribute is not read-only
     if(_args->at(0)->value()->attributeField()->Class() == XTT_ATT_CLASS_RO)
     {
          setLastError(QString(FUNCTION_ERROR_STRING) + _userRepresentation + " error: the attribute \'" + _args->at(0)->value()->attributeField()->Name() + "\' is read only.");
          return NULL;
     }*/
     
     // The values of the first argument
     hMultipleValueResult mvr1 = _args->at(0)->value()->value();
     if(!(bool)mvr1)
     {
          setLastError(QString(FUNCTION_ERROR_STRING) + (QString)mvr1);
          return NULL;
     }
 
     // The values of the second argument
     hMultipleValueResult mvr2 = _args->at(1)->value()->value();
     if(!(bool)mvr2)
     {
          setLastError(QString(FUNCTION_ERROR_STRING) + (QString)mvr2);
          return NULL;
     }
     
     // Checking if the set is correct
     if(_args->at(0)->value()->attributeField()->Type()->isSetCorrect(mvr2.toSet(false), _verifyResult) != H_TYPE_ERROR_NO_ERROR)
     {
          hSet* vals = mvr2.toSet(false)->toVals();
          setLastError(QString(FUNCTION_ERROR_STRING) + "The result set: " + mvr2.toSet(false)->toString() + " = " + vals->toString() + " is incorrect.\nError message: " + _args->at(0)->value()->attributeField()->Type()->lastError());
          delete vals;
          return NULL;
     }
     
     hSet* s1 = mvr1.toSet(false)->toVals();
     hSet* s2 = mvr2.toSet(false)->toVals();
     hSet* rs = s1->sub(s2);
     
     hMultipleValue* mv = new hMultipleValue(rs->toVals());
     if(!_args->at(0)->value()->attributeField()->checkValue(mv))
     {
          setLastError(QString(FUNCTION_ERROR_STRING) + _userRepresentation + " error: incorrect attribute \'" + _args->at(0)->value()->attributeField()->Name() + "\' value: " + mv->toString() + ".");
          delete s1;
          delete s2;
          delete mv;
          return NULL;
     }
     _args->at(0)->value()->attributeField()->setValue(mv);
     
     delete s1;
     delete s2;
     // delete rs; -- jest kasowany w momencie kiedy sie kasuje zmienna mv
     delete mv;

     return new hMultipleValue(hValue::toSet(_requiredType, new hValue));
}
// -----------------------------------------------------------------------------

// #brief Function that returns the string that is the user representation of the functions inculing arguments
//
// #return the string that is the user representation of the functions inculing arguments
QString hopretract::toString(void)
{
     if(_args->size() != 2)
          return _internalRepresentation + " erorr: incorrect number of arguments.";
     
     QString res = "";
     
     res += _userRepresentation + " ";
     
     if(_args->at(0)->isVisible())
          res += _args->at(0)->value()->toString();
          
     if((_args->at(0)->isVisible()) && (_args->at(1)->isVisible()))
          res += ",";
     
     if(_args->at(1)->isVisible())
          res += _args->at(1)->value()->toString();
     
     return res;
}
// -----------------------------------------------------------------------------

// #brief Function that maps the function to the prolog code.
//
// #return The prolog code that represents this function as string.
QString hopretract::toProlog(void)
{
     QString res = "";
     
     res += _userRepresentation + "(";
     res += _args->at(0)->value()->toString() + "(";
     res += _args->at(1)->value()->toString() + "))";
     
     return res;
}
// -----------------------------------------------------------------------------
