 /**
 *                                                                             *
 * @file gARDlevel.h                                                           *
 * Project name: HeKatE hqed                                                   *
 * $Id: gARDlevel.h,v 1.20 2009-05-13 10:28:21 kinio Exp $                      *
 * Author: Krzysztof Kaczor                                                    *
 * Date: 02.01.2008                                                            *
 *                                                                             *
 * Implementation by Krzysztof Kaczor <kinio4444@gmail.com>                    *
 *                                                                             *
 * Copyright (C) 2006-9 by the HeKatE Project                                  *
 *                                                                             *
 * HQEd has been develped within the HeKatE Project,                           *
 * see http://hekate.ia.agh.edu.pl                                             *
 *                                                                             *
 * This file is part of HQEd.                                                  *
 *                                                                             *
 * HQEd is free software: you can redistribute it and/or modify                *
 * it under the terms of the GNU General Public License as published by        *
 * the Free Software Foundation, either version 3 of the License, or           *
 * (at your option) any later version.                                         *
 *                                                                             *
 * HQEd is distributed in the hope that it will be useful,                     *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU General Public License for more details.                                *
 *                                                                             *
 * You should have received a copy of the GNU General Public License           *
 * along with HQEd.  If not, see <http://www.gnu.org/licenses/>.               *
 *                                                                             *
 * Comment: Plik zawierajacy definicje obiektu bedacego zarzadzajacym          *
 * postacia pojedynczego poziomu ARD                                           *
 *                                                                             *
 */


#ifndef GARDLEVELH
#define GARDLEVELH
//---------------------------------------------------------------------------

#define MARGIN_LENGTH 800			// Dlugosc linii marginesu
#define MARGINS_COUNT 2
#define TEXTS_COUNT 1
//---------------------------------------------------------------------------

class QGraphicsLineItem;
class QGraphicsTextItem;

class gARDexpander;
class ARD_Dependency;
class ARD_PropertyList;
class ARD_Property;
class ARD_Level;
class ardGraphicsScene;
class gARDnode;
class gARDdependency;
//---------------------------------------------------------------------------

class gARDlevel
{
private:

	/// Nastepny identyfikator wezla
	unsigned int fNextNodeID;
	
	/// Nastepny identyfikator zaleznosci
	unsigned int fNextDependencyID;
	
     /// Rodzic
     ARD_Level* fParent;
	
	/// Lista wezlow
	QList<gARDnode*>* fNodes;
	
	/// Lista wezlow
	QList<gARDdependency*>* fDependencies;
	
	/// Linie wyznaczajace obszar poziomu
	QGraphicsLineItem* fMargins[MARGINS_COUNT];
	
	/// Tytul poziomu
	QGraphicsTextItem* fTexts[TEXTS_COUNT];
	
	/// Przycisk do zwijania i rozwijania poziomow
	gARDexpander* fExpander;
	
	/// Okresla czy poziom jest rozwiniety (true) czy zwiniety (false)
	bool fExpanded;

public:

     /// Konstruktor
     gARDlevel(ARD_Level*);

     /// Destruktor
     ~gARDlevel(void);
	
	/// ----------------- Metody zwracajace wartosci pol klasy -----------------
	
	/// Zwraca kolejny identyfikator wezla i zwieksza stary
	unsigned int nextNodeId(bool _increase = true);
	
	/// Zwraca kolejny identyfikator zaleznosci i zwieksza stary
	unsigned int nextDependencyId(bool _increase = true);
	
	/// Zwraca czy poziom jest zwiniety czy rozwiniety
	bool expanded(void);
	
	/// Zwraca liste wezlow
	QList<gARDnode*>* nodes(void);
	
	/// Zwraca liste zaleznosci
	QList<gARDdependency*>* dependencies(void);
	
	/// Zwraca wskaznik do rodzica
	ARD_Level* parent(void);
	
	/// ----------------- Metody ustawiajace wartosci pol klasy -----------------
	
	/// Ustawia czy poziom jest zwiniety czy rozwiniety
	void setExpanded(bool);
	
	/// -------------------------- Metody operacyjne ---------------------------
	
	/// Oproznia zawartosc pojemnika
	void flush(void);
	
	/// Zwraca index wezla o podanym ID
	int indexOfNodeID(unsigned int);
	
	/// Zwraca index zaleznosci o parametrach takich samych jak wejsciowa
	int indexOfDependencyInOut(ARD_Dependency*);
	
	/// Zwraca index zaleznosci o podanych koncach
	int indexOfDependencyInOut(QString, QString);
	
	/// Zwraca index wezla ktory ma rodzica o podanym ID
	int indexOfParentID(QString);
	
	/// Zwraca liste zaleznosci o podanych koncach
	QList<gARDdependency*>* indexOfinout(QString, QString);
	
	/// Zwraca wskaznik do wezla ktory ma rodzica o podanym ID
	gARDnode* indexOfParentIDptr(QString);
	
	/// Wyszukuje wezly korzenie drzewa, diagramu
	QList<gARDnode*>* rootsNodes(void);
	
	/// Zwraca liste potomkow danego wezla
	QList<gARDnode*>* childsList(gARDnode* _parentNode, bool skipParentNode = true);
	
	/// Odnawia caly diagram
	void refresh(void);
	
	/// Usuwa polaczenia zwizane z dana wlasnoscia
	void removeDependencies(ARD_Property* _input = NULL);
	
	/// Usuwa polaczenia majace odpowiedniego rodzica
	void removeDependencies(ARD_Dependency*);
	
	/// Odswieza polaczenia
	void repaintDependencies(ARD_Property* _input = NULL);
	
	/// Ustawia w kazdym wezle wskaznik do rodzica
	void setParents(void);
	
	/// Sprawdza czy wejsciowe wlasnosci maja wspolnego rodzica
	int hasCommonParent(QList<gARDnode*>*, QList<gARDnode*>*);
	
	/// Ustawia zeleznosci pomiedzy odleglosciami miedzy wezlami a ich rodzicami
	void createTree(QList<gARDnode*>*);
	
	/// Ustawia zeleznosci pomiedzy odleglosciami miedzy wezlami a ich rodzicami
	void getTreeGeometry(int, QList<gARDnode*>*, QList<QList<gARDnode*>*>*, QList<QList<float>*>*, QList<float>*);
	
	/// Rozmieszcza elementy poziomu
	void placeObjects(void);
	
	/// Zwraca obszar prostokanty zamowany przez poziom
	QRectF levelArea(void);
	
	/// Przesuwa obszar na scenie o zadane wartosci
	void moveLevelBy(float, float);
};
//---------------------------------------------------------------------------
#endif
