 /**
 *                                                                             *
 * @file gARDnode.cpp                                                          *
 * Project name: HeKatE hqed                                                   *
 * $Id: gARDnode.cpp,v 1.23 2009-05-13 10:28:22 kinio Exp $                     *
 * Author: Krzysztof Kaczor                                                    *
 * Date: 2.01.2008                                                             *
 *                                                                             *
 * Implementation by Krzysztof Kaczor <kinio4444@gmail.com>                    *
 *                                                                             *
 * Copyright (C) 2006-9 by the HeKatE Project                                  *
 *                                                                             *
 * HQEd has been develped within the HeKatE Project,                           *
 * see http://hekate.ia.agh.edu.pl                                             *
 *                                                                             *
 * This file is part of HQEd.                                                  *
 *                                                                             *
 * HQEd is free software: you can redistribute it and/or modify                *
 * it under the terms of the GNU General Public License as published by        *
 * the Free Software Foundation, either version 3 of the License, or           *
 * (at your option) any later version.                                         *
 *                                                                             *
 * HQEd is distributed in the hope that it will be useful,                     *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU General Public License for more details.                                *
 *                                                                             *
 * You should have received a copy of the GNU General Public License           *
 * along with HQEd.  If not, see <http://www.gnu.org/licenses/>.               *
 *                                                                             *
 * Comment: Plik zawierajacy definicje metod klasy bedacej wezlem              *
 * w diagramie ARD                                                             *
 *                                                                             *
 */

#include <QMessageBox>
#include <QtGui>
#include <QDialog>
#include <QObject>

#include <QAction>
#include <QActionGroup>
#include <QLabel>
#include <QMenu> 
#include <QtGui>

#include <QGraphicsItem>

#include "../../C/Settings_ui.h"
#include "../../C/MainWin_ui.h"
#include "../../C/ARD/ardGraphicsScene.h"

#include "../../M/ARD/ARD_Property.h"
#include "../../M/ARD/ARD_PropertyList.h"
#include "../../M/ARD/ARD_Level.h"
#include "../../M/ARD/ARD_LevelList.h"
#include "../../M/ARD/ARD_Attribute.h"
#include "../../M/XTT/XTT_Attribute.h"

#include "gARDlevel.h"
#include "gARDlevelList.h"
#include "gARDdependency.h"
#include "gARDnode.h"
//---------------------------------------------------------------------------

#define HORIZONTALMARGIN 10.
#define VERTICALMARGIN 10.
//---------------------------------------------------------------------------

/**
 * Konstruktor klasy
 * @param ARD_Property* _parent - Wskanik do wezla rodzica
 * @returns Brak zwracanych wartosci
*/
gARDnode::gARDnode(unsigned int _id, ARD_Property* _parent, gARDlevel* _gParent) : QGraphicsItem(0)
{
	fID = _id;
	fParent = _parent;
	fGparent = _gParent;
     setFlag(ItemIsSelectable);
     setFlag(ItemIsFocusable);
     setFlag(ItemIsMovable);
     setZValue(1);
	
	fARDparents = new QList<gARDnode*>;
	setToolTip(_parent->id());
}
//---------------------------------------------------------------------------

/**
 * Destruktor klasy
 * @returns brak zwracanych wartosci
*/
gARDnode::~gARDnode(void)
{
	delete fARDparents;
}
//---------------------------------------------------------------------------

/**
 * Zwraca identyfikator wezla
 * @returns identyfikator wezla
*/
unsigned int gARDnode::id(void)
{
	return fID;
}
//---------------------------------------------------------------------------

/**
 * Zwraca liste rodzicow w diagramie ARD
 * @returns zwraca liste rodzicow w diagramie ARD
*/
QList<gARDnode*>* gARDnode::ardParents(void)
{
	return fARDparents;
}
//--------------------------------------------------------------------------

/**
 * Zwraca index rodzica o zadanym identyfikatorze
 * @param unsigned int _pid - identyfikaotr wezla rodzica
 * @returns index rodzica o zadanym identyfikatorze, lub -1 gdy rodzic o podanym identyfikatorze nie istnieje
*/
int gARDnode::indexOfParent(unsigned int _pid)
{
	for(int i=0;i<fARDparents->size();++i)
		if(fARDparents->at(i)->id() == _pid)
			return i;
		
	return -1;
}
//--------------------------------------------------------------------------

/**
 * Zwraca wskaznik do rodzica
 * @returns zwraca wskaznik do rodzica
*/
ARD_Property* gARDnode::parent(void)
{
	return fParent;
}
//--------------------------------------------------------------------------

/**
 * Zwraca wskaznik do rodzica graficznego
 * @returns zwraca wskaznik do rodzica graficznego
*/
gARDlevel* gARDnode::gParent(void)
{
	return fGparent;
}
//--------------------------------------------------------------------------

/**
 * Metoda klasy GRect rysujaca kwadrat na ekranie
 * @param QPainter* _p - Wskanik do plutna rysowania
 * @param const QStyleOptionGraphicsItem* - Wskanik do stylu rysowania
 * @param QWidget* widget - Wskanik nieuzywany
 * @returns Brak zwracanych wartoci
*/
void gARDnode::paint(QPainter* _p, const QStyleOptionGraphicsItem*, QWidget*)
{
	//_p->drawRect(boundingRect());
	
     // Rysowanie kwadratu
     _p->setBrush(Settings->ardPropertyColor());
	if(!fParent->canBeTransformated())
		_p->setBrush(Settings->ardTPHnotTransformatedNodeColor());
     if(isSelected())
          _p->setBrush(Settings->ardSelectedPropertyColor());
     if(MainWin->ModelBW())
          _p->setBrush(QColor(255, 255, 255, 255));
		
	// Wyrysowanie ramki
	_p->setPen(Qt::NoPen);
	if(Settings->useFramedNodes())
	{
		QPen ppen;
		ppen.setWidth(1);
		ppen.setColor(Qt::black);
		ppen.setStyle(Qt::SolidLine);
		if(!fParent->canBeTransformated())
			ppen.setStyle(Qt::DashLine);
			
		_p->setPen(ppen);
	}
		
	// Narysowanie prostokata
	QRectF rect = fieldArea();
     _p->drawRect(rect);
	
	// Wypisanie nazw atrybutow
	QFont nodeFont = Settings->ardARDdiagramFont();
	QFontMetrics fontM(nodeFont);
	
	_p->setPen(Qt::SolidLine);
	_p->setPen(Settings->ardPropertyFontColor());
	_p->setFont(nodeFont);
	int fontHeight = fontM.height();
	float leftMargin = HORIZONTALMARGIN/2.;
	float topMargin = VERTICALMARGIN/2.;
	
	for(int i=0;i<fParent->size();++i)
		_p->drawText(QPointF(leftMargin, topMargin+(float)((i+1)*fontHeight)), fParent->at(i)->name());
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy gARDnode wywolywana w momencie pojedynczego klikniecia na obiekcie, prostokacie
 * @param QGraphicsSceneMouseEvent* _e - Wskanik do informacji o zdarzeniu
 * @returns Brak zwracanych wartoci
*/
void gARDnode::mousePressEvent(QGraphicsSceneMouseEvent* _e)
{
	QList<QGraphicsItem*> items = scene()->items();
	for(int i=0;i<items.size();++i)
		items.at(i)->setSelected(true);
		
	scene()->clearSelection();
	setSelected(true);
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy gARDnode wywolywana w momencie zwolnienia przycisku myszki nad obiektem
 * @param QGraphicsSceneMouseEvent* _e - Wskanik do informacji o zdarzeniu
 * @returns Brak zwracanych wartoci
*/
void gARDnode::mouseReleaseEvent(QGraphicsSceneMouseEvent* _e)
{
	// Zaznaczenie zaleznosci
	if((Settings->ardNodeSelectPolicy() == SELECT_CONNECTIONS) 
	|| 
	((_e->modifiers() == Qt::ControlModifier) && (Settings->ardNodeSelectPolicy() == SELECT_CONNECTIONS_WITH_CTRL)))
	{
		// Find all dependencies relates with selected node
		
		// If not defined graphical level parent
		if(fGparent == NULL)
			return;
			
		QList<gARDdependency*>* IN_connlist = fGparent->indexOfinout(fParent->id(), "");
		QList<gARDdependency*>* OUT_connlist = fGparent->indexOfinout("", fParent->id());
		
		for(int i=0;i<IN_connlist->size();++i)
			IN_connlist->at(i)->selectAll();
		for(int i=0;i<OUT_connlist->size();++i)
			OUT_connlist->at(i)->selectAll();
		
		delete IN_connlist;
		delete OUT_connlist;
	}
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy GRect wywolywana w momencie dwukrotnego klikniecia na obiekcie, 
 * prostokacie
 * @param QGraphicsSceneMouseEvent* _e - Wskanik do informacji o zdarzeniu
 * @returns Brak zwracanych wartoci
*/
void gARDnode::mouseDoubleClickEvent(QGraphicsSceneMouseEvent* _e)
{
	NULL;
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy zwracajaca obszar prostokatny w ktorym sie znajduje obiekt
 * @returns Obszar prostokatny zawierajacy obiekt
*/
QRectF gARDnode::boundingRect() const
{
	return fieldArea();
}
//---------------------------------------------------------------------------

/**
 * Metoda wywolywana w momencie zmiany polozenia obiektu
 * @param GraphicsItemChange change - Zawiera wartosc elementu ktora ulegla zmianie
 * @param const QVariant &value - Referencja do wartosci zmienionych
 * @returns Wartosc ktora ulela zmianie
*/
QVariant gARDnode::itemChange(GraphicsItemChange change, const QVariant &value)
{     
     if((change == ItemPositionChange) && (scene()))
     {
		fGparent->repaintDependencies(fParent);
     }
	
	if((change == ItemSelectedChange) && (scene()))
     {		
		fGparent->repaintDependencies(fParent);
     }
	
     return QGraphicsItem::itemChange(change, value);
}
//---------------------------------------------------------------------------

/**
 * Metoda obliczajaca wielksoc pola w ktorym sa wypisywane atrybuty
 * @returns obszar prostoktny w ktorym sa wypisywane atrybuty
*/
QRectF gARDnode::fieldArea(void) const
{
	// Ustalanie rozmiarow wezla
	float x = 0.;
	float y = 0.;
	float width = 0.;
	float height = 0.;
	
	QFont nodeFont = Settings->ardARDdiagramFont();
	QFontMetrics fontM(nodeFont);
	
	for(int i=0;i<fParent->size();++i)
	{
		int currStrWidth = fontM.width(fParent->at(i)->name());
		height += fontM.height();
		
		if(currStrWidth > width)
			width = currStrWidth;
	}
	
	width += HORIZONTALMARGIN;
	height += VERTICALMARGIN;
	
	return QRectF(x, y, width, height);
}
//---------------------------------------------------------------------------
