/*
 *	   $Id: GText.cpp,v 1.29 2009-05-13 10:28:22 kinio Exp $
 *	
 *     Implementation by Krzysztof Kaczor <kinio4444@gmail.com>
 *
 *     Copyright (C) 2006-9 by the HeKatE Project
 *
 *     HQEd has been develped within the HeKatE Project,
 *     see http://hekate.ia.agh.edu.pl
 *
 *     This file is part of HQEd.
 *
 *     HQEd is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     HQEd is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with HQEd.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


#include <QMessageBox>
#include <QtGui>
#include <QDialog>
#include <QObject>
#include <QFont>
#include <QFontMetrics>

#include "../../M/hMultipleValue.h"

#include "../../M/XTT/XTT_Cell.h"
#include "../../M/XTT/XTT_Table.h"
#include "../../M/XTT/XTT_TableList.h"
#include "../../M/XTT/XTT_Attribute.h"
#include "../../M/XTT/XTT_AttributeGroups.h"
#include "../../C/XTT/MyGraphicsScene.h"
#include "../../C/XTT/AttributeEditor_ui.h"
#include "../../C/XTT/ExpressionEditor_ui.h"
#include "../../C/Settings_ui.h"
#include "../../C/MainWin_ui.h"
#include "../../namespaces/ns_xtt.h"

#include "GTable.h"
#include "GText.h"
//---------------------------------------------------------------------------

// #brief Constructor GText class.
//
// #param _parent Pointer to the parent of the table.
// #param _tt Index of the table.
// #param _row Index of the row of the cell.
// #param _col Index of the column of the cell.
GText::GText(XTT_Table* _parent, unsigned int _tt, unsigned int _row, unsigned int _col) : QObject(0), QGraphicsItem(0)
{
     setZValue(3);
     fParent = _parent;
     fTextType = _tt;
     fRow = _row;
     fCol = _col;
     exprEditor = NULL;
	isSelected = false;
	fIsHeaderSelected = false;
	setAcceptsHoverEvents(true);
}
//---------------------------------------------------------------------------

// #brief Destructor of GText class.
GText::~GText(void)
{
     if(exprEditor != NULL)
          delete exprEditor;
}
//---------------------------------------------------------------------------

// #brief Function is called mouse is hanging over the cell.
//
// #param event Pointer to event object.
// #return No return values.
void GText::hoverEnterEvent(QGraphicsSceneHoverEvent* event)
{
	if(scene() == NULL)
		return;

	isSelected = true;
	if((Settings->xttEnabledAutocentering()) && (scene()->mouseGrabberItem() != this))
		ensureVisible(boundingRect(), 0, 0);
	QGraphicsItem::hoverEnterEvent(event);
}
//---------------------------------------------------------------------------

// #brief Function is called when mouse is leavin the cell.
//
// #param event Pointer to event object.
// #return No return values.
void GText::hoverLeaveEvent(QGraphicsSceneHoverEvent* event)
{
	if(fTextType == 0)
		fParent->Row(fRow)->Cell(fCol)->setSelected(false);

	isSelected = false;
	fIsHeaderSelected = false;
	QGraphicsItem::hoverLeaveEvent(event);
}
//---------------------------------------------------------------------------

// #brief Allow for select header cell
//
// #param bool _selStat - selection status
// #return No return value
void GText::selectHeaderCell(bool _selStat)
{
	fIsHeaderSelected = _selStat;
	if(_selStat)
		ensureVisible(boundingRect(), 0, 0);
	update();
}
//---------------------------------------------------------------------------

// #brief Return status of header selection
//
// #return True is header is selected, otherwise false
bool GText::isHeaderSelected(void)
{
	return fIsHeaderSelected;
}
//---------------------------------------------------------------------------

// #brief Function retrieves rectangular in which object is located.
//
// #return Rectangular in which object is located.
QRectF GText::boundingRect() const
{
	// Ustalanie rozmiarow tekstu
	int tWidth = 0;
	int tHeight = 0;
	
	// Jezeli zawartosc komorki
	if(fTextType == 0)
	{
		tWidth = fParent->ColWidth(fCol);
		tHeight = fParent->RowHeight(fRow);
	}
	
	// Jezeli tytul tabeli
	if(fTextType == 1)
	{
		// Sprawdzanie wymiarow czcionki
		QFontMetrics fm(Settings->xttTableTitleFont());
		tWidth = fm.width("Table id: " + fParent->TableId() + " - " + fParent->Title());
		tHeight = fm.height();
	}
	
	// Jezeli tresc naglowka
	if(fTextType == 2)
	{
		tWidth = fParent->ColWidth(fCol);
		tHeight = fParent->HeaderHeight();
	}
	
     return QRectF(-5, 0, tWidth, tHeight);
}
//---------------------------------------------------------------------------

// #brief Function draws square on the screen.
//
// #param _p Pointer to the canvas.
// #param _sogi Pointer to the drawing style.
// #param widget Unused pointer.
// #return No return value.
void GText::paint(QPainter* _p, const QStyleOptionGraphicsItem*, QWidget*)
{
     QString text = "";
     QRect rect;
     int flags = Qt::AlignHCenter | Qt::AlignVCenter | Qt::TextWordWrap;
	const int horizontal_text_margin = 10;
	const int vertical_text_margin = 4;
     
     if(fTextType == 0)
     {
          if((fRow >= fParent->RowCount()) || (fCol >= (unsigned int)fParent->ColCount()))
               return;
          text = fParent->Row(fRow)->Cell(fCol)->Content()->toString();
		setToolTip("");
		if(Settings->xttShowErrorInToolTip())
			setToolTip(fParent->Row(fRow)->Cell(fCol)->CellError(false));
		_p->setFont(Settings->xttTableFont());
		_p->setPen(Settings->xttTableFontColor());
		
		// Sprawdzanie wymiarow czcionki
		QFontMetrics fm(Settings->xttTableFont());
		int text_widht = fParent->ColWidth(fCol);
		int text_height = fParent->RowHeight(fRow);
		
		// Ewentualne powiekszenie rozmiarow
		if(Settings->xttCellAutosize())
		{
			if(text_widht < (fm.width(text)+horizontal_text_margin))
			{
				text_widht = fm.width(text) + horizontal_text_margin;
				fParent->setColWidth(fCol, text_widht);
				fParent->setLocalization(fParent->Localization());
			}
			if(text_height < (fm.height()+vertical_text_margin))
			{
				text_height = fm.height() + vertical_text_margin;
				fParent->setRowHeight(fRow, text_height);
				fParent->setLocalization(fParent->Localization());
			}
		}
		
		rect.setRect(0, 0, text_widht, text_height);
     }
     else if(fTextType == 1)
     {
          text = "Table id: " + fParent->TableId() + " - " + fParent->Title();
          
          flags = Qt::AlignLeft | Qt::AlignVCenter | Qt::TextWordWrap;
		_p->setFont(Settings->xttTableTitleFont());
		_p->setPen(Settings->xtTableTitleFontColor());
		
		// Sprawdzanie wymiarow czcionki
		QFontMetrics fm(Settings->xttTableTitleFont());
		int text_widht = fm.width(text);
		int text_height = fm.height();
		
		rect.setRect(0, 0, text_widht, text_height);
     }
     else if(fTextType == 2)
     {
          int context = fParent->ColContext(fCol);
          
          QString aname = "";
          if(context != 5)
               aname = AttributeGroups->CreatePath(fParent->ContextAtribute(fCol)) + fParent->ContextAtribute(fCol)->Name();
          if(aname == "#NAN")
          {
               QMessageBox::critical(NULL, "HQEd", "Missing pointer.", QMessageBox::Ok);
               return;
          }
          // Ustalanie contekstu i dodanie znacznika kontekstu
          QStringList ctx_smbl;
          ctx_smbl << "(?) " << "(+) " << "(-) " << "(->) " << "*";
          aname = ctx_smbl.at(context-1) + aname;
          text = aname;
          
		_p->setFont(Settings->xttTableHeaderFont());
		_p->setPen(Settings->xttTableHeaderFontColor());
		
		// Sprawdzanie wymiarow czcionki
		QFontMetrics fm(Settings->xttTableHeaderFont());
		int text_widht = fParent->ColWidth(fCol);
		int text_height = fParent->HeaderHeight();
		
		// Ewentualne powiekszenie rozmiarow
		if(Settings->xttCellAutosize())
		{
			if(text_widht < (fm.width(text)+horizontal_text_margin))
			{
				text_widht = fm.width(text) + horizontal_text_margin;
				fParent->setColWidth(fCol, text_widht);
				fParent->setLocalization(fParent->Localization());
			}
			if(text_height < (fm.height()+vertical_text_margin))
			{
				text_height = fm.height() + vertical_text_margin;
				fParent->setHeaderHeight(text_height);
				fParent->setLocalization(fParent->Localization());
			}
		}
		
		rect.setRect(0, 0, text_widht, text_height);
     }
	
	// Jezeli niepoprawna skladnia to podswietlamy komorke
	if((fTextType == 0) && (!fParent->Row(fRow)->Cell(fCol)->isCorrectSyntax()) && (Settings->xttHighlightErrorCell()))
		_p->fillRect(rect, QBrush(Settings->xttErrorCellColor(), MainWin->ModelBW()?Qt::Dense6Pattern:Qt::SolidPattern));
		
	// Jezeli komorka jest zaznaczona, lub jezeli jest to naglowek i jest zaznaczony
	if(((fTextType == 0) && ((isSelected) || (fParent->Row(fRow)->Cell(fCol)->isSelected()))) || ((fTextType == 2) && (fIsHeaderSelected)))
		_p->fillRect(rect, QBrush(Settings->xttHoveredCellColor(), MainWin->ModelBW()?Qt::Dense7Pattern:Qt::SolidPattern));
		
     _p->drawText(rect, flags, text);
}
//---------------------------------------------------------------------------

// Metoda tworzaca polenia menu
// #brief Function creates menu actions.
//
// #return No return value.
void GText::createActions(void)
{
	aEidtTable = new QAction(QIcon(":/all_images/images/Edit1.png"), QObject::tr("Edit table..."), NULL);
     aEidtTable->setStatusTip(QObject::tr("Allows edit this table"));
     QObject::connect(aEidtTable, SIGNAL(triggered()), MainWin, SLOT(TableEditorEdit()));
	
	aCellEditor = new QAction(QIcon(":/all_images/images/Edit2.png"), QObject::tr("Cell editor..."), NULL);
     aCellEditor->setStatusTip(QObject::tr("Show cell editor"));
     QObject::connect(aCellEditor, SIGNAL(triggered()), MainWin, SLOT(ShowCellEditor()));
	
	aEditCell = new QAction(QIcon(":/all_images/images/Edit1.png"), QObject::tr("Edit cell..."), NULL);
     aEditCell->setStatusTip(QObject::tr("Allows edit this cell content"));
     QObject::connect(aEditCell, SIGNAL(triggered()), this, SLOT(editCell()));
	
	aDeleteTable = new QAction(QIcon(":/all_images/images/DelTabele.png"), QObject::tr("Delete table"), NULL);
     aDeleteTable->setStatusTip(QObject::tr("Allows remove this table"));
     QObject::connect(aDeleteTable, SIGNAL(triggered()), MainWin, SLOT(deleteSelectedTable()));
	
	aInsertRowBefore = new QAction(QIcon(":/all_images/images/AddRowBefore.png"), QObject::tr("Insert row before"), NULL);
     aInsertRowBefore->setStatusTip(QObject::tr("Allows add new row before this row"));
     QObject::connect(aInsertRowBefore, SIGNAL(triggered()), this, SLOT(insertRowBefore()));
	
	aInsertRowAfter = new QAction(QIcon(":/all_images/images/AddRowAfter.png"), QObject::tr("Insert row after"), NULL);
     aInsertRowAfter->setStatusTip(QObject::tr("Allows add new row after this row"));
     QObject::connect(aInsertRowAfter, SIGNAL(triggered()), this, SLOT(insertRowAfter()));
	
	aMoveRowUp = new QAction(QIcon(":/all_images/images/RowMoveUp.png"), QObject::tr("Move row up"), NULL);
     aMoveRowUp->setStatusTip(QObject::tr("Allows move this row one position up"));
     QObject::connect(aMoveRowUp, SIGNAL(triggered()), this, SLOT(moveRowUp()));
	
	aMoveRowDown = new QAction(QIcon(":/all_images/images/RowMoveDown.png"), QObject::tr("Move row down"), NULL);
     aMoveRowDown->setStatusTip(QObject::tr("Allows move this row one position down"));
     QObject::connect(aMoveRowDown, SIGNAL(triggered()), this, SLOT(moveRowDown()));
	
	aDeleteRow = new QAction(QIcon(":/all_images/images/DelRow.png"), QObject::tr("Delete row"), NULL);
     aDeleteRow->setStatusTip(QObject::tr("Allows remove this row from this table"));
     QObject::connect(aDeleteRow, SIGNAL(triggered()), this, SLOT(deleteRow()));
	
	aDeleteCol = new QAction(QIcon(":/all_images/images/DelCol.png"), QObject::tr("Delete col"), NULL);
     aDeleteCol->setStatusTip(QObject::tr("Allows remove this column from this table"));
     QObject::connect(aDeleteCol, SIGNAL(triggered()), this, SLOT(deleteCol()));
	
	aAttributeEditor = new QAction(QIcon(":/all_images/images/Edit1.png"), QObject::tr("Edit attribute"), NULL);
     aAttributeEditor->setStatusTip(QObject::tr("Allows to edit attribute related with this column"));
     QObject::connect(aAttributeEditor, SIGNAL(triggered()), this, SLOT(editAttribute()));
	
	if(fTextType == 0)
	{
		QFont boldFont = aEditCell->font();
		boldFont.setBold(true);
		aEditCell->setFont(boldFont);
	}
	
	if(fTextType == 2)
	{
		QFont boldFont = aEidtTable->font();
		boldFont.setBold(true);
		aEidtTable->setFont(boldFont);
	}
}
//---------------------------------------------------------------------------

// #brief Function is called when mouse right button is pressed.
//
// #param event Pointer to event object.
// #return No return values.
void GText::contextMenuEvent(QGraphicsSceneContextMenuEvent* contextMenuEvent)
{
	if(fTextType == 1)
		return;
		
     int context = fParent->ColContext(fCol);
	createActions();
	
	QMenu mainMenu(NULL);
	
	if(fTextType != 2)
		mainMenu.addAction(aEditCell);
	mainMenu.addAction(aEidtTable);
	mainMenu.addAction(aCellEditor);
	
	mainMenu.addSeparator();
	if(fTextType != 2)
	{
		mainMenu.addAction(aInsertRowBefore);
		mainMenu.addAction(aInsertRowAfter);
		
		mainMenu.addSeparator();
		mainMenu.addAction(aMoveRowUp);
		mainMenu.addAction(aMoveRowDown);
		
		mainMenu.addSeparator();
		mainMenu.addAction(aDeleteRow);
	}
	mainMenu.addAction(aDeleteCol);
	
	mainMenu.addSeparator();
	mainMenu.addAction(aDeleteTable);
	mainMenu.addSeparator();
     if(context != 5)
          mainMenu.addAction(aAttributeEditor);
	
	mainMenu.exec(contextMenuEvent->screenPos());
	
	delete aEidtTable;
	delete aCellEditor;
	delete aEditCell;
	delete aDeleteTable;
	delete aInsertRowBefore;
	delete aInsertRowAfter;
	delete aDeleteRow;
	delete aDeleteCol;
	delete aMoveRowDown;
	delete aMoveRowUp;
}
//---------------------------------------------------------------------------

// #brief Function shows cell contents window.
//
// #return No return value.
void GText::editCell(void)
{
	if(((int)fCol) >= 0)
	{
		//delete Hinter;
		
	     // Parametry uruchomienia okienka
	     //XTT_Attribute* attr = fParent->ContextAtribute(fCol);
	     XTT_Cell* cell = NULL;
	     cell = fParent->Row(fRow)->Cell(fCol);
          
          exprEditor = new ExpressionEditor_ui(NULL);
          if(!exprEditor->setMode(cell))
          {
               QMessageBox::critical(NULL, "HQEd", "Incorrect expression editor mode.", QMessageBox::Ok);
               return;
          }
          exprEditor->show();
	}
}
//---------------------------------------------------------------------------

// #brief Function called when cell editor is closed using the OK button
//
// #return No return value.
void GText::cellEditorConfirm(void)
{
     xtt::fullVerification();
}
//---------------------------------------------------------------------------

// #brief Function called on cell editor close event
//
// #return No return value.
void GText::cellEditorClose(void)
{
     delete exprEditor;
     exprEditor = NULL;
}
//---------------------------------------------------------------------------

// #brief Function moves one row up.
//
// #return No return value.
void GText::moveRowUp(void)
{
	fParent->ExchangeRows(fRow, fRow-1);
	scene()->update();
}
//---------------------------------------------------------------------------

// #brief Function moves one row down.
//
// #return No return value.
void GText::moveRowDown(void)
{
	fParent->ExchangeRows(fRow, fRow+1);
	scene()->update();
}
//---------------------------------------------------------------------------

// #brief Function inserts one row after another.
//
// #return No return value.
void GText::insertRowAfter(void)
{
	fParent->insertRow(fRow+1);
}
//---------------------------------------------------------------------------

// #brief Function inserts one row before another.
//
// #return No return value.
void GText::insertRowBefore(void)
{
	fParent->insertRow(fRow);
}
//---------------------------------------------------------------------------

// #brief Function removes current row.
//
// #return No return value.
void GText::deleteRow(void)
{
	if(QMessageBox::question(NULL, "HQEd", "Are you really want to remove selected row?", QMessageBox::Yes | QMessageBox::No) == QMessageBox::No)
		return;
	
	fParent->DeleteRow(fRow);
}
//---------------------------------------------------------------------------

// #brief Function removes current column.
//
// #return No return value.
void GText::deleteCol(void)
{
	if(QMessageBox::question(NULL, "HQEd", "Are you really want to remove selected column?", QMessageBox::Yes | QMessageBox::No) == QMessageBox::No)
		return;
		
	fParent->DeleteCol(fCol);
	if(fParent->ColCount() == 0)
		TableList->Delete(TableList->IndexOfID(fParent->TableId()));
}
//---------------------------------------------------------------------------

// #brief Function shows edit attribute window.
//
// #return No return value.
void GText::editAttribute(void)
{
	if(AttributeGroups->IsEmpty())
     {
          const QString str2 = "There is no attributes defined yet.";
          QMessageBox::information(NULL, "HQEd", str2, QMessageBox::Ok);
          return;
     }

     delete AttributeEditor;
     AttributeEditor = new AttrEditor(fParent->Row(fRow)->Cell(fCol)->AttrType()->Id(), false);
     AttributeEditor->show();
}
//---------------------------------------------------------------------------
