/*
 *	   $Id: vasProlog.cpp,v 1.4 2009-05-13 10:28:21 kinio Exp $
 *	
 *     Implementation by Krzysztof Kaczor <kinio4444@gmail.com>
 *
 *     Copyright (C) 2006-9 by the HeKatE Project
 *
 *     HQEd has been develped within the HeKatE Project,
 *     see http://hekate.ia.agh.edu.pl
 *
 *     This file is part of HQEd.
 *
 *     HQEd is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     HQEd is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with HQEd.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <QMessageBox>
#include <QtGui>
#include <QDialog>

#include <QKeyEvent>
#include <QStringList>
#include <QString>
#include <QList>

#include "../namespaces/ns_xtt.h"

#include "vasProlog.h"
// -----------------------------------------------------------------------------

// #brief Exports the given model to SWI Prolog language format
//
// #param __msg - reference to the variable where the result (info, errors) messages are stored
// #param __device - the output device (File, Socket, Process, ...)
// #param __prefix - this argument contains string that is addes before generated code
// #param __postfix - this argument contains string that is addes after generated code
// #return exit code:
// #li VAS_SWI_PROLOG_SUCCES - on succes
// #li VAS_SWI_PROLOG_UNKNOWN_DEVICE - when device is not specified
// #li VAS_SWI_PROLOG_DEVICE_NOT_WRITABLE - when device is not writable
// #li VAS_SWI_PROLOG_ERROR - when during the generation error occurs
int vas_out_SWI_Prolog(QIODevice* __device, QString& __msg, QString __prefix, QString __postfix)
{
     if(__device == NULL)
          return VAS_SWI_PROLOG_UNKNOWN_DEVICE;
     if(!__device->isWritable())
          return VAS_SWI_PROLOG_DEVICE_NOT_WRITABLE;
     
     bool __succes;                                         // the result of generation
     QString content = xtt::generateProlog(__succes);       // code generating
     
     // When error occurs
     if(!__succes)
     {
          __msg = content;
          return VAS_SWI_PROLOG_ERROR;
     }
     
     QTextStream out(__device);
     out << __prefix;
     out << content;
     out << __postfix;
           
     return VAS_SWI_PROLOG_SUCCES;
}
// -----------------------------------------------------------------------------
